/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.account;

import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.service.AccountService;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.general.system.common.model.exception.BaseCommonException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 账户处理
 * @author hanfei
 * @version $Id: AccountServiceImpl.java, v 0.1 2019-12-27 12:42 AM hanfei Exp $ 
 */
@Service
public class AccountServiceImpl implements AccountService {
    
    @Autowired
    private AcctManager acctManager;
    @Autowired
    private PlanRepo planRepo;
    
    /**
     * 查询存续中计划所有账户余额
     * @return
     */
    @Override
    public List<Account> queryBalance(){
        // 1. 查询存续中的所有计划
        List<PlanBase> planBases = planRepo.getPlansByStatus(null, PlanStatusEnum.ONGOING);
        List<Account> accountList = new ArrayList<>();
        for(PlanBase planBase : planBases){
            int maxCnt = 3;
            List<Account> accounts = null;
            while (accounts == null && maxCnt > 0){
                try {
                    // 查询计划对应虚拟户账户余额
                    accounts = acctManager.getAcctByPlanNoAndType(planBase.getPlanNo(),
                            AccountTypeEnum.PLAT_VIRTUAL,true);
                    accountList.addAll(accounts);
                    maxCnt = 0;
                    Thread.sleep(2000);
                } catch (BaseCommonException e){
                    maxCnt --;
                } catch (InterruptedException e) {
                    throw new AbssqrBizException("线程等待异常");
                }
            }
        }
        return accountList;
    }
}
