/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.json.JSONUtil;
import com.abssqr.plat.biz.api.analyse.node.AnalyseChartContext;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.AnalyseServiceEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.param.analyse.AnalyseBaseParam;
import com.abssqr.plat.common.facade.param.analyse.AnalyseTrendChartParam;
import com.abssqr.plat.common.facade.result.analyse.AnalyseChartResultVO;
import com.abssqr.plat.common.facade.result.analyse.AnalyseDayAmtInfoVO;
import com.abssqr.plat.common.facade.result.analyse.AnalyseDayAstInfoVO;
import com.abssqr.plat.common.facade.result.analyse.FormationAstItemVO;
import com.abssqr.plat.common.facade.service.analyse.AnalyseInfoChartService;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.report.AnalyseSpvReportManager;
import com.abssqr.plat.core.service.util.AnalyseReportUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 统计页面相关展示数据
 * @author hanfei
 * @version $Id: AnalyseInfoChartServiceImpl.java, v 0.1 2019-09-24 5:13 PM hanfei Exp $ 
 */
@Service
public class AnalyseInfoChartServiceImpl extends ServiceSupport implements AnalyseInfoChartService {
    
    @Autowired
    private AnalyseSpvReportManager      analyseSpvReportManager;
    @Autowired
    private PlanReportRepo               planReportRepo;
    @Autowired
    private AnalyseChartMultipleSelector analyseChartMultipleSelector;
    @Autowired
    private PlanAllocRepo                planAllocRepo;
    @Autowired
    private OrgAcctDayComponent          orgAcctDayComponent;
    
    /**
     * 资产情况概览
     * @param param
     * @return
     */
    @Override
    public AnalyseDayAstInfoVO getDayAstInfo(AnalyseBaseParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            String planNo = param.getPlanNo();
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 查询计划
            analyseSpvReportManager.checkGetPlan(planNo);
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(param.getPlanNo(), ReportTypeEnum.RPT001);
            Date accDate = AnalyseReportUtil.getStatDate(trfDate, param.getAccountDate(), planReport);
            // 统计上一日变化
            Date lastDate = DateUtil.offsetDay(accDate, -1);

            List<ReportField> statFields = Lists.newArrayList(
                    // 总资产
                    WkAstReportFields.WK017,
                    WkAstReportFields.WK018,
                    WkAstReportFields.WK019
            );
            // 查询计划对应统计项数据
            Map<String, PlanReportItem> itemMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(accDate, planNo, statFields, true);

            // 查询上一日计划对应统计项数据
            Map<String, PlanReportItem> itemLastMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(lastDate, planNo, statFields, true);

            AnalyseDayAstInfoVO infoVO = new AnalyseDayAstInfoVO();
            infoVO.setPlanNo(planNo);
            List<ReportField> astScaleFields =
                    CollectionUtil.newArrayList(WkAstReportFields.WK017, WkAstReportFields.WK018, WkAstReportFields.WK019);
            // 总资产规模
            infoVO.setAstScale(AnalyseReportUtil.getItemMoney(itemMap, astScaleFields));
            // 与上一日变化
            infoVO.setAstScaleChange(infoVO.getAstScale().subtract(AnalyseReportUtil.getItemMoney(itemLastMap, astScaleFields)));

            List<ReportField> spvFields = Lists.newArrayList(
                    // 转让资产规模
                    WkAstReportFields.WK028,
                    // 累计转让资产规模
                    WkAstReportFields.WK027,
                    // M3+ 逾期率
                    WkAstReportFields.WK026);
            Map<String, PlanReportItem> itemSpvMap =
                                analyseSpvReportManager.querySpvStatPlanDayItem(accDate, planNo, spvFields, false);
            // 当日交易贷款
            infoVO.setAstTransfer(AnalyseReportUtil.getItemMoney(itemSpvMap, WkAstReportFields.WK028));
            // 累计交易贷款金额
            infoVO.setAstTransferTotal(AnalyseReportUtil.getItemMoney(itemSpvMap, WkAstReportFields.WK027));
    
            List<ReportField> ovdFields = Lists.newArrayList(
                    // 总贷款本金余额
                    WkAstReportFields.WK017,
                    // M3+ 逾期本金余额
                    WkAstReportFields.WK025);
            Map<String, PlanReportItem> ovdSpvMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(accDate, planNo, ovdFields, true);
            // M3+逾期率 获取放款池加承接池的 总贷款本金余额、逾期本金余额
            infoVO.setOvdM3Rate(CalcUtil.dividePercRate(
                    AnalyseReportUtil.getItemBig(ovdSpvMap, WkAstReportFields.WK025),
                    AnalyseReportUtil.getItemBig(ovdSpvMap, WkAstReportFields.WK017)));

            // 统计月初至今的还款发生额
            List<ReportField> repayStat = CollectionUtil.newArrayList(
                    WkAstReportFields.WK104, WkAstReportFields.WK105, WkAstReportFields.WK106, WkAstReportFields.WK107);
            Map<String, PlanReportItem> itemMonthMap =
                    analyseSpvReportManager.querySpvStatPlanSectionItem(
                            DateUtil.beginOfMonth(accDate), accDate, planNo, repayStat, true);
            // 已还还款计划 =  应还已还本金 + 应还已还利息
            infoVO.setRepaidTotal(AnalyseReportUtil.getItemMoney(itemMonthMap,
                    WkAstReportFields.WK106, WkAstReportFields.WK107));
            // 还款计划总额 =  应还本金  + 应还利息
            infoVO.setRepayPlanTotal(AnalyseReportUtil.getItemMoney(itemMonthMap,
                    WkAstReportFields.WK104, WkAstReportFields.WK105));

            final Date statDate =  DateUtil.offsetDay(accDate, AbssqrConstants.AVG_SPARE);
            // 获取逾期率
            BigDecimal ovdRate = analyseSpvReportManager.getOvdRate(
                    statDate, accDate, planNo);
            // 获取上一日逾期率
            BigDecimal ovdLastRate = analyseSpvReportManager.getOvdRate(
                    statDate, lastDate,planNo);
            infoVO.setOvdRate(ovdRate);
            infoVO.setOvdLastRate(ovdLastRate);

            // 获取回款率
            BigDecimal repayRate = analyseSpvReportManager.getRepayRate(
                    statDate, accDate, planNo);
            // 获取上一日回款率
            BigDecimal repayLastRate = analyseSpvReportManager.getRepayRate(
                    statDate, lastDate, planNo);
            infoVO.setRepayAvgRate(repayRate);
            infoVO.setRepayLastRate(repayLastRate);
    
            // 获取早尝率
            BigDecimal preRepayRate = analyseSpvReportManager.getPreRepayRate(
                    statDate, accDate, planNo);
            // 获取上一日早尝率
            BigDecimal preRepayLastRate = analyseSpvReportManager.getPreRepayRate(
                    statDate, lastDate, planNo);
            infoVO.setPreRepayRate(preRepayRate);
            infoVO.setPreRepayLastRate(preRepayLastRate);
            return infoVO;
        });
    }

    /**
     * 资金情况概览
     * @param param
     * @return
     */
    @Override
    public AnalyseDayAmtInfoVO getDayAmtInfo(AnalyseBaseParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            String planNo = param.getPlanNo();
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 查询计划
            PlanBase planBase = analyseSpvReportManager.checkGetFullPlan(planNo);
            // 查询对应受让池计划
            List<PlanBase> planBases = analyseSpvReportManager.getPlanBySpv(planBase.getPlanNo());
            // 获取放款池、承接池信托计划规模
            Money denominator = AnalyseReportUtil.getPlanTotalAmtBySpvPlanNo(planBase, planBases);
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(planBase.getPlanNo(), ReportTypeEnum.RPT001);
            // 统计日
            Date accDate = AnalyseReportUtil.getStatDate(trfDate, param.getAccountDate(), planReport);
            // 上一日变化
            Date lastDate = DateUtil.offsetDay(accDate, -1);
            List<ReportField> statFields = Lists.newArrayList(
                    // 余额
                    WkAstReportFields.WK001,
                    // 总贷款本金余额
                    WkAstReportFields.WK017,

                    // 流出 = 兑付流出 + 金融服务费 + 新增贷款投放
                    WkAstReportFields.WK005,
                    WkAstReportFields.WK108,
                    WkAstReportFields.WK111,

                    // 累计流出 = 累计金融服务费 + 累计新增贷款 + 累计分配金额
                    // 累计金融服务费
                    WkAstReportFields.WK109,
                    // 累计新增贷款
                    WkAstReportFields.WK116,

                    // 流入 = 当日还款 + 其他流入
                    WkAstReportFields.WK004,
                    WkAstReportFields.WK007
            );
            // 查询计划对应统计项数据
            Map<String, PlanReportItem> itemMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(accDate, planNo, statFields, true);
            // 查询上一日计划对应统计项数据
            Map<String, PlanReportItem> itemLastMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(lastDate, planNo, statFields, true);

            AnalyseDayAmtInfoVO infoVO = new AnalyseDayAmtInfoVO();
            infoVO.setPlanNo(planNo);
            // 资金余额
            infoVO.setFundBal(AnalyseReportUtil.getItemMoney(itemMap, WkAstReportFields.WK001));
            // 资金变化
            infoVO.setFundChange(infoVO.getFundBal().subtract(
                    AnalyseReportUtil.getItemMoney(itemLastMap, WkAstReportFields.WK001)));
            // 流出
            infoVO.setFlowOut(AnalyseReportUtil.getItemMoney(itemMap, CollectionUtil.newArrayList(
                    WkAstReportFields.WK005, WkAstReportFields.WK108, WkAstReportFields.WK111)));
            // 累计流出
            Money flowOutTotal = AnalyseReportUtil.getItemMoney(itemMap, CollectionUtil.newArrayList(
                    WkAstReportFields.WK109, WkAstReportFields.WK116));
            // 获取累计分配金额
            if (ToolUtil.isNotEmpty(planBases)) {
                flowOutTotal.addTo(planAllocRepo.queryStatTotalAllocAmtByPlanNos(
                        planBases.stream().map(PlanBase::getPlanNo).collect(Collectors.toList())));
            }
            infoVO.setFlowOutTotal(flowOutTotal);

            // 资金流入
            infoVO.setFlowIn(AnalyseReportUtil.getItemMoney(itemMap,
                    Lists.newArrayList(WkAstReportFields.WK004, WkAstReportFields.WK007)));

            List<ReportField> spvFeilds = Lists.newArrayList(
                    // 累计流入 = 还款（WK029...WK038）+ 计划本金投入
                    WkAstReportFields.WK029,
                    WkAstReportFields.WK030,
                    WkAstReportFields.WK031,
                    WkAstReportFields.WK032,
                    WkAstReportFields.WK033,
                    WkAstReportFields.WK034,
                    WkAstReportFields.WK035,
                    WkAstReportFields.WK036,
                    WkAstReportFields.WK037,
                    WkAstReportFields.WK038);
            Map<String, PlanReportItem> itemSpvMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(accDate, planNo, spvFeilds, true);
            // 累计流入
            Money flowInTotal = denominator.add(AnalyseReportUtil.getItemMoney(itemSpvMap, spvFeilds));
            infoVO.setFlowInTotal(flowInTotal);
    
            // 查询报表实际闲置率
            List<ReportField> idleFeilds = Lists.newArrayList(WkAstReportFields.WK002);
            Map<String, PlanReportItem> idleSpvMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(accDate, planNo, idleFeilds, false);
            // 日闲置率 与 报表实际闲置率
            infoVO.setIdleRate(CalcUtil.calcPercRate(AnalyseReportUtil.getItemBig(idleSpvMap, WkAstReportFields.WK002)));
            
//            // 日闲置率
//            infoVO.setIdleRate(CalcUtil.dividePercRate(
//                    // 闲置资金余额  / 累计流入 + 追加放款池信托计划规模
//                    AnalyseReportUtil.getItemBig(itemMap, WkAstReportFields.WK001), denominator.getAmount()));

            // 平均闲置率
            infoVO.setIdleAvgRate(analyseSpvReportManager.getAvgIdleRateRate(accDate, planBase));
            return infoVO;
        });
    }

    /**
     * 资产期限结构
     * 产品期限结构
     * @param param
     * @return
     */
    @Override
    public Map<String, List<FormationAstItemVO>> getFormationAst(AnalyseBaseParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 查询计划
            PlanBase planBase = analyseSpvReportManager.checkGetFullPlan(param.getPlanNo());
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(planBase.getPlanNo(), ReportTypeEnum.RPT001);
            Date accDate = AnalyseReportUtil.getStatDate(trfDate, param.getAccountDate(), planReport);

    //        // 剩余期限结构 查询当前资产情况
    //        List<LoanInstalFormationItem> astItems = loanInstalRepository.queryAstFormation(planNo);
    //
    //        // 产品期限结构 查询当前资产情况
    //        List<LoanInstalFormationItem> prodItems = loanInstalRepository.queryProdFormation(planNo);
            // 剩余结构
            List<ReportField> astFields = Lists.newArrayList(
                    WkAstReportFields.WKAstT001,
                    WkAstReportFields.WKAstT002,
                    WkAstReportFields.WKAstT003,
                    WkAstReportFields.WKAstT004,
                    WkAstReportFields.WKAstT005
            );
            // 产品结构
            List<ReportField> prodFields = Lists.newArrayList(
                    WkAstReportFields.WKProdT001,
                    WkAstReportFields.WKProdT002,
                    WkAstReportFields.WKProdT003,
                    WkAstReportFields.WKProdT004,
                    WkAstReportFields.WKProdT005
            );
            List<ReportField> all = Lists.newArrayList(astFields);
            all.addAll(prodFields);
            // 查询结构
            Map<String, PlanReportItem> itemMap =
                    analyseSpvReportManager.querySpvStatPlanDayItem(accDate, param.getPlanNo(), all, true);
            // 总资产金额
            Money astScale = AnalyseReportUtil.getItemMoney(itemMap, astFields);
            Money prodScale = AnalyseReportUtil.getItemMoney(itemMap, prodFields);

            Map<String, List<FormationAstItemVO>> formation = new HashMap<>();
            // 剩余期限结构
            formation.put("astFormation", AnalyseReportUtil.convertFormation(param.getPlanNo(), astScale, astFields, itemMap));
            // 产品期限结构
            formation.put("prodFormation", AnalyseReportUtil.convertFormation(param.getPlanNo(), prodScale, prodFields, itemMap));
            return formation;
        });
    }

    /**
     * 趋势统计图实现逻辑
     * @param trendChartParam
     * @return
     */
    @Override
    public AnalyseChartResultVO trendChartService(AnalyseTrendChartParam trendChartParam) {
        return super.doQry(trendChartParam, () -> {
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 计划
            PlanBase planBase = analyseSpvReportManager.checkGetFullPlan(trendChartParam.getPlanNo());
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(planBase.getPlanNo(), ReportTypeEnum.RPT001);
            // 获取业务编码
            List<AnalyseServiceEnum> serviceEnums = AnalyseServiceEnum.getByCodes(trendChartParam.getServices());
            VarChecker.checkNotEmpty(serviceEnums, "没有匹配的业务类型[{0}]", JSONUtil.toJsonStr(trendChartParam.getServices()));
            // 统计日 默认D-1日
            Date accDate = AnalyseReportUtil.getStatDate(trfDate, trendChartParam.getAccountDate(), planReport);
            AnalyseChartContext context = new AnalyseChartContext();
            context.setPlanBase(planBase);
            // 取负数回推日期
            context.setStartDate(DateUtil.offsetDay(accDate, 0 - trendChartParam.getRange()));
            context.setEndDate(accDate);
            context.setServices(serviceEnums);
            // 暂时只支持单一统计单位的图形数据
            context.setAnalyseType(serviceEnums.get(0).getType());
            analyseChartMultipleSelector.execute(context);

            AnalyseChartResultVO resultVO = new AnalyseChartResultVO();
            resultVO.setUnit(context.getAnalyseType().getUnit());
            resultVO.setItems(context.getItems());
            resultVO.setPlanNo(planBase.getPlanNo());
            resultVO.setPlanName(planBase.getPlanName());
            return resultVO;
        });
    }
}
