/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.org;

import java.util.List;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.OrganizationDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.OrgListPage;
import com.abssqr.plat.common.facade.model.org.Organization;
import com.abssqr.plat.common.facade.param.org.OrgEditParam;
import com.abssqr.plat.common.facade.param.org.OrgPageQryParam;
import com.abssqr.plat.common.facade.param.org.OrgQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.org.OrgListVO;
import com.abssqr.plat.common.facade.service.OrgService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.core.service.org.OrgManager;
import com.general.system.common.model.BaseResult;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.api.org: OrgService.java, v 0.1 2019-05-28 10:56 zhenxuan.luo Exp $
 */
@Component
public class OrgServiceImpl extends ServiceSupport implements OrgService {

    @Autowired
    private OrgManager       orgManager;

    private Logger logger = LoggerFactory.getLogger(OrgServiceImpl.class);

    @Override
    public BasePage<OrgListVO> getOrgList(@Valid OrgPageQryParam param) {
        return this.doQry(param, () -> {
            OrgListPage listPage = orgManager.getOrgList(param);
            List<OrgListVO> abbrs = Lists.newArrayList();

            for (OrganizationDO organization : listPage.getDatas()) {
                //机构编码，机构简称，机构全称，联系人，联系人电话
                OrgListVO orgListVO = new OrgListVO();
                orgListVO.setOrgCode(organization.getOrgCode());
                orgListVO.setOrgAbbr(organization.getOrgAbbreviation());
                orgListVO.setOrgName(organization.getOrgName());
                orgListVO.setLinkmanName(organization.getLinkmanName());
                orgListVO.setLinkmanPhone(organization.getLinkmanPhone());
                abbrs.add(orgListVO);
            }
            BasePage<OrgListVO> res = new BasePage<>();
            res.setCurrPageNo(listPage.getCurrPageNo());
            res.setTotal(listPage.getTotal());
            res.setLimit(listPage.getLimit());
            res.setDatas(abbrs);

            return res;
        });
    }

    @Override
    public BaseResult<String> addOrUpdate(@Valid OrgEditParam param) {
        return BaseResult.success(this.doOpr(param, () -> orgManager.addOrUpdate(param)));

    }

    @Override
    public Organization getOrg(@Valid OrgQryParam param) {
        return this.doQry(param, () -> orgManager.getOrg(param.getOrgCode()));
    }

}