/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.biz.api.plan;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.enums.PriceTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.facade.model.plan.PlanAstPriceRule;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.priceRule.PlanAstPriceRuleParam;
import com.abssqr.plat.common.facade.param.plan.priceRule.PriceQryParam;
import com.abssqr.plat.common.facade.result.plan.PlanAstPriceRuleVo;
import com.abssqr.plat.common.facade.service.plan.PlanAstPriceRuleService;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.repo.dataChange.ChangeRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAstPriceRuleRepo;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.abssqr.plat.core.service.plan.PlanAstPriceRuleManager;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.VarChecker;
import com.general.system.facade.current.IdentityUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Optional;

/**
 * @author xiachenxiang
 * @version com.abssqr.plat.biz.api.plan: PlanAstPriceRuleServiceImpl.java, v 0.1 2019-11-07 5:33 PM xiachenxiang Exp $
 */
@Service
public class PlanAstPriceRuleServiceImpl extends ServiceSupport implements PlanAstPriceRuleService {
    @Autowired
    private PlanAstPriceRuleRepo planAstPriceRuleRepo;
    @Autowired
    private PlanAstPriceRuleManager planAstPriceRuleManager;
    @Autowired
    private PlanManager planManager;
    @Autowired
    private ChangeManager changeManager;
    @Autowired
    private ChangeRepo changeRepo;


    @Override
    public PlanAstPriceRuleVo queryDtl(@Valid PriceQryParam param) {
        return this.doQry(param, () -> {
            PlanAstPriceRuleVo result = new PlanAstPriceRuleVo();
            if (param.getWork()) {
                BizDataChange<PlanBase> dataChange = changeManager.getDataChange(param.getPlanNo());
                VarChecker.checkNotNull(dataChange, "未找到对应待审批的定价规则数据");
                PlanBase planBase = dataChange.getDatas();
                result.setPlanNo(planBase.getPlanNo());
                result.setPlanName(planBase.getPlanName());
                result.setPriceType(planBase.getPriceType().getCode());
                result.setPlanAstPriceRuleList(planBase.getPlanAstPriceRuleList());
            } else {

                PlanBase planBase = planManager.getPlanFullByNo(param.getPlanNo());
                VarChecker.checkNotNull(planBase, "未找到对应的计划");
                result.setPlanNo(planBase.getPlanNo());
                result.setPriceType(Optional.ofNullable(planBase.getPriceType()).map(PriceTypeEnum::getCode).orElse(null));
                result.setPlanAstPriceRuleList(planAstPriceRuleRepo.queryByPriceRulePlanNo(param.getPlanNo()));
            }
            // 判断是否有审批中的定价
            Long cnt = changeRepo.cntByBizNoAndStatusAndBizType(DataChangeBizTypeEnum.PanAstPriceRule.getCode(),result.getPlanNo(), ApproveStatusEnum.PENDING.getCode());
            result.setPending(cnt > 0L);
            if(CollectionUtils.isNotEmpty(result.getPlanAstPriceRuleList())){
                for (PlanAstPriceRule planAstPriceRule : result.getPlanAstPriceRuleList()) {
                    if(planAstPriceRule.getIntrDiscountRate()!=null) {
                        planAstPriceRule.setIntrDiscountRate(
                            CalcUtil.calcPercRate(planAstPriceRule.getIntrDiscountRate()));
                    }
                }
            }
            return result;
        });

    }

    @Override
    public Void edit(PlanAstPriceRuleParam param) {
        if(param!=null&&CollectionUtils.isNotEmpty(param.getPlanAstPriceRuleList())){
            for (PlanAstPriceRule planAstPriceRule : param.getPlanAstPriceRuleList()) {
                if(planAstPriceRule.getIntrDiscountRate()!=null) {
                    planAstPriceRule.setIntrDiscountRate(CalcUtil.calcRealRate(planAstPriceRule.getIntrDiscountRate()));
                }
            }
        }
        return this.doOpr(param, () -> planAstPriceRuleManager.updateRule(param, IdentityUtil.getUserCode(), IdentityUtil.getUserOrgCode()));
    }

}
