/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2018 All Rights Reserved. 
 */
package com.abssqr.plat.biz.interceptor;

import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.util.constants.LogConstants;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.abssqr.plat.common.util.loggers.DigestLoggers;
import com.abssqr.plat.common.util.loggers.LogInfo;
import com.abssqr.plat.common.util.thread.BizContextHolder;
import com.general.system.common.model.IErrorCode;
import com.general.system.common.model.exception.BaseException;
import com.general.system.common.util.DateTimeUtil;
import org.aopalliance.intercept.MethodInvocation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author zhenxuan.luo
 * @version com.loan.common.model.log: BizLogUtil.java, v 0.1 2018-12-19 AM11:56 zhenxuan.luo Exp $ 
 */
public class BizLogUtil {
    
    /**
     *
     */
    private final static Logger LOGGER = LoggerFactory.getLogger(BizLogUtil.class);
    
    private final static Logger BIZ_DIGEST = DigestLoggers.SERVICE_DIGEST;
    
    private final static Logger BIZ_REQUEST = CommonLoggers.BIZ_REQUEST;
    
    private final static String UNDEFINE = CommonErrorCodeEnum.UNDEFINE.getCode();
    
    /**
     * 记录业务请求日志
     *
     * @param invocation
     */
    public static void logBizReqeust(LogInfo logInfo, MethodInvocation invocation) {
        if (BIZ_REQUEST.isInfoEnabled()) {
            StringBuilder sb = new StringBuilder(512);
            sb.append(logInfo.getFacadeName()).append(LogConstants.DOT).append(logInfo.getMethodName());
            sb.append(LogConstants.FMT_BEGIN);
            Object[] params = invocation.getArguments();
            if (params != null) {
                for (int i = 0; i < params.length; i++) {
                    sb.append(params[i]);
                    if (i != params.length - 1) {
                        sb.append(LogConstants.COMMA);
                    }
                }
            }
            sb.append(LogConstants.FMT_END);
            BIZ_REQUEST.info(sb.toString());
        }
    }
    
    /**
     * 构建操作类型的摘要日志
     *
     * @param eventCode     本系统事件码
     * @param throwable
     */
    public static void logBizDigest(LogInfo logInfo, String eventCode, Throwable throwable) {
        
        try {
            //1.1 构建开始标志
            StringBuilder sb = new StringBuilder(400);
            sb.append(LogConstants.FMT_BEGIN);
            
            sb.append(logInfo.getFacadeName()).append(LogConstants.DOT);
            sb.append(logInfo.getMethodName()).append(LogConstants.COMMA);
            sb.append((throwable == null) ? LogConstants.YES : LogConstants.NO).append(LogConstants.COMMA);
            sb.append(logInfo.getElapseTime()).append(LogConstants.MS);
            
            sb.append(LogConstants.FMT_MIDDLE);
            
            sb.append(eventCode).append(LogConstants.COMMA);
            sb.append(ifNull(BizContextHolder.getBizNo())).append(LogConstants.COMMA);
            sb.append(ifNull(DateTimeUtil.formatYMDHMsS(BizContextHolder.getTransTime())));
            
            sb.append(LogConstants.FMT_MIDDLE);
            IErrorCode errorCode = getErrorCode(throwable);
            sb.append((errorCode == null) ? LogConstants.NONE : errorCode.getCode()).append(LogConstants.COMMA);
            sb.append((errorCode == null) ? LogConstants.NONE : errorCode.getDesc());
            sb.append(LogConstants.FMT_END);
            
            //2 写入摘要日志文件
            if (BIZ_DIGEST.isInfoEnabled()) {
                BIZ_DIGEST.info(sb.toString());
            }
            
        } catch (Throwable e) {
            LOGGER.error("打印摘要日志异常", e);
        }
    }
    
    /**
     * 入参为null时,返回"-",否则返回自身toString
     *
     * @param value 入参
     * @return
     */
    private static String ifNull(String value) {
        return value == null ? LogConstants.NONE : value;
    }
    
    private static IErrorCode getErrorCode(Throwable ex) {
        if (null == ex) {
            return null;
        }
        IErrorCode errorCode;
        if (ex instanceof BaseException) {
            errorCode = ((BaseException) ex).getErrCode();
        } else {
            errorCode = CommonErrorCodeEnum.UNDEFINE;
        }
        return errorCode;
    }
}