/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.support;

import com.abssqr.plat.common.facade.base.BaseOprParam;
import com.abssqr.plat.common.facade.base.BasePageQryParam;
import com.abssqr.plat.common.facade.base.BaseParam;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.thread.BizContextHolder;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.service.support: ServiceSupport.java, v 0.1 2019-01-17 16:47 zhenxuan.luo Exp $
 */
public abstract class ServiceSupport {

    protected final static Logger logger = LoggerFactory.getLogger(ServiceSupport.class);

    @Autowired
    private TransactionTemplate transactionTemplate;
    
    /**
     * 查询数据
     * @param param
     * @param qryTemplate
     * @param <P>
     * @param <R>
     * @return
     */
    protected <P extends BaseParam, R> R doQry(final P param, final QryTemplate<R> qryTemplate) {

        if (param instanceof BasePageQryParam) {
            if (((BasePageQryParam)param).getPageNum() == null) {
                ((BasePageQryParam)param).setPageNum(BasePageQryParam.DEFAULT_PAGENUM);
            }

            if (((BasePageQryParam)param).getPageSize() == null) {
                ((BasePageQryParam)param).setPageSize(BasePageQryParam.DEFAULT_PAGESIZE);
            }
        }

        String bizNo = SequenceUtil.genFakeSeqNo();
        BizContextHolder
            .set(bizNo, param.getEventCode().getCode());

        R ret = qryTemplate.query();
        return qryTemplate.afterProcess(ret);
    }
    
    /**
     * 变更数据
     * @param param
     * @param oprTemplate
     * @param <P>
     * @param <R>
     * @return
     */
    protected <P extends BaseOprParam, R> R doOpr(final P param, final OprTemplate<R> oprTemplate) {

        String bizNo = SequenceUtil.genSeqNo(param.getEventCode().getCode());
        BizContextHolder
            .set(bizNo, param.getEventCode().getCode());

        if (StringUtils.isBlank(param.getBizNo())) {
            param.setBizNo(bizNo);
        }

        R ret = transactionTemplate.execute(status -> oprTemplate.execute());

        return oprTemplate.afterProcess(ret);
    }
}