/**
 * abssqr.com Inc.
 * Copyright (c) 2017-${YEAR} All Rights Reserved.
 */
package com.abssqr.plat.biz.validate;


import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

/**
 * 每个字段错误的存储，以保存所有的参数错误信息，避免出现错误信息中断。
 * 
 * @author zhenxuan.luo
 * @version $Id: FieldError.java, v 0.1 2015年5月18日 下午7:36:15 zhenxuan.luo Exp $
 */
public class FieldError {
    /** 对象名称 */
    private final String objectName;

    /** 字段名称 */
    private final String fieldName;

    /** 字段的值 */
    private final Object fieldValue;

    /** 错误信息 */
    private String       errorMsg;

    /**
     * @param objectName 字段所在的对象
     * @param fieldName 字段的名称
     * @param fieldValue 字段的值
     */
    public FieldError(String objectName, String fieldName, Object fieldValue) {
        super();
        this.objectName = objectName;
        this.fieldName = fieldName;
        this.fieldValue = fieldValue;
    }

    /**
     * Getter method for property <tt>errorMsg</tt>.
     * 
     * @return property value of errorMsg
     */
    public String getErrorMsg() {
        return errorMsg == null ? "" : errorMsg;
    }

    /**
     * Setter method for property <tt>errorMsg</tt>.
     * 
     * @param errorMsg value to be assigned to property errorMsg
     */
    public void setErrorMsg(String errorMsg) {
        this.errorMsg = errorMsg;
    }

    /**
     * Getter method for property <tt>objectName</tt>.
     * 
     * @return property value of objectName
     */
    public String getObjectName() {
        return objectName;
    }

    /**
     * Getter method for property <tt>fieldName</tt>.
     * 
     * @return property value of fieldName
     */
    public String getFieldName() {
        return fieldName;
    }

    /**
     * Getter method for property <tt>fieldValue</tt>.
     * 
     * @return property value of fieldValue
     */
    public Object getFieldValue() {
        return fieldValue;
    }

    /**
     * 以短格式的方式输出对象属性
     *
     * @see Object#toString()
     */
    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this, ToStringStyle.SHORT_PREFIX_STYLE);
    }
}
