/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldBigDecimal;

import java.math.BigDecimal;

/**
 * 注解@{@link VldBigDecimal}的校验类实现
 *
 * @author zhenxuan.luo
 * @version $Id: MoneyValidator.java, v 0.1 2015年5月19日 上午11:01:04 zhenxuan.luo Exp $
 */
public final class BigDecimalValidator extends AFieldValidator<VldBigDecimal> {
    /**
     * 默认构造器
     */
    public BigDecimalValidator() {
        super(VldBigDecimal.class);
    }
    
    @Override
    public boolean isValid(ValidateService validateService, VldBigDecimal annotationObj,
                           Object validTargetFieldObj, FieldError errorConext) {
        boolean isValid = false;
        try {
            return isValid = validate(annotationObj, validTargetFieldObj);
        } finally {
            if (!isValid) {
                errorConext.setErrorMsg(annotationObj.message());
            }
        }
    }
    
    /**
     * 校验
     * @param annotationObj 校验注解对象
     * @param validTargetFieldObj 需要校验的字段对象
     * @return true:校验通过,反之返回false
     */
    private boolean validate(VldBigDecimal annotationObj, Object validTargetFieldObj) {
        if (validTargetFieldObj == null) {
            return (annotationObj.nullable()) ? true : false;
        }
        if (validTargetFieldObj instanceof BigDecimal) {
            BigDecimal decimal = (BigDecimal) validTargetFieldObj;
            if (annotationObj.isGreaterThanZero() && decimal.compareTo(BigDecimal.ZERO) <= 0) {
                return false;
            }
            
            if (!annotationObj.canNegative() && decimal.compareTo(BigDecimal.ZERO) < 0) {
                return false;
            }
            
            return true;
        }
        return false;
    }
}
