/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2018 All Rights Reserved. 
 */
package com.abssqr.plat.biz.shared.dispatcher;

import com.abssqr.plat.common.model.domain.task.DispatcherNode;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.general.system.common.util.LogUtil;
import org.quartz.*;
import org.slf4j.Logger;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.dispatcher: DispatcherStarter.java, v 0.1 2018-01-09 下午23:51 zhenxuan.luo Exp $ 
 */
public abstract class DispatcherStarter<T extends DispatcherNode> implements InitializingBean {

    private final static Logger LOG = CommonLoggers.COMMON_DISPATCHER;
    
    @Autowired
    private BeanFactory beanFactory;
    
    @Autowired
    private JobControlRepository jobControlRepository;

    @Autowired
    Scheduler scheduler;

    @Override
    public void afterPropertiesSet() throws Exception {
        LOG.info("[" + getName() + ")]TaskGroup Initializer begin...");
        // 1. 先做启动前恢复
        doRecovery();
        
        // 2. 启动线程池
        doStartDispatcher();
        
        LOG.info("[" + getName() + "]TaskGroup Initializer end...");

    }
    
    private void doRecovery() {
        LOG.info("[" + getName() + ")]TaskGroup recovery hang tasks...");
        try {
            recovery();
        } catch (Throwable e) {
            LOG.error(getName() + " dispatcher启动恢复异常，暂时跳过恢复，等待人工处理", e);
        }
    }
    
    private void doStartDispatcher() {
        
        Dispatcher<T> taskDispatcher = new Dispatcher<T>();
        
        taskDispatcher.setDispatcherLoader(getLoader());
        taskDispatcher.setDispatcherExecutor(getExecutor());
        taskDispatcher.setDispatcherSplitor(getSplitor());
        
        taskDispatcher.setCoreSize(getCoreSize());
        taskDispatcher.setMaxSize(getMaxSize());
        taskDispatcher.setQueueSize(getQueueSize());
        taskDispatcher.setHungrySize(getHungrySize());
        taskDispatcher.setName(getName());
        
        registerBean(getName(), taskDispatcher);
    
        taskDispatcher.setJobControlRepository(jobControlRepository);
        
        Thread dispatcherThread = new Thread(taskDispatcher, getName());
        dispatcherThread.start();

    }
    
    protected abstract void recovery();
    
    public abstract String getName();
    
    protected abstract int getCoreSize();
    
    protected abstract int getMaxSize();
    
    protected abstract int getQueueSize();
    
    protected abstract int getHungrySize();
    
    protected abstract DispatcherLoader<T> getLoader();
    
    protected abstract DispatcherExecutor<T> getExecutor();

    protected DispatcherSplitor getSplitor(){
        return null;
    }

    private void registerBean(String beanName, Object bean) {
        ConfigurableBeanFactory configurableBeanFactory = (ConfigurableBeanFactory) beanFactory;
        configurableBeanFactory.registerSingleton(beanName, bean);
    }
}