/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.buyback;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.dal.mysql.auto.dao.TrTaskDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrTaskDO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.convertor.TrTaskConvert;
import com.abssqr.plat.common.model.domain.ast.LoanAstBuyBackPack;
import com.abssqr.plat.common.model.domain.task.TrCtrTask;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.repo.tr.TrCtrTaskRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.ast.LoanAstBuyBackManager;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.List;

/**
 * 回购转让交易任务捞取
 * @author hanfei
 * @version $Id: LoanAstBuyBackLoader.java, v 0.1 2020-02-11 5:15 PM hanfei Exp $
 */
@Component
public class LoanAstBuyBackLoader implements DispatcherLoader<TrTask> {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(
            LoanAstBuyBackLoader.class);
    
    @Autowired
    private TrTaskDAO             trTaskDAO;
    @Autowired
    private TransactionTemplate   transactionTemplate;
    @Autowired
    private HostManager           hostManager;
    @Autowired
    private TrCtrTaskRepo         trCtrTaskRepo;
    @Autowired
    private LoanAstBuyBackManager loanAstBuyBackManager;
    @Autowired
    private OrgAcctDayComponent   orgAcctDayComponent;
    /**
     * 装载待分发节点
     *
     * @param size
     * @return
     */
    @Override
    public List<TrTask> loadToDoNodes(int size) {
        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        // 1、捞取回购任务总控STATUS为init
        List<TrCtrTask> trCtrTasks = trCtrTaskRepo.queryInitCtrTaskList(
                curDate, TaskStatusEnum.INIT, AstPackTradeTypeEnum.BUY_BACK_BIZ);
        if (ToolUtil.isEmpty(trCtrTasks)) {
            return Lists.newArrayList();
        }
    
        TrCtrTask trCtrTask = null;
        for(TrCtrTask one : trCtrTasks){
            // 2、捞取对应回购筛选包校验 PACK_STATUS为SCEEN_FIS  和 APPROVE_STATUS为PASSED
            LoanAstBuyBackPack pack = loanAstBuyBackManager.getByPackCode(one.getPackCode());
            if (pack.canExec()){
                trCtrTask = one;
                break;
            }
        }
        // 没有可执行任务
        if (trCtrTask == null){
            return Lists.newArrayList();
        }
        Long canExecId = trCtrTask.getId();
        return transactionTemplate.execute(transactionStatus -> {
            TrCtrTask lockedCtrTask = trCtrTaskRepo.lockById(canExecId);
            if (lockedCtrTask.getTaskStatus() != TaskStatusEnum.INIT) {
                return Lists.newArrayList();
            }
            
            Long undoCnt = trTaskDAO.cntInitTaskByBatchNo(lockedCtrTask.getBatchNo());
            if (undoCnt == 0) {
                LogUtil.info(LOGGER, "转让控制[{0}]无待转任务，标记结束", lockedCtrTask);
                trCtrTaskRepo.updateStatus(lockedCtrTask.getId(), TaskStatusEnum.SUCCESS);
                return Lists.newArrayList();
            }
            trCtrTaskRepo.updateExecTms(lockedCtrTask.getId(), lockedCtrTask.getExecTimes() + 1);
    
            // 3、捞取回购资产任务TASK_STATUS为init
            List<TrTaskDO> taskDOS = trTaskDAO.selectInitTask(SystemDateUtil.getSystemDate(), Long.valueOf(size),
                    lockedCtrTask.getBatchNo(),  AstPackTradeTypeEnum.BUY_BACK_BIZ.getCode());
            if (CollectionUtils.isEmpty(taskDOS)) {
                return Lists.newArrayList();
            }
            // 4、更新任务TASK_STATUS为exec
            int count = trTaskDAO.updateTaskExec(SystemDateUtil.getSystemDate(),
                    hostManager.getHostInfo().getHostName(),
                    taskDOS);
            VarChecker.checkEquals(count, taskDOS.size(), "[TaskLoader.loadToDoNodes]更新任务数跟期望数不一致，实际更新：{0}，期望更新：{1}",
                    count,
                    taskDOS.size());
    
            LogUtil.info(LOGGER, "转让控制[{0}]捞取[{1}]待执行任务", lockedCtrTask, count);
            return TrTaskConvert.convertDTOs(taskDOS);
        });
    }
}
