/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.buyback;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutor;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.biz.shared.dispatcher.DispatcherStarter;
import com.abssqr.plat.common.dal.mysql.auto.dao.TrTaskDAO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.core.service.host.HostManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * 回购转让交易任务组件绑定
 * @author hanfei
 * @version $Id: LoanAstBuyBackLoader.java, v 0.1 2020-02-11 5:15 PM hanfei Exp $
 */
@Component
public class LoanAstBuyBackStarter extends DispatcherStarter<TrTask> {

    @Autowired
    private LoanAstBuyBackExecutor loanAstBuyBackExecutor;
    @Autowired
    private LoanAstBuyBackLoader   loanAstBuyBackLoader;
    @Autowired
    private HostManager            hostManager;
    @Autowired
    private TrTaskDAO              trTaskDAO;

    @Value("${dispatcher.tr.coreSize}")
    private int coreSize;

    @Value("${dispatcher.tr.maxSize}")
    private int maxSize;
    
    @Value("${dispatcher.tr.hungrySize}")
    private int hungrySize;
    
    @Value("${dispatcher.tr.queueSize}")
    private int queueSize = 200;

    @Override
    protected void recovery() {
        trTaskDAO.updateHangTasksByIp(
                hostManager.getHostInfo().getHostName(), AstPackTradeTypeEnum.BUY_BACK_BIZ.getCode());
    }
    
    public static String BUY_BACK_NAME = "LoanAstBuyBackStarter";
    @Override
    public String getName() {
        return BUY_BACK_NAME;
    }

    @Override
    protected int getCoreSize() {
        return coreSize;
    }

    @Override
    protected int getMaxSize() {
        return maxSize;
    }

    @Override
    protected int getQueueSize() {
        return queueSize;
    }

    @Override
    protected int getHungrySize() {
        return hungrySize;
    }

    @Override
    protected DispatcherLoader<TrTask> getLoader() {
        return loanAstBuyBackLoader;
    }

    @Override
    protected DispatcherExecutor<TrTask> getExecutor() {
        return loanAstBuyBackExecutor;
    }

}
