package com.abssqr.plat.biz.shared.dispatcher.capitalAssetMatch;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.core.service.host.HostManager;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class PlanCapitalAssertMatchLoader implements DispatcherLoader<CommonTask> {
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanCapitalAssertMatchLoader.class);
    @Autowired
    PlanRepo planRepo;
    @Autowired
    CommonTaskRepository commonTaskRepository;

    @Autowired
    private HostManager hostManager;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    JobControlRepository jobControlRepository;

    @Autowired
    PlanCapitalAssertMatchExecutor planCapitalAssertMatchExecutor;
    @Override
    public List<CommonTask> loadToDoNodes(int size) {
        List<CommonTask> commonTasksList=new ArrayList<>();
        transactionTemplate.execute(transactionStatus->{
            JobControl jobControl = jobControlRepository.lockByJobName(planCapitalAssertMatchExecutor.getName());
            if(jobControl==null||jobControl.getIsEff()== YNEnum.N){
                return false;
            }
//            LogUtil.info(LOGGER,"[PlanCapitalAssertMatchLoader] 捞取资金资产匹配/解绑任务开始");

            List<PlanBase> plans = planRepo.getAllPlanAstByType(PlanTypeEnum.SPV.getCode());

            for(PlanBase plan:plans){
                String planNo = plan.getPlanNo();
                //查询是否有正在执行中的解绑/匹配任务
                List<CommonTask> runningCommonTask = commonTaskRepository.queryExecUnMatchOrMatchTaskByContext(planNo);
                if(CollectionUtils.isNotEmpty(runningCommonTask)){
                    continue;
                }
                //查询初始化的匹配任务
                List<CommonTask> initCommonTask = commonTaskRepository.queryInitUnMatchOrMatchTaskByContext(planNo);
                if(CollectionUtils.isNotEmpty(initCommonTask)){
                    CommonTask commonTask = initCommonTask.get(0);
                    int cnt=commonTaskRepository.updateTaskExec(new Date(),hostManager.getHostInfo().getHostName(), Lists.newArrayList(commonTask));
                    if(cnt==1){
                        commonTasksList.add(commonTask);
                    }
                }

            }
//            LogUtil.info(LOGGER,"[PlanCapitalAssertMatchLoader] 捞取资金资产匹配/解绑任务结束");
           return true;
        });

        return commonTasksList;
    }
}
