/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.match;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.facade.enums.AstPackStatusEnum;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.ast.Pack;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.ast.AstPackManager;
import com.abssqr.plat.core.service.host.HostManager;
import com.abssqr.plat.core.service.util.CommonTaskUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.ThreadUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.dispatcher.match: PlanAstMatchDispatcherStarter.java, v 0.1 2019-11-12 11:03
 * bangis.wangdf Exp $
 */

@Component
@Configuration
public class PlanAstMatchDispatcherLoader implements DispatcherLoader<CommonTask> {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanAstMatchDispatcherLoader.class);

    private static long sleepTime = 1000 * 60;
    @Autowired
    private TransactionTemplate  transactionTemplate;
    @Autowired
    private HostManager          hostManager;
    @Autowired
    private AstPackManager       astPackManager;
    @Autowired
    private AstSyncTaskCtrRepo   astSyncTaskCtrRepo;
    @Autowired
    private JobControlRepository jobControlRepository;
    @Autowired
    private CommonTaskRepository commonTaskRepository;
    @Autowired
    private OrgAcctDayComponent  orgAcctDayComponent;

    @Override
    public List<CommonTask> loadToDoNodes(int size) {
        ThreadUtil.sleep(sleepTime, LOGGER);
        Date curDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        return transactionTemplate.execute(transactionStatus -> {
            // 太多乐观锁冲突报错了，还是加个锁吧
            jobControlRepository.lockByJobName(PlanAstMatchDispatcherStarter.JOB_NAME);
            List<CommonTask> tasks = Lists.newArrayList();
            List<CommonTask> existsTask = commonTaskRepository.selectUnifinishTaskByType(CommonTaskTypeEnum.PACK_MATCH);
            Map<String, CommonTask> taskMap = existsTask.stream().collect(
                    Collectors.toMap(CommonTask::getContext, item -> item));
            // 获取当前资产内部模型同步完成
            List<String> orgCodes = astSyncTaskCtrRepo.getLastStatusFinishedOrgCodes(curDate, SyncTaskTypeEnum.AST_MOD);
            if (CollectionUtils.isNotEmpty(orgCodes)) {
                //获取以资产机构维度的包
                Map<String, List<Pack>> orgPackMap = astPackManager.queryPacksByOrgCodes(orgCodes, curDate,
                        AstPackStatusEnum.INIT);
                orgPackMap.forEach((orgCode, packs) -> {
                    if (orgCodes.contains(orgCode)) {
                        Map<String, List<Pack>> spvPlanPackMap = trfSpvPlanPackMap(packs);
                        if (MapUtils.isNotEmpty(spvPlanPackMap)) {
                            for (Map.Entry<String, List<Pack>> entity : spvPlanPackMap.entrySet()) {
                                CommonTask commonTask = taskMap.get(entity.getKey());
                                if (null == commonTask) {
                                    commonTask = CommonTaskUtil.buildCommonTaskWithContext(
                                            SequenceUtil.genWkSeqNo(), CommonTaskTypeEnum.PACK_MATCH, entity.getKey());
                                    commonTaskRepository.create(commonTask);
                                } else {
                                    if (commonTask.getTaskStatus() != TaskStatusEnum.INIT) {
                                        continue;
                                    } else if (SystemDateUtil.getSystemDate().compareTo(commonTask.getNextExecTime())
                                            < 0) {
                                        continue;
                                    }
                                }
                                commonTask.setExtObj(spvPlanPackMap.get(entity.getKey()));
                                tasks.add(commonTask);
                                if (tasks.size() >= size) {
                                    break;
                                }
                            }
                        }
                        spvPlanPackMap.get(0);
                    } else {
                        LogUtil.warn(LOGGER, "[PlanAstMatchDispatcherLoader][{0}]AST_MOD未完成，[{1}]忽略执行,非正常生成筛选包",
                                orgCode, packs);
                        astPackManager.updatePackStatus(packs, AstPackStatusEnum.SCEEN_IGR,
                                AstPackStatusEnum.INIT);
                    }
                });
            }
            if (CollectionUtils.isNotEmpty(tasks)) {
                commonTaskRepository
                        .updateTaskExec(SystemDateUtil.getSystemDate(), hostManager.getHostInfo().getHostName(),
                                tasks);
            }
            return tasks;
        });
    }

    /**
     * 放款计划-受让计划 一对多
     *
     * @param packs
     * @return
     */
    private Map<String, List<Pack>> trfSpvPlanPackMap(List<Pack> packs) {
        Map<String, List<Pack>> spvPlanPackMap = Maps.newHashMap();
        packs.forEach(pack -> {
            String spvPlanNo = pack.getTransferRule().getAstScreenRuleList().stream().findFirst()
                    .get().getLenderOrgCode();
            List<Pack> spvPacks = spvPlanPackMap.get(spvPlanNo);
            if (CollectionUtils.isEmpty(spvPacks)) {
                spvPacks = Lists.newArrayList();
                spvPlanPackMap.put(spvPlanNo, spvPacks);
            }
            spvPacks.add(pack);
        });
        return spvPlanPackMap;
    }

    public static void setSleepTime(long sleepTime) {
        PlanAstMatchDispatcherLoader.sleepTime = sleepTime;
    }
}