package com.abssqr.plat.biz.shared.handler;

import com.abssqr.plat.biz.shared.scheduler.job.capitalAssertMatch.CapitalAssertMatchJob;
import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstUnMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstScale;
import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAllocItem;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAstGroup;
import com.abssqr.plat.common.model.domain.ast.LoanBak;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.TransInfoTemplate;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.ast.LoanBakRepo;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.loanTrans.LoanLogRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.model.repo.tr.TrTaskRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.ast.LoanAstAssembler;
import com.abssqr.plat.core.service.plan.PlanInvestStructureManager;
import com.abssqr.plat.core.service.system.TimeStampUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.*;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.abssqr.plat.common.model.enums.IdTypeEnum.CAP_AST_UNMATCH;


/**
 * 资金资产解除绑定
 */
@Component
public class CapitalAstUnMatchHandler {
    private final Logger LOGGER = LoggerFactory.getLogger(getClass());
    @Autowired
    private TrTaskRepo trTaskRepo;

    @Autowired
    private PlanInvestStructureManager planInvestStructureManager;

    @Autowired
    private PlanAllocRepo planAllocRepo;

    @Autowired
    private PlanAcctTitleRepo planAcctTitleRepo;

    @Autowired
    private LoanBakRepo loanBakRepo;

    @Autowired
    LoanRepository loanRepository;

    @Autowired
    LoanLogRepository loanLogRepository;
    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private LoanAstAssembler loanAstAssembler;

    @Autowired
    private CommonTaskRepository commonTaskRepository;

    @Autowired
    private AbsConfig           absConfig;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;

    @Autowired
    private JobControlRepository jobControlRepository;
    @Autowired
    private CapitalAssertMatchJob capitalAssertMatchJob;

    private int queryLimit = 2000;

    /**
     *
     * @param planNo
     * @param allocNo
     * @param acctDate
     * @return
     */
    public boolean execute(String planNo, String allocNo,Date acctDate){

        //判断存在ignore、init、exec状态的转让任务
        Long cnt = trTaskRepo.cntTrTaskByTfrOut(planNo, Lists.newArrayList(TaskStatusEnum.IGNORE.getCode(), TaskStatusEnum.INIT.getCode(), TaskStatusEnum.EXECUTING.getCode()),
                AstPackTradeTypeEnum.AST_TRF_BIZ);
        VarChecker.checkArgument(cnt.intValue() == 0, "[CapitalAstUnMatchHandler] - [{0}]转让任务未完成", planNo);
        LogUtil.info(LOGGER, "[CapitalAstUnMatchHandler] - [{0}]开始资金资产解绑兑付计划编号[{1}]", planNo,allocNo);
        //获取该计划下的所有的科目信息
        List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(planNo);
        //获取该计划下的所有投资结构编号
        List<InvestStructure> investStructureList = planInvestStructureManager.selectByPlanNo(planNo);
        Set<String> allLayerNos = investStructureList.stream()
                .collect(Collectors.groupingBy(InvestStructure::getLayerNo))
                .keySet();
        //通过兑付计划获取兑付计划对应的投资层级编号
        PlanAlloc planAllocDetail = planAllocRepo.getPlanAllocDetail(allocNo);
        VarChecker.checkArgument(planAllocDetail!= null, "[CapitalAstUnMatchHandler] - [{0}]兑付单不存在", allocNo);
        //投资科目编号
        Set<String> titleNos = planAllocDetail.getPlanAllocItemList().stream()
                .filter(v->v.getConfirmAmt()!=null&&v.getConfirmAmt().isGreaterThanZero())
                .collect(Collectors.groupingBy(PlanAllocItem::getTitleNo))
                .keySet();
        //兑付单对应的投资层级编号
        Set<String> allocLayerNos = planAcctTitles.stream()
                    .filter(v->titleNos.contains(v.getTitleNo()))
                    .collect(Collectors.groupingBy(PlanAcctTitle::getRefNo))
                    .keySet();

        //计算各个投资层级需要解绑的资产额度
        final Map<String, Money> needUnMatchBalMap = caclNeedUnMatchCapitalBal(planNo, planAcctTitles, allLayerNos,allocLayerNos,acctDate);
        needUnMatchBalMap.forEach((layerNo,needUnMatchMoney)->{
            CapitalAstUnMatchPage capitalAstMatchPage = new CapitalAstUnMatchPage();
            capitalAstMatchPage.setCurrPageNo(1);
            capitalAstMatchPage.setLimit(queryLimit);
            capitalAstMatchPage.setCapitalNo(layerNo);
            do{
                boolean b = unMatchCaptialAst(planNo,capitalAstMatchPage, needUnMatchMoney,acctDate);
                if(!b) {
                    break;
                }
            }while(capitalAstMatchPage.hasNextPage());
        });
        ;
        //生成解绑重新匹配的Task
        CommonTask commonTask = buildUnMatchMatchTask(planNo, generateBizNo(CommonTaskTypeEnum.UNMATCH_CAPTITALASSERT_MATCH,planNo,acctDate,allocNo));
        commonTaskRepository.create(commonTask);
        return true;
    }

    /**
     * 计算需要解绑的金额
     * @param planNo            计划编号
     * @param planAcctTitles    科目
     * @param allLayerNos       所有的投资层级编号
     * @param acctDate          会计日
     * @param allocLayerNos     兑付计划对应的投资层级编号
     * @return
     */
    private Map<String,Money> caclNeedUnMatchCapitalBal(String planNo,List<PlanAcctTitle> planAcctTitles,Set<String> allLayerNos,Set<String> allocLayerNos,Date acctDate){
        //获取已审批未兑付的金额
        Map<String, Money> willPayMoneyMap = planAllocRepo.queryApprovedNotPayMoneyGroupByLayerNo(planNo);
        //计算所有投资结构持有的资金和资金总额
        Money investStructureBalSum = new Money();
        Map<String, Money> balMap=calcCaptial(willPayMoneyMap,investStructureBalSum,planAcctTitles);

        if (!(allLayerNos.size() == balMap.size())) {
            LogUtil.error(LOGGER, "[CapitalAstMatchHandler] - [{0}]投资结构与本金科目不一致", planNo);
            throw new AbssqrBizException(
                    MessageUtil.formatMsg("[{0}]投资结构与本金科目不一致", planNo),
                    CommonErrorCodeEnum.INVALID_PARAM, false);
        }
        if(!investStructureBalSum.isGreaterThanZero()){
            throw new AbssqrBizException(
                    MessageUtil.formatMsg("投资结构资金小于0", planNo),
                    CommonErrorCodeEnum.INVALID_PARAM, false);
        }
        // 计算资金比例
        Map<String, BigDecimal> rateMap = this.calcCapistalRate(allLayerNos, balMap, investStructureBalSum);

        //计算获取信托资产规模
        List<AstScale> astScaleList = loanBakRepo.getAstScale(acctDate, planNo);
        Money astScaleSum = new Money();

        Map<String, Money> astScaleMap = Maps.newHashMap();
        astScaleList.forEach(item -> {
            astScaleSum.addTo(item.getPrinBal());
            astScaleMap.put(item.getCapitalNo(), item.getPrinBal());
        });


        // 计算目标匹配余额并移除变更金额<0且不为兑付对应投资结构的投资结构
        calcUnMatchMatchBal(allLayerNos, astScaleMap ,astScaleSum,rateMap,allocLayerNos,investStructureBalSum,balMap);

        //重新计算资金比
        calcCapistalRate(allLayerNos,balMap,astScaleSum);

        //重新计算资产差额
        Map<String, Money> allRebalanceBalMap = calcUnMatchMatchBal(allLayerNos, astScaleMap, astScaleSum, rateMap, allocLayerNos, investStructureBalSum, balMap);
        Map<String,Money> needUnbinBalMap=new HashMap<>();
        allRebalanceBalMap.forEach((layerNo,unBindBal)->{
            if(allocLayerNos.contains(layerNo)&&unBindBal.isNegtive()){
                needUnbinBalMap.put(layerNo,unBindBal);
            }
        });
        return needUnbinBalMap;
    }


    /**
     * 计算所有机构持有的资金
     * @param willPayMoneyMap
     * @param investStructureBalSum
     * @param planAcctTitles
     * @return
     */
    private Map<String,Money> calcCaptial(Map<String, Money> willPayMoneyMap,Money investStructureBalSum,List<PlanAcctTitle> planAcctTitles){
        Map<String,Money> balMap=new HashMap<>();
        for(PlanAcctTitle planAcctTitle:planAcctTitles){
            if (planAcctTitle.getTitleType() == TitleTypeEnum.INVESTED
                    || planAcctTitle.getTitleType() == TitleTypeEnum.PRIOR_INVESTED
                    || planAcctTitle.getTitleType() == TitleTypeEnum.SECND_INVESTED) {
                Money money = planAcctTitle.getBal().clone();
                Money willPayMoney = willPayMoneyMap.getOrDefault(planAcctTitle.getRefNo(),new Money());
                money.subtractFrom(willPayMoney);
                balMap.put(planAcctTitle.getRefNo(), money);
                investStructureBalSum.addTo(money);
            }
        }
        return balMap;
    }
    /**
     * 计算资金比例
     *
     * @param layerNos 投资结构编号
     * @param balMap 余额
     * @param sum 总额
     * @return
     */
    private Map<String, BigDecimal> calcCapistalRate(Set<String> layerNos, Map<String, Money> balMap, Money sum) {

        Map<String, BigDecimal> rateMap = Maps.newHashMap();

        layerNos.forEach(item -> {
            Money bal = balMap.get(item);
            BigDecimal rate = bal.getAmount().divide(sum.getAmount(), 2, BigDecimal.ROUND_HALF_UP);
            rateMap.put(item, rate);
        });
        return rateMap;
    }

    /**
     * 计算资金资产差额
     * @param allLayerNos           所有的投资层级编号
     * @param astScaleMap           各投资结构的资产规模
     * @param astScaleSum           所有投资结构的资产总额
     * @param rateMap               各投资结构的资金比例
     * @param allocLayerNos         兑付计划对应的投资层级编号
     * @param investStructureBalSum 投资层级资金总额度
     * @param balMap                各投资层级对应的资金
     * @return
     */
    private Map<String, Money>  calcUnMatchMatchBal(Set<String> allLayerNos,Map<String, Money> astScaleMap ,Money astScaleSum,Map<String, BigDecimal> rateMap,Set<String> allocLayerNos,Money investStructureBalSum,Map<String, Money> balMap){
        Map<String, Money> needUnMatchBalMap=new HashMap<>();
        Iterator<String> iterator = allLayerNos.iterator();
        while(iterator.hasNext()){
            String layerNo = iterator.next();
            BigDecimal balRate = rateMap.get(layerNo);
            //目标兑付资产
            Money targetBal = astScaleSum.multiply(balRate);
            //持有的资产
            Money matchedBal = astScaleMap.getOrDefault(layerNo,new Money());
            //资产差额(目标兑付的资产-持有资产)
            Money divide = targetBal.subtract(matchedBal);
            //资产差额是负的且不属于兑付计划的移除
            if(divide.isNegtive()&&!allocLayerNos.contains(layerNo)){
                investStructureBalSum.subtractFrom(balMap.get(layerNo));
                balMap.remove(layerNo);
                iterator.remove();
                continue;
            }
            needUnMatchBalMap.put(layerNo,divide);
        }
        return needUnMatchBalMap;
    }

    /**
     * 进行解绑操作
     * @param capitalAstUnMatchPage
     * @param needUnMatchBal
     * @param acctDate
     * @return
     */
    private boolean unMatchCaptialAst(String planNo,CapitalAstUnMatchPage capitalAstUnMatchPage,Money needUnMatchBal,Date acctDate){
        boolean match = true;
        //增加Job控制
        JobControl jobControl = jobControlRepository.selectByJobName(capitalAssertMatchJob.getJobName());
        if(jobControl.getIsEff()== YNEnum.N){
            LogUtil.error(LOGGER, "计划[{0}]解绑失败，资金资产匹配任务运行中被关闭", planNo);
            return false;
        }
        capitalAstUnMatchPage = loanRepository.queryForCapitalAstUnMatch(capitalAstUnMatchPage);
        List<String> astNoList = capitalAstUnMatchPage.getDatas();
        for (String astNo : astNoList) {
            match=transactionTemplate.execute(status -> {
                //判断资金是否为负
                if(needUnMatchBal.isGreaterThanZero()){
                    return false;
                }

                Loan loan = loanAstAssembler.lockByAstNo(astNo, orgAcctDayComponent.getTrfAcctDay().getStandardDate());
                // 判断是否全部转让
                if (loan.getAstOrg().getAstOwnership().getOwnStatus() == OwnStatusEnum.TRANS_OUT) {
                    LogUtil.info(LOGGER, "[{0}]资产解绑跳过,[{1}]资产已全部转让,可能存在转让任务,跳过该资产", astNo, loan.getLastUpdateDate(), acctDate);
                    return false;
                }
                return this.unMatchCaptical(loan,needUnMatchBal,acctDate);
            });
           if(!match){
               break;
           }
        }
        return match;
    }

    /**
     * 进行解绑操作
     * @param loan
     * @param needUnMatchBal
     * @param acctDate
     * @return
     */
    private boolean unMatchCaptical(Loan loan,Money needUnMatchBal,Date acctDate){

        LoanAstGroup loanAstGroup = loan.filterLoanAstByOrg(loan.getAstOrg().getAstOwnership().getLenderOrgCode(), loan.getTermCnt());
        //计算资产额度
        Money loanAstBal = new Money();
        loanAstGroup.getLoanAsts().forEach(item -> {
            loanAstBal.addTo(item.getPrinBal());
        });

        // 更新目标匹配余额
        needUnMatchBal.addTo(loanAstBal);

        //创建交易详情
        String batchNo = SequenceUtil.genId(CAP_AST_UNMATCH);
        String timeStamp = TimeStampUtil.getDefaultTimeStamp();
        TransInfoTemplate transInTemplate = new TransInfoTemplate();
        transInTemplate.setTransTime(SystemDateUtil.getSystemDate());
        transInTemplate.setAccountDate(acctDate);
        transInTemplate.setEventCode(EventCodeEnum.CAP_AST_UNMATCH);
        transInTemplate.setOutBizNo(batchNo);
        transInTemplate.setBizNo(SequenceUtil.genSeqNo(TransCodeEnum.CAP_AST_UNMATCH));
        transInTemplate.setTimestamp(timeStamp);
        // 构造交易详情
        LoanTransDetail loanTransDetail = LoanTransDetail.createLoanAstTxnDtl(loan, transInTemplate);
        // 更新资金编号
        loanTransDetail.setCapitalInfo(CapitalAstMatchEnum.DEFAULT.getCode(),null);
        loanTransDetail.processBeforeUpdate();
        // 更新信息
        loanAstAssembler.store(loan, acctDate, false);
        List<LoanBak> loanBaks = LoanTransDetail.buildBaks(loan, transInTemplate.getAccountDate());
        loanRepository.backup(loanBaks);
        return true;
    }
    
    private CommonTask buildUnMatchMatchTask(String planNo,String bizNo){
        CommonTask commonTask=new CommonTask();
        commonTask.setContext(planNo);
        commonTask.setTaskStatus(TaskStatusEnum.IGNORE);
        commonTask.setTaskType(CommonTaskTypeEnum.UNMATCH_CAPTITALASSERT_MATCH);
        commonTask.setEnv(absConfig.getEnv());
        commonTask.setBizNo(bizNo);
        commonTask.setExecTimes(0L);
        return commonTask;
    }
    //生成解绑匹配业务码
    private static String generateBizNo(CommonTaskTypeEnum commonTaskTypeEnum,String planNo,Date accountDate,String allocNo){
        StringBuilder sb=new StringBuilder();
        sb.append(commonTaskTypeEnum.getCode());
        sb.append("-");
        sb.append(planNo);
        sb.append("-");
        sb.append(DateTimeUtil.formatYMD(accountDate));
        sb.append("-");
        sb.append(allocNo);
        return sb.toString();
    }
}