/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.biz.shared.handler;

import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAstBuyBackPack;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.domain.trade.AstTradeParam;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.ast.LoanAstAssembler;
import com.abssqr.plat.core.service.ast.LoanAstBuyBackManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 回购转让交易执行处理器
 * @author hanfei
 * @version $Id: LoanAstBuyBackHandler.java, v 0.1 2020-02-11 5:15 PM hanfei Exp $ 
 */
@Component
public class LoanAstBuyBackHandler {
    
    private final static Logger                LOGGER = LoggerFactory.getLogger(
            LoanAstBuyBackHandler.class);
    @Autowired
    private              LoanAstAssembler      loanAstAssembler;
    @Autowired
    private              LoanAstBuyBackManager loanAstBuyBackManager;
    /**
     *
     * @param trTask
     * @return
     */
    @PerformTrace
    public boolean astBuyBackExecute(TrTask trTask) {
        // 1 捞取加锁loan资产
        Loan loan = loanAstAssembler.lockByAstNo(trTask.getAstNo(), trTask.getTfrDt());
        if (loan == null) {
            LogUtil.warn(LOGGER, "[astBuyBackExecute][{0}]捞不到待转借据", trTask.getAstNo());
            return false;
        }
        // 查询资产包
        LoanAstBuyBackPack pack = loanAstBuyBackManager.getByPackCode(trTask.getTfrPack());
        VarChecker.checkNotNull(pack, "[astBuyBackExecute]-资产回购交易失败,没有对应的Pack");
        AstTradeParam astTrade = new AstTradeParam();
        astTrade.setPack(pack);
        astTrade.setLoan(loan);
        astTrade.setAstTradeType(AstPackTradeTypeEnum.BUY_BACK_BIZ);
        // 11 成功回购后，回购资产任务更新TASK_STATUS为success
        return loanAstBuyBackManager.astBuyBackTrade(trTask, astTrade);
    }
}
