/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.biz.shared.listener;

import com.abssqr.plat.common.facade.enums.PlanIndicatorEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanMonitor;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.repo.plan.impl.PlanMonitorRecordRepoImpl;
import com.abssqr.plat.common.model.repo.plan.impl.PlanMonitorRepoImpl;
import com.abssqr.plat.core.service.notice.AbssqrNoticeManager;
import com.abssqr.plat.core.service.util.RptBuilderUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author yangcheng
 * @version PlanMonitorRecordListener.java, v0.12019-07-0414:22 yangchengExp$
 */
@Component
public class PlanMonitorRecordListener {

    private static final Logger LOGGER = LoggerFactory.getLogger(PlanMonitorRecordListener.class);

    @Autowired
    private PlanMonitorRepoImpl       planMonitorRepo;
    @Autowired
    private PlanMonitorRecordRepoImpl planMonitorRecordRepo;
    @Autowired
    private AbssqrNoticeManager       abssqrNoticeManager;
    @Autowired
    private TransactionTemplate       transactionTemplate;

    /**
     * 更新每日监控事件触发情况
     *
     * @param orderEvent 监听事件
     */
    @EventListener
    public void planMonitorRecordListener(PlanMonitorRecordEvent orderEvent) {

        PlanReport planReport = orderEvent.getPlanReport();
        VarChecker.checkNotNull(planReport);
        String planNo = planReport.getPlanNo();
        List<PlanMonitor> planMonitorList = planMonitorRepo.getByPlan(planNo);
        if(CollectionUtils.isNotEmpty(planMonitorList)) {
            transactionTemplate.execute(status -> {
                for (PlanMonitor planMonitor : planMonitorList) {
                    //获取文件名,报表信息
                    String reportFieldCode = planMonitor.getIndicator().getReportFieldCode();

                    BigDecimal nowVal = RptBuilderUtil.getReportVal(planReport, reportFieldCode);
                    
                    // nowVal * 100
                    CalcUtil.calcPercRate(nowVal);
                    
                    LogUtil.info(LOGGER, "该计划{0}监控指标{1}检测", planMonitor.getPlanNo(),
                        planMonitor.getIndicator().getDesc());

                    if (this.allowInsert(planMonitor.getIndicator(), planMonitor.getValue(), nowVal)) {

                        LogUtil.info(LOGGER, "该计划{0}监控指标{1}为{2}已违标{3}", planMonitor.getPlanNo(),
                            planMonitor.getIndicator(),
                            planMonitor.getValue(), nowVal);

                        //监控信息表新插入
                        planMonitorRecordRepo.insert(planMonitor, nowVal);

                        //消息通知表新插入
                        this.insertMess(orderEvent.getPlanEntity() , planMonitor, nowVal);
                    }
                }
                return null;
            });
        }
    }

    /**
     * 插入消息通知
     *
     * @param param
     */
    private void insertMess(PlanBase planBase, PlanMonitor param, BigDecimal nowVal) {
        //插入消息通知数据
        String bizCode = param.getPlanNo();
        PlanIndicatorEnum bizType = param.getIndicator();
        String content = MessageUtil.formatMsg("指标[{1}] 事项名称[{0}] 标准为[{2}]%，当前为[{3}]%，{}",
                bizType.getDesc(), param.getName(), param.getValue(), nowVal, bizType.isTopLimit() ? "已超标" : "已不足");
        abssqrNoticeManager.sendBusinessNotice(planBase, bizCode, bizType, content, null);
    }

    /**
     * 上下限统一判断
     *
     * @param indicator
     * @param repoVal
     * @param nowVal
     * @return
     */
    private boolean allowInsert(PlanIndicatorEnum indicator, BigDecimal repoVal, BigDecimal nowVal) {
        return indicator.isTopLimit() ? repoVal.compareTo(nowVal) < 1 : repoVal.compareTo(nowVal) > -1;
    }
}
