/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.shared.report;

import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAstReportStat;
import com.general.system.tool.util.ToolUtil;

import java.math.BigDecimal;

/**
 * 统计金额字段计算工具
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.report: PlanAstStatAmtUtil.java, v 0.1 2019-06-28 16:22 zhenxuan.luo Exp $ 
 */
public class PlanAstStatAmtUtil {
    /**
     * 总发生额 = 本金总额 + 利息总额 + 罚息总额
     * @param stat
     * @return
     */
    public static BigDecimal getAll(PlanAstReportStat stat) {
        return getPrin(stat).add(getAllIntrWithPny(stat));
    }
    
    /**
     * 本金金额
     * @param stat
     * @return
     */
    public static BigDecimal getPrin(PlanAstReportStat stat) {
        return stat.getPrinAmt().add(stat.getOvdPrinAmt()).getAmount();
    }
    
    /**
     * 成本发生额
     * @param stat
     * @return
     */
    public static BigDecimal getCost(PlanAstReportStat stat) {
        return ToolUtil.isEmpty(stat.getCost()) ? BigDecimal.ZERO : stat.getCost().getAmount();
    }
    
    /**
     * 金融服务费发生额
     * @param stat
     * @return
     */
    public static BigDecimal getOrgDistPftAmt(PlanAstReportStat stat) {
        return ToolUtil.isEmpty(stat.getOrgDistpftAmt()) ? BigDecimal.ZERO : stat.getOrgDistpftAmt().getAmount();
    }
    
    /**
     * 金融服务费发生额
     * @param stat
     * @return
     */
    public static BigDecimal getOtherFeeAmt(PlanAstReportStat stat) {
        return ToolUtil.isEmpty(stat.getOtherFeeAmt()) ? BigDecimal.ZERO : stat.getOtherFeeAmt().getAmount();
    }
    
    /**
     * 本罚+息罚+费罚
     * @param stat
     * @return
     */
    public static BigDecimal getAllPny(PlanAstReportStat stat) {
        return stat.getPrinPenalAmt().add(stat.getIntrPenalAmt()).add(stat.getFeePenalAmt()).getAmount();
    }
    
    /**
     * 利息发生额+费用发生额
     * @param stat
     * @return
     */
    public static BigDecimal getAllIntr(PlanAstReportStat stat) {
        // 正常利息 + 逾期利息 + 费用
        return stat.getIntrAmt().add(stat.getOvdIntrAmt()).add(stat.getFeeAmt()).getAmount();
    }
    
    /**
     * 利息总额 + 罚息总额
     * @param stat
     * @return
     */
    public static BigDecimal getAllIntrWithPny(PlanAstReportStat stat) {
        return getAllIntr(stat).add(getAllPny(stat));
    }
}