package com.abssqr.plat.biz.shared.scheduler.job.capitalAssertMatch;

import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.enums.OrgTypeEnum;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import org.apache.commons.collections.CollectionUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 资金资产解绑任务生成。顺序，解绑->解绑匹配->同步匹配
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class CapitalAssertMatchJob extends BaseQuartzJob {

    @Autowired
    PlanRepo planRepo;

    @Autowired
    PlanAllocRepo planAllocRepo;

    @Autowired
    CommonTaskRepository commonTaskRepository;

    @Autowired
    JobControlRepository jobControlRepository;

    @Autowired
    AstSyncTaskCtrRepo astSyncTaskCtrRepo;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    /**
     * 间隔1分钟执行生成匹配和解绑任务
     */
    @Value("${sync.quartz.plan.capitalAssertMatch}")
    private String              cron;

    @Autowired
    private AbsConfig absConfig;

    @Autowired
    private PlanAcctTitleRepo planAcctTitleRepo;

    @Override
    protected String getCronExpr() {
        return cron;
    }


    @Override
    public void doExecute() {
        JobControl jobControl = jobControlRepository.lockByJobName(getJobName());
        if (jobControl == null || jobControl.getIsEff() == YNEnum.N) {
            return;
        }
        LogUtil.info(LOGGER, "[capitalAssetMatchJob] 生成资金资产解绑/匹配任务开始");
        Date execDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        List<PlanBase> plans = planRepo.getAllPlanAstByType(PlanTypeEnum.SPV.getCode());
        for (PlanBase planBase : plans) {
            if (planBase.getPlanType() == PlanTypeEnum.SPV) {
                OrganizationEntity plan = (OrganizationEntity)planRepo.getPlanByNo(planBase.getPlanNo());
                String planNo = plan.getPlanNo();

                List<CommonTask> initAndExecCommonTasks = commonTaskRepository.queryInitAndExecUnMatchOrMatchTaskByContext(planNo);
                if (CollectionUtils.isNotEmpty(initAndExecCommonTasks)) {
                    LogUtil.info(LOGGER, "[capitalAssetMatchJob]计划[{1}] 存在[初始化/正在执行中]的[资产匹配/解绑任务任务]", planNo);
                    continue;
                }
                //查询是否有解绑匹配的Task
                List<CommonTask> initAndIgnoreCommonTask = commonTaskRepository.queryIgnoreUnMatchOrMatchTaskByContext(planNo);
                List<CommonTask> toMatchedCapticalTask = initAndIgnoreCommonTask.stream()
                        .filter(v -> v.getTaskType() == CommonTaskTypeEnum.UNMATCH_CAPTITALASSERT_MATCH)
                        .collect(Collectors.toList());

                if (CollectionUtils.isNotEmpty(toMatchedCapticalTask)) {
                    CommonTask commonTask = toMatchedCapticalTask.get(0);
                    commonTaskRepository.updateIgnoreTaskStatus2Init(commonTask.getId());
                    LogUtil.info(LOGGER, "[capitalAssetMatchJob]计划[{1}] 存在状态为Ignore的解绑匹配任务[{2}]", planNo, commonTask.getBizNo());
                    continue;
                }

                //查询存在已审批通过未解绑的兑付计划
                List<PlanAlloc> planAllocs = planAllocRepo.queryUnMatchedApprovedAlloc(planNo);
                if (CollectionUtils.isNotEmpty(planAllocs)) {
                    PlanAlloc planAlloc = planAllocs.get(0);
                    LogUtil.info(LOGGER, "[capitalAssetMatchJob]计划[{1}] 存在已审批通过的兑付计划[{2}]", planNo, planAlloc.getAllocNo());
                    //生成解绑任务
                    CommonTask commonTask = buildUnMatchTask(planAlloc.getAllocNo(), planNo);
                    commonTaskRepository.create(commonTask);
                    continue;
                }

                //判断今日的模型转换是否已经完成
                AstSyncTaskCtrLog lastTaskLog = astSyncTaskCtrRepo.getTaskLogByExecDate(execDate, SyncTaskTypeEnum.AST_MOD, plan.getManagerOrgCode());
                if (lastTaskLog != null) {
                    String syncMatchBizNo = generateSyncMatchBizNo(CommonTaskTypeEnum.SYNC_CAPTITALASSERT_MATCH, execDate, planNo);
                    //查询今日是否已经生成了同步匹配任务
                    List<CommonTask> commonTasks = commonTaskRepository.queryByBizNo(syncMatchBizNo);
                    if (CollectionUtils.isNotEmpty(commonTasks)) {
                        continue;
                    }
                    //如果投资结构本机余额是0则不创建匹配任务
                    if (checkCanBuildMatchJob(planNo)) {
                        LogUtil.info(LOGGER, "[capitalAssetMatchJob]计划[{1}]生成同步匹配任务", planNo);
                        CommonTask commonTask = buildSyncMatchTask(syncMatchBizNo, planNo);
                        commonTaskRepository.create(commonTask);
                    }
                }
            }
        }
        LogUtil.info(LOGGER, "[capitalAssetMatchJob] 生成资金资产解绑/匹配任务结束");
    }

    private boolean checkCanBuildMatchJob(String planNo) {
        // 获取本金科目
        List<PlanAcctTitle> planAcctTitleList = planAcctTitleRepo.getByEntityNo(planNo);
        Money totalMoney = new Money();
        for (PlanAcctTitle planAcctTitle : planAcctTitleList) {
            if (planAcctTitle.getTitleType().isInvest()) {
                Money money = planAcctTitle.getBal().clone();
                totalMoney.addTo(money);
            }
        }
        if (!totalMoney.isGreaterThanZero()) {
            LOGGER.info("计划[{0}]投资结构本金总额小于等于0不进行资金资产匹配任务", planNo);
            return false;
        }
        return true;
    }

    private String generateSyncMatchBizNo(CommonTaskTypeEnum commonTaskTypeEnum, Date acctDate, String planNo) {
        StringBuilder sb = new StringBuilder();
        sb.append(commonTaskTypeEnum.getCode());
        sb.append("-");
        sb.append(planNo);
        sb.append("-");
        sb.append(DateTimeUtil.formatYMD(acctDate));
        return sb.toString();
    }

    private CommonTask buildUnMatchTask(String bizNo, String planNo) {
        CommonTask commonTask = buildBasicCommonTask(CommonTaskTypeEnum.CAPTITALASSERT_UNMATCH, planNo);
        commonTask.setBizNo(bizNo);
        return commonTask;
    }

    private CommonTask buildSyncMatchTask(String bizNo, String planNo) {
        CommonTask commonTask = buildBasicCommonTask(CommonTaskTypeEnum.SYNC_CAPTITALASSERT_MATCH, planNo);
        commonTask.setBizNo(bizNo);
        return commonTask;
    }

    private CommonTask buildBasicCommonTask(CommonTaskTypeEnum planTaskTypeEnum, String planNo) {
        CommonTask commonTask = new CommonTask();
        commonTask.setContext(planNo);
        commonTask.setTaskType(planTaskTypeEnum);
        commonTask.setTaskStatus(TaskStatusEnum.INIT);
        commonTask.setEnv(absConfig.getEnv());
        commonTask.setExecTimes(0L);
        return commonTask;
    }

}
