/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import cn.hutool.core.date.DateUtil;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.OrgAstStatStatusEnum;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.SystemDateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Set;

/**
 * 放款计划统计任务监控
 *
 * @author hanfei
 * @version $Id: PlanAstStatJobKeeper.java, v 0.1 2019-07-26 2:28 PM hanfei Exp $
 */
@Component
public class PlanAstStatJobKeeper extends AbstractKeeperNode {

    @Autowired
    private PlanAstStatTaskRepo planAstStatTaskRepo;


    @Override
    protected Void doHandle(Date date) {

        LogUtil.info(LOGGER, "放款计划统计任务监控-start");
        Date currentTime = getCurrentStanderDate();
        Date deadLine = DateUtil.offsetMinute(currentTime, -20).toJdkDate();
        Long count = planAstStatTaskRepo.countByStatusAndTime(deadLine, OrgAstStatStatusEnum.PROCESS.getCode());
        if (count != null && count > 0) {
            String msg = MessageUtil.formatMsg("放款计划统计任务告警,有[{0}]个任务超过10分钟未完成，请排查具体原因", count);
            // 获取用户
            Set<String> receivers = super.getAdminReceivers();
            sendSystemNotice("放款计划统计任务告警", "ALL", msg, receivers);
        }


        LogUtil.info(LOGGER, "放款计划统计任务监控-end");

        return null;
    }

    @Override
    public IEnum getBizType() {
        return AbssqrNoticeTypeEnum.PlanAstStat;
    }

    @Override
    protected String getWatchJob() {
        return "PlanAstStatTaskDispatcher";
    }


}
