/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.enums;

import com.general.system.common.model.IEnum;

import java.util.HashMap;
import java.util.Map;

/**
 * 资产风险等级枚举 —— 参考五级分类
 *
 * @author zhenxuan.luo
 * @version $Id: AstRiskClassEnum.java, v 0.1 2017-06-07 下午21:07 zhenxuan.luo Exp $
 */
public enum AstRiskClassEnum implements IEnum {
    
    NORMAL("1", "正常"),
    
    ATTENTION("2", "关注"),
    
    SECONDARY("3", "次级"),
    
    SUSPICIOUS("4", "可疑"),
    
    LOSS("5", "损失"),;
    
    private static final Map<String, AstRiskClassEnum> codeMap = new HashMap<String, AstRiskClassEnum>();
    
    static {
        for (AstRiskClassEnum item : AstRiskClassEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }
    
    private String code;
    private String desc;
    
    private AstRiskClassEnum(String code, String desc) {
        this.code = code;
        this.desc = desc;
    }
    
    public static AstRiskClassEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
        public String getDesc() {
        return desc;
    }
    
    /**
     * 五级分类风险比较
     *
     * @param target
     * @return
     */
    public boolean isMoreRiskThan(AstRiskClassEnum target) {
        int myInt = Integer.parseInt(this.getCode());
        int targetInt = Integer.parseInt(target.getCode());
        return myInt > targetInt;
    }
}