/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.param.plan;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import com.abssqr.plat.common.facade.base.BaseOprParam;
import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.param.acct.AccountParam;
import com.abssqr.plat.common.facade.param.plan.sub.FeeStructureParam;
import com.abssqr.plat.common.facade.param.plan.sub.InvestStructureParam;
import com.abssqr.plat.common.facade.param.plan.sub.PlanAstPriceRuleParam;
import com.abssqr.plat.common.facade.param.plan.sub.PlanTransferRuleParam;
import com.abssqr.plat.common.facade.validation.*;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.facade.param.plan: PlanOprParam.java, v 0.1 2019-05-28 15:24 bangis.wangdf Exp $
 */
public class PlanOprParam extends BaseOprParam {
    private static final long serialVersionUID = -2195239669227550002L;
    /**
     * 实体类型（SPV）
     */
    @VldEnums(enumClass = PlanTypeEnum.class)
    private String planType = PlanTypeEnum.PLAN.getCode();
    /**
     * planNo 计划编号.
     */
    @VldString(message = "计划编号不能为空")
    private String planNo;
    /**
     * planName 计划名称.
     */
    @VldString(message = "计划名称不能为空")
    private String planName;
    /**
     * totalAmount 总投入资本 发行总规模.
     */
    private String totalAmt = "0";
    /**
     * beginDate 计划设立日（信托设立日）.
     */
    private Date beginDate;
    /**
     * endDate 计划到期日（信托终止日）计划截止日.
     */
    private Date endDate;
    /**
     * 募集方式
     * PUBLIC
     * PRIVATE
     * 公募
     * 私募.
     */
    @VldEnums(enumClass = FundTypeEnum.class)
    private String fundType;
    /**
     * sponsorOrgCode 发起机构.
     */
    private String sponsorOrgCode;
    /**
     * managerOrgCode 代理机构 管理机构.
     */
    @VldString(message = "管理机构不能为空")
    private String planMgrOrgCode;
    /**
     * initDate 初始起算日.
     */
    private Date initCalcDate;
    /**
     * effectDate 计划起息日.
     */
    private Date effectDate;
    /**
     * firstPayDay 首次结算日.
     */
    private Date firstClearDate;
    /**
     * firstPayDay 首次偿付日 首次兑付日.
     */
    private Date firstPayDate;
    /**
     * 兑付频率
     * M
     * Q
     * 半年
     * Y
     * Month
     * Season
     * HalfYear
     * Year
     */
    @VldEnums(enumClass = PlanPayFreqEnum.class)
    private String payFreq;
    /**
     * 计息方式
     * ACT/360
     * ACT/365
     */
    @VldEnums(enumClass = CalcIntrTypeEnum.class)
    private String calcIntrType;
    /**
     * 增值税率
     */
    private BigDecimal vatRate;
    /**
     * 是否新增 true为新增 Flase为修改
     */
    @VldNotNull
    private Boolean isAdd;
    /**
     * 转让规则
     */
    private PlanTransferRuleParam transferRule;
    /**
     * 账户信息
     */
    @VldNotNull
    private List<AccountParam> accounts;
    /**
     * 投资结构
     */
    @VldCollection(nullable = true)
    private List<InvestStructureParam> investStructureList;
    /**
     * 费用结构 费用清单
     */
    @VldCollection(nullable = true)
    private List<FeeStructureParam> feeStructureList;

    /**
     * 定价规则类型
     */
    private String priceType;

    /**
     * 资产定价规则
     */
    private List<PlanAstPriceRuleParam> planAstPriceRuleList;

    /**
     * 是否是暂存
     */
    private Boolean draft = false;

    /**
     * 金融服务费率
     */
    @VldBigDecimal(message = "金融服务费率不能为空")
    private BigDecimal finSerFeeRate;

    @Override
    public EventCodeEnum getEventCode() {
        return EventCodeEnum.ADMIN_EDIT;
    }

    public String getPlanNo() {
        return planNo;
    }

    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }

    public String getPlanName() {
        return planName;
    }

    public void setPlanName(String planName) {
        this.planName = planName;
    }

    public String getTotalAmt() {
        return totalAmt;
    }

    public void setTotalAmt(String totalAmt) {
        this.totalAmt = totalAmt;
    }

    public Date getBeginDate() {
        return beginDate;
    }

    public void setBeginDate(Date beginDate) {
        this.beginDate = beginDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getFundType() {
        return fundType;
    }

    public void setFundType(String fundType) {
        this.fundType = fundType;
    }

    public String getSponsorOrgCode() {
        return sponsorOrgCode;
    }

    public void setSponsorOrgCode(String sponsorOrgCode) {
        this.sponsorOrgCode = sponsorOrgCode;
    }

    public String getPlanMgrOrgCode() {
        return planMgrOrgCode;
    }

    public void setPlanMgrOrgCode(String planMgrOrgCode) {
        this.planMgrOrgCode = planMgrOrgCode;
    }

    public Date getInitCalcDate() {
        return initCalcDate;
    }

    public void setInitCalcDate(Date initCalcDate) {
        this.initCalcDate = initCalcDate;
    }

    public Date getEffectDate() {
        return effectDate;
    }

    public void setEffectDate(Date effectDate) {
        this.effectDate = effectDate;
    }

    public Date getFirstClearDate() {
        return firstClearDate;
    }

    public void setFirstClearDate(Date firstClearDate) {
        this.firstClearDate = firstClearDate;
    }

    public Date getFirstPayDate() {
        return firstPayDate;
    }

    public void setFirstPayDate(Date firstPayDate) {
        this.firstPayDate = firstPayDate;
    }

    public PlanTransferRuleParam getTransferRule() {
        return transferRule;
    }

    public void setTransferRule(PlanTransferRuleParam transferRule) {
        this.transferRule = transferRule;
    }

    public List<AccountParam> getAccounts() {
        return accounts;
    }

    public void setAccounts(List<AccountParam> accounts) {
        this.accounts = accounts;
    }

    public List<InvestStructureParam> getInvestStructureList() {
        return investStructureList;
    }

    public void setInvestStructureList(List<InvestStructureParam> investStructureList) {
        this.investStructureList = investStructureList;
    }

    public List<FeeStructureParam> getFeeStructureList() {
        return feeStructureList;
    }

    public void setFeeStructureList(List<FeeStructureParam> feeStructureList) {
        this.feeStructureList = feeStructureList;
    }

    public String getPayFreq() {
        return payFreq;
    }

    public void setPayFreq(String payFreq) {
        this.payFreq = payFreq;
    }

    public String getCalcIntrType() {
        return calcIntrType;
    }

    public void setCalcIntrType(String calcIntrType) {
        this.calcIntrType = calcIntrType;
    }

    public BigDecimal getVatRate() {
        return vatRate;
    }

    public void setVatRate(BigDecimal vatRate) {
        this.vatRate = vatRate;
    }

    public Boolean getAdd() {
        return isAdd;
    }

    public void setAdd(Boolean add) {
        isAdd = add;
    }

    public String getPlanType() {
        return planType;
    }

    public void setPlanType(String planType) {
        this.planType = planType;
    }

    public String getPriceType() {
        return priceType;
    }

    public void setPriceType(String priceType) {
        this.priceType = priceType;
    }

    public List<PlanAstPriceRuleParam> getPlanAstPriceRuleList() {
        return planAstPriceRuleList;
    }

    public void setPlanAstPriceRuleList(List<PlanAstPriceRuleParam> planAstPriceRuleList) {
        this.planAstPriceRuleList = planAstPriceRuleList;
    }

    public Boolean getDraft() {
        return draft;
    }

    public void setDraft(Boolean draft) {
        this.draft = draft;
    }

    public BigDecimal getFinSerFeeRate() {
        return finSerFeeRate;
    }

    public void setFinSerFeeRate(BigDecimal finSerFeeRate) {
        this.finSerFeeRate = finSerFeeRate;
    }
}