/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanFeeStructureDO;
import com.abssqr.plat.common.facade.enums.CalcTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeScaleTypeEnum;
import com.abssqr.plat.common.facade.enums.FeeTimeTypeEnum;
import com.abssqr.plat.common.facade.enums.FinSerFeeTypeEnum;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;

/**
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.convertor: PlanFeeStructureConvert.java, v 0.1 2019-05-31 21:35 bangis.wangdf Exp $ 
 */
public class PlanFeeStructureConvert {


    public static List<FeeStructure> convert2FeeStructureDomainList(List<PlanFeeStructureDO> feeStructureDOS) {
        List<FeeStructure> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(feeStructureDOS)) {
            feeStructureDOS.forEach(planFeeStructureDO -> {
                result.add(convert2FeeStructureDomain(planFeeStructureDO));
            });
        }
        return result;
    }

    private static FeeStructure convert2FeeStructureDomain(PlanFeeStructureDO param) {
        if (param == null){
            return null;
        }
        FeeStructure result = new FeeStructure();
        //fixedFeeAmt 固定费用
        result.setFixedFeeAmt(Money.createWithCentNull(param.getFixedFeeAmt()));
        //payLimitAmt 支付上限
        result.setPayLimitAmt(Money.createWithCentNull(param.getPayLimitAmt()));
        //feeNo 费用编号
        result.setFeeNo(param.getFeeNo());
        //feeName 费用名称
        result.setFeeName(param.getFeeName());
        //feeCalcType 计算方式
        result.setFeeCalcType(CalcTypeEnum.getByCode(param.getFeeCalcType()));
        //feeTimeType 计费时间类型
        result.setFeeTimeType(FeeTimeTypeEnum.getByCode(param.getFeeTimeType()));
        //feeScaleType 费率基准

        result.setFeeScaleType(FeeScaleTypeEnum.getByCode(param.getFeeScaleType()));
        //calcByDay 按日计费
        result.setCalcByDay(param.getCalcByDay());
        //feeRate 费率
        result.setFeeRate(param.getFeeRate());
        result.setFinSerFeeType(FinSerFeeTypeEnum.getByCode(param.getFinSerFeeType()));
        return result;
    }

    public static List<PlanFeeStructureDO> convert2Dos(String planNo,
                                                                     List<FeeStructure> feeStructureList) {
        if (CollectionUtils.isEmpty(feeStructureList)) {
            return null;
        }
        List<PlanFeeStructureDO> result = Lists.newArrayList();
        feeStructureList.forEach(param -> {
            result.add(convert2Do(planNo, param));
        });
        return result;
    }

    private static PlanFeeStructureDO convert2Do(String planNo, FeeStructure param) {
        PlanFeeStructureDO result = new PlanFeeStructureDO();
        
        //fixedFeeAmt 固定费用
        if(param.getFixedFeeAmt()!=null) {
            result.setFixedFeeAmt(param.getFixedFeeAmt().getCent());
        }
        //payLimitAmt 支付上限
        if(param.getPayLimitAmt()!=null) {
            result.setPayLimitAmt(param.getPayLimitAmt().getCent());
        }
    
        if (StringUtils.isBlank(param.getFeeNo())) {
            param.setFeeNo(SequenceUtil.genId(IdTypeEnum.FEE));
        }
        //feeNo 费用编号
        result.setFeeNo(param.getFeeNo());
        
        //planNo 计划编号
        result.setPlanNo(planNo);
        //feeName 费用名称
        result.setFeeName(param.getFeeName());
        //feeCalcType 计算方式
        if(param.getFeeCalcType()!=null) {
            result.setFeeCalcType(param.getFeeCalcType().getCode());
        }
        //feeTimeType 计费时间类型
        if(param.getFeeTimeType()!=null) {
            result.setFeeTimeType(param.getFeeTimeType().getCode());
        }
        //feeScaleType 费率基准
        if(param.getFeeScaleType()!=null) {
            result.setFeeScaleType(param.getFeeScaleType().getCode());
        }
        //calcByDay 按日计费
        result.setCalcByDay(param.getCalcByDay());
        //feeRate 费率
        result.setFeeRate(param.getFeeRate());
        if(param.getFinSerFeeType()!=null) {
            result.setFinSerFeeType(param.getFinSerFeeType().getCode());
        }
        return result;
    }

    
}
