/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.domain.ast;

import com.abssqr.plat.common.facade.base.CreationAwareEntity;
import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.model.domain.contract.LoanContract;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.model.utils.DebtMoneyUtil;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 贷款资产信息
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.asset: Loan.java, v 0.1 2017-12-12 下午2:48 guozhenhua Exp $
 */
public class Loan extends CreationAwareEntity {

    /**
     * 贷款资产日期
     */
    final private AstPeriod    astPeriod = new AstPeriod();
    /**
     * 资产机构信息
     */
    final private AstOrg       astOrg    = new AstOrg();
    /**
     *
     */
    private       Long         transCnt  = 0L;
    /**
     * 贷款合同
     */
    private       LoanContract contract;
    /**
     * 资产ID
     */
    private       String       astNo;
    /**
     * 用户ID
     */
    private       String       userId;
    /**
     *
     */
    private       String       certNo;
    /**
     * 外部传入的资产唯一编号
     */
    private       String       outAstNo;
    /**
     * 合同编号
     */
    private       String       contNo;
    /**
     * 内部产品编码
     */
    private       String       prodNo;
    /**
     * 外部产品产品
     */
    private       String       outProdNo;
    /**
     * 所属资产包
     */
    private       String       ownPack;
    /**
     * 借据当前所在的期次
     */
    private       Integer      curTerm;
    /**
     * 总期数
     */
    private       Integer      termCnt;
    /**
     * 债类的金额字段
     */
    private       DebtMoney    bal       = new DebtMoney();
    /**
     * 已还的金额字段
     */
    private       DebtMoney    paid      = new DebtMoney();

    /**
     * 状态
     */
    private LoanStatusEnum status;

    /**
     * 风险信息
     */
    private LoanRisk loanRisk = new LoanRisk();

    /**
     * 最近数据发生日(yyyyMMdd)
     */
    private Date lastUpdateDate;

    /**
     * capitalNo 资金编号.
     */
    private String capitalNo;
    /**
     * layerName 投资结构名称.
     */
    private String layerName;
    /**
     * extInf 扩展信息.
     */
    private Map<String,String> extInf = Maps.newHashMap();

    private List<LoanInstalDomain> loanInstalList = Lists.newArrayList();

    private List<LoanTransDetail> loanTransDetailList = Lists.newArrayList();

    private LoanTransDetail activeTransDtl;

    public Loan() {
        super(true);
    }

    public Loan(boolean newModel) {
        super(newModel);
    }

    public Loan cloneSimple() {
        Loan newLoan = new Loan();
        newLoan.setAstNo(this.astNo);
        newLoan.setUserId(this.userId);
        newLoan.setCertNo(this.certNo);
        newLoan.setOutAstNo(this.outAstNo);
        newLoan.setContNo(this.contNo);
        newLoan.setProdNo(this.prodNo);
        newLoan.setOutProdNo(this.outProdNo);
        newLoan.setCurTerm(this.curTerm);
        newLoan.setTermCnt(this.termCnt);
        newLoan.getLoanRisk().cloneTo(this.loanRisk);
        newLoan.getAstOrg().cloneTo(this.astOrg);
        newLoan.getAstPeriod().cloneTo(this.astPeriod);
        newLoan.setLastUpdateDate(this.getLastUpdateDate());
        newLoan.setCapitalNo(this.capitalNo);
        newLoan.setLayerName(this.layerName);
        newLoan.setExtInf(this.extInf);
        return newLoan;
    }
    
    /**
     * 根据所属机构 和最大期此号获取机构资产分组
     * @param ownOrg
     * @param maxTermNo
     * @return
     */
    public LoanAstGroup filterLoanAstByOrg(String ownOrg, int maxTermNo) {
        LoanAstGroup loanAstGroup = new LoanAstGroup();

        for (LoanInstalDomain loanInstalDomain : loanInstalList) {
            if (!loanInstalDomain.getStatus().isActive()) {
                continue;
            }
            if (loanInstalDomain.getTermNo() > maxTermNo) {
                loanAstGroup.setAllAst(false);
                continue;
            }
            LoanAstGroup instalAstGroup = loanInstalDomain.filterLoanAst(ownOrg);
            if (CollectionUtils.isEmpty(instalAstGroup.getLoanAsts())) {
                // 前面已经判断了分期的状态，因此因为分期结清导致无法筛选出可用机构资产的场景不成立。
                // 所以，如果到此处无法筛选出可用机构资产，那一定是因为该分期的机构资产已经转让给其他计划，存在部分转让。
                loanAstGroup.setAllAst(false);
                continue;
            } else {
                loanAstGroup.merge(instalAstGroup);
            }
        }

        return loanAstGroup;
    }
    
    /**
     * 根据最小期次号
     * @param minTermNo
     * @return
     */
    public LoanAstGroup filterLaterLoanAstByTermNo(int minTermNo) {
        LoanAstGroup loanAstGroup = new LoanAstGroup();

        for (LoanInstalDomain loanInstalDomain : loanInstalList) {
            if (!loanInstalDomain.getStatus().isActive()) {
                continue;
            }
            if (loanInstalDomain.getTermNo() < minTermNo) {
                loanAstGroup.setAllAst(false);
                continue;
            }
            loanAstGroup.merge(loanInstalDomain.buildAstGroup());
        }

        return loanAstGroup;
    }

    public void checkSelf(Date accountDate) {

        if (this.status == LoanStatusEnum.REVERSE) {
            return;
        }

        DebtMoney bal = new DebtMoney();
        DebtMoney paid = new DebtMoney();

        boolean isAllClear = true;
        boolean isOvd = false;
        Date lastEndDate = null;

        Collections.sort(this.loanInstalList, Comparator.comparing(LoanInstal::getTermNo));
        for (LoanInstalDomain instal : this.getLoanInstalList()) {
            paid.addAll(instal.getPaidAmt());

            instal.checkSelf(accountDate);
            DebtMoneyUtil.addInstal2Loan(bal, instal.getBal(), instal.getStatus());

            if (instal.getStatus() == LoanStatusEnum.OVERDUE) {
                isOvd = true;
            }

            isAllClear = isAllClear && instal.getStatus() == LoanStatusEnum.CLEAR;

            if (0 == instal.getTermNo()) {
                continue;
            }
            if (null != lastEndDate) {
                VarChecker
                        .checkArgument(DateTimeUtil.calcDayDiff(instal.getAstPeriod().getStartDate(), lastEndDate) == 1,
                                "[loanCheck]分期[{0}-{1}]到期日[{2}]和上一分期[{3}]不连续", instal.getAstNo(), instal.getTermNo(),
                                instal.getAstPeriod().getStartDate(), lastEndDate);
            }
            lastEndDate = instal.getAstPeriod().getEndDate();
        }

        VarChecker.checkEquals(bal, this.bal, "[loanCheck]贷款[{0}]余额[{1}]和分期余额[{2}]汇总不相等", this.astNo, this.bal, bal);
        VarChecker
                .checkEquals(paid, this.paid, "[loanCheck]贷款[{0}]已还[{1}]和分期已还[{2}]汇总不相等", this.astNo, this.paid, paid);

        if (isOvd) {
            VarChecker.checkArgument(LoanStatusEnum.OVERDUE == this.status, "[loanCheck]贷款[{0}]状态应该为逾期 - [{1}] - [{2}]",
                    this.astNo, this.status, this.outAstNo);
        } else {
            VarChecker.checkArgument(LoanStatusEnum.OVERDUE != this.status, "[loanCheck]贷款[{0}]状态不应该为逾期 - [{1}] - [{2}]",
                    this.astNo, this.status, this.outAstNo);
        }

        if (isAllClear) {
            VarChecker
                    .checkArgument(LoanStatusEnum.CLEAR == this.status, "[loanCheck]贷款[{0}]状态应该为结清 - [{1}] - [{2}]", this.astNo,
                            this.status, this.outAstNo);
        } else {
            VarChecker
                    .checkArgument(LoanStatusEnum.CLEAR != this.status, "[loanCheck]贷款[{0}]状态不应该为结清 - [{1}] - [{2}]",
                            this.astNo,
                            this.status, this.outAstNo);
        }

    }

    public List<LoanInstalDomain> getInstalsByTermNo(int termNo) {
        if (termNo == 0) {
            return loanInstalList;
        }

        List<LoanInstalDomain> res = Lists.newArrayList();

        for (LoanInstalDomain loanInstal : loanInstalList) {
            if (loanInstal.getTermNo() == termNo) {
                res.add(loanInstal);
            }
        }

        return res;
    }

    public void addLoanTransDtl(LoanTransDetail loanTransDetail) {
        this.loanTransDetailList.add(loanTransDetail);
    }

    public void fillInfoByInstalls() {
        DebtMoney totalPaid = new DebtMoney();
        for (LoanInstal loanInstal : this.loanInstalList) {
            totalPaid.addAll(loanInstal.getPaidAmt());
        }

        this.setPaid(totalPaid);
    }

    public Map<Integer, LoanInstalDomain> getLoanInstalMap() {
        return this.loanInstalList.stream().collect(Collectors.toMap(LoanInstal::getTermNo, item -> item));
    }

    public Map<Integer, BigDecimal> getPrinDist() {
        Map<Integer, BigDecimal> ret = Maps.newHashMap();

        // 不做机构合并，可能存在一个机构持有两个机构资产
        for (LoanInstal instal : loanInstalList) {
            if (instal.getStatus() == LoanStatusEnum.CLEAR) {
                continue;
            }
            ret.put(instal.getTermNo(), instal.getBal().getAllPrin().getAmount());
        }

        return ret;
    }

    public Long nextTransCnt() {
        transCnt = transCnt + 1;
        if (transCnt >= 999999) {
            transCnt = 0L;
        }
        return transCnt;
    }

    public Long getTransCnt() {
        return transCnt;
    }

    public void setTransCnt(Long transCnt) {
        this.transCnt = transCnt;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getCertNo() {
        return certNo;
    }

    public void setCertNo(String certNo) {
        this.certNo = certNo;
    }

    public String getAstNo() {
        return astNo;
    }

    public void setAstNo(String astNo) {
        this.astNo = astNo;
    }

    public String getOutAstNo() {
        return outAstNo;
    }

    public void setOutAstNo(String outAstNo) {
        this.outAstNo = outAstNo;
    }

    public String getContNo() {
        return contNo;
    }

    public void setContNo(String contNo) {
        this.contNo = contNo;
    }

    public String getProdNo() {
        return prodNo;
    }

    public void setProdNo(String prodNo) {
        this.prodNo = prodNo;
    }

    public String getOwnPack() {
        return ownPack;
    }

    public void setOwnPack(String ownPack) {
        this.ownPack = ownPack;
    }

    public AstPeriod getAstPeriod() {
        return astPeriod;
    }

    public AstOrg getAstOrg() {
        return astOrg;
    }

    public Integer getCurTerm() {
        return curTerm;
    }

    public void setCurTerm(Integer curTerm) {
        this.curTerm = curTerm;
    }

    public Integer getTermCnt() {
        return termCnt;
    }

    public void setTermCnt(Integer termCnt) {
        this.termCnt = termCnt;
    }

    public DebtMoney getBal() {
        return bal;
    }

    public void setBal(DebtMoney bal) {
        this.bal = bal;
    }

    public DebtMoney getPaid() {
        return paid;
    }

    public void setPaid(DebtMoney paid) {
        this.paid = paid;
    }

    public LoanStatusEnum getStatus() {
        return status;
    }

    public void setStatus(LoanStatusEnum status) {
        this.status = status;
    }

    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }

    public void setLastUpdateDate(Date lastUpdateDate) {
        this.lastUpdateDate = lastUpdateDate;
    }

    public String getOutProdNo() {
        return outProdNo;
    }

    public void setOutProdNo(String outProdNo) {
        this.outProdNo = outProdNo;
    }

    public boolean isTransfered() {
        return astOrg.getAstOwnership().isTransfered();
    }

    public List<LoanInstalDomain> getLoanInstalList() {
        return loanInstalList;
    }

    public void setLoanInstalList(List<LoanInstalDomain> loanInstalList) {
        this.loanInstalList = loanInstalList;
    }

    public List<LoanTransDetail> getLoanTransDetailList() {
        return loanTransDetailList;
    }

    public LoanRisk getLoanRisk() {
        return loanRisk;
    }

    public void setLoanRisk(LoanRisk loanRisk) {
        this.loanRisk = loanRisk;
    }

    public LoanTransDetail getActiveTransDtl() {
        return activeTransDtl;
    }

    public void setActiveTransDtl(LoanTransDetail activeTransDtl) {
        this.activeTransDtl = activeTransDtl;
    }

    public LoanContract getContract() {
        return contract;
    }

    public void setContract(LoanContract contract) {
        this.contract = contract;
    }

    /**
     * 获取未转让余额
     *
     * @return
     */
    public DebtMoney getNoOutBal() {
        DebtMoney outOutBal = new DebtMoney();

        if (this.status == LoanStatusEnum.REVERSE) {
            return outOutBal;
        }

        List<LoanInstal> noOutInstalList = this.loanInstalList.stream().filter(
                loanInstal -> !loanInstal.getAstOrg().getAstOwnership().isTransfered()).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(noOutInstalList)) {
            for (LoanInstal loanInstal : noOutInstalList) {
                DebtMoneyUtil.addInstal2Loan(outOutBal, loanInstal.getBal(), loanInstal.getStatus());
            }
        }
        return outOutBal;
    }

    public String getCapitalNo() {
        return capitalNo;
    }

    public void setCapitalNo(String capitalNo) {
        this.capitalNo = capitalNo;
    }

    public String getLayerName() {
        return layerName;
    }

    public void setLayerName(String layerName) {
        this.layerName = layerName;
    }


    public Map<String, String> getExtInf() {
        return extInf;
    }

    public void setExtInf(Map<String, String> extInf) {
        this.extInf = extInf;
    }
}
