/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.rule;

import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.model.rule.AstPriceRule;
import com.abssqr.plat.common.facade.model.rule.TradePriceRule;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.google.common.collect.Lists;

import java.util.List;

/**
 * 资产定价计算组件参数模型
 * @author hanfei
 * @version $Id: LoanAstPriceCalcParam.java, v 0.1 2020-02-07 6:10 PM hanfei Exp $
 */
public class LoanAstPriceCalcParam<PriceRule extends TradePriceRule> {
    /**
     * 转入机构
     */
    private String              transIn;
    /**
     * 转出机构
     */
    private String              transOut;
    /**
     * 机构资产
     */
    private LoanAst             loanAst;
    /**
     * 计价规则
     */
    private List<PriceRule>     tradePriceRules = Lists.newArrayList();
    /**
     * 交易类型
     */
    private AstPackTradeTypeEnum astTradeType;
    /**
     * 获取对应规则条件
     */
    private Comparable           condition;
    
    /**
     * 根据回购任务创建定价规则
     * @param task
     * @param abssqrProdCode
     */
    public void addPriceRuleByTask(TrTask task, String abssqrProdCode) {
        AstPriceRule priceRule = new AstPriceRule();
        priceRule.setRiskCl(task.getRiskCl());
        priceRule.setAbssqrProdCode(abssqrProdCode);
        priceRule.setIntrRate(task.getIntrRate());
        priceRule.setPrinRate(task.getPrinRate());
        priceRule.setPenalFeeRate(task.getPenalFeeRate());
        tradePriceRules.add((PriceRule) priceRule);
    }
    
    
    public LoanAst getLoanAst() {
        return loanAst;
    }
    
    public void setLoanAst(LoanAst loanAst) {
        this.loanAst = loanAst;
    }
    
    public List<PriceRule> getTradePriceRules() {
        return tradePriceRules;
    }
    
    public void setTradePriceRules(List<PriceRule> tradePriceRules) {
        this.tradePriceRules = tradePriceRules;
    }
    
    public AstPackTradeTypeEnum getAstTradeType() {
        return astTradeType;
    }
    
    public void setAstTradeType(AstPackTradeTypeEnum astTradeType) {
        this.astTradeType = astTradeType;
    }
    
    public String getTransIn() {
        return transIn;
    }
    
    public void setTransIn(String transIn) {
        this.transIn = transIn;
    }
    
    public String getTransOut() {
        return transOut;
    }
    
    public void setTransOut(String transOut) {
        this.transOut = transOut;
    }
    
    public Comparable getCondition() {
        return condition;
    }
    
    public void setCondition(Comparable condition) {
        this.condition = condition;
    }
    
}
