/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.ast;

import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.CapitalAstUnMatchPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanBakListPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.LoanListPage;
import com.abssqr.plat.common.facade.param.loan.LoanPageQryParam;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanBak;

import java.util.Date;
import java.util.List;

/**
 * 账单仓储
 *
 * @author guozhenhua
 * @version com.abssqr.plat.processor.core.service.loanTrans.impl: AstBillRepositoryImpl.java, v 0.1 2017-12-13 下午1:51 guozhenhua Exp $
 */
public interface LoanRepository {
    
    /**
     * 根据astNo上锁
     *
     * @param astNo
     * @return
     */
    Loan lockByAstNo(String astNo);
    
    /**
     * 根据astNo查询
     *
     * @param astNo
     * @return
     */
    Loan getByAstNo(String astNo);
    
    /**
     * 根据主键查询账单并上锁
     *
     * @param outAstNo   外部资产编号
     * @param managerOrgCode 管理机构号
     * @return
     */
    Loan lockByOutAstNo(String outAstNo, String managerOrgCode);
    
    /**
     * 根据主键查询账单
     *
     * @param outAstNo   外部资产编号
     * @param managerOrgCode 管理机构号
     * @return
     */
    Loan getByOutAstNo(String outAstNo, String managerOrgCode);
    
    /**
     * 创建资产
     *
     * @param loan
     */
    int createOrUpdate(Loan loan);
    
    /**
     * 记录bak镜像
     * @param bakList
     * @return
     */
    int backup(List<LoanBak> bakList);
    
    /**
     * 根据资产包分页查询资产
     *
     * @param loanPageQryParam
     * @return
     */
    LoanBakListPage getPageByOwnOrg(LoanPageQryParam loanPageQryParam, Date accountDate);
    /**
     * 根据资产包分页查询资产
     *
     * @param page
     * @return
     */
    LoanBakListPage getPageByOwnOrg(LoanBakListPage page);

    /**
     * 根据原始放款机构查询资产列表
     * @param param
     * @return
     */
    LoanListPage queryLoanPage(LoanPageQryParam param);

    /**
     * 资金资产匹配分页
     * @param capitalAstMatch 资产编号
     * @return
     */
    CapitalAstMatchPage queryForCapitalAstMatch(CapitalAstMatchPage capitalAstMatch);


    /**
     * 资金资产解绑分页
     * @param capitalAstUnBind 资产编号
     * @return
     */
    CapitalAstUnMatchPage queryForCapitalAstUnMatch(CapitalAstUnMatchPage capitalAstUnBind);
}
