/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.holiday;

import com.abssqr.plat.common.dal.mysql.auto.dao.OrgHolidayDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.OrgHolidayDO;
import com.abssqr.plat.common.facade.model.holiday.Holiday;
import com.abssqr.plat.common.model.convertor.HolidayConvert;
import com.abssqr.plat.common.util.utils.BeanCopierUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @author yaphet
 * @version com.abssqr.plat.common.model.repo.holiday: HolidayRepositoryImpl.java, v 0.1 2018-06-27 下午3:39 yaphet Exp $
 */
@Repository
public class HolidayRepositoryImpl implements HolidayRepository {

    @Autowired
    private OrgHolidayDAO orgHolidayDAO;

    /**
     * 查询日期列表
     *
     * @param orgCode
     * @param beginDate
     * @param endDate
     * @return
     */
    @Override
    public List<Holiday> queryHolidayByOrgCodeAndDate(String orgCode, String beginDate, String endDate) {
        List<OrgHolidayDO> orgHolidayDOList = orgHolidayDAO.queryHolidayByOrgCodeAndDate(endDate, orgCode, beginDate);
        List<Holiday> holiday = BeanCopierUtil.copyList(Holiday.class, orgHolidayDOList);
        return holiday;
    }

    /**
     * 新增节假日信息
     *
     * @param holiday
     */
    @Override
    public void addHoliday(Holiday holiday) {
        OrgHolidayDO orgHolidayDO = HolidayConvert.holiday2DO(holiday);
        orgHolidayDAO.insert(orgHolidayDO);
    }

    /**
     * 更新节假日信息
     *
     * @param holiday
     */
    @Override
    public void updateHoliday(Holiday holiday) {
        OrgHolidayDO orgHolidayDO = HolidayConvert.holiday2DO(holiday);
        orgHolidayDAO.updateByOrgCodeAndDate(orgHolidayDO);
    }

    /**
     * 查询一个日期信息是否存在
     *
     * @param orgCode
     * @param date
     * @return
     */
    @Override
    public Boolean existHoliday(String orgCode, String date) {
        OrgHolidayDO orgHolidayDO = orgHolidayDAO.getByOrgCodeAndDate(orgCode, date);
        return null == orgHolidayDO ? Boolean.FALSE : Boolean.TRUE;
    }

    /**
     * 查询一个日期类型
     *
     * @param orgCode
     * @param date
     * @return
     */
    @Override
    public Holiday getHolidayType(String orgCode, String date) {
        OrgHolidayDO orgHolidayDO = orgHolidayDAO.getByOrgCodeAndDate(orgCode, date);
        return BeanCopierUtil.copy(Holiday.class, orgHolidayDO);
    }
}
