/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.repo.plan.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanCapitalOrderDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanCapitalOrderDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanCapitalOrderPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.CapitalOrderAmt;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.model.convertor.PlanCapitalOrderConvertor;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 资金订单
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.repo.plan.impl: PlanCapitalOrderRepoImpl.java, v 0.1 2019-06-03 16:24 zhenxuan.luo Exp $
 */
@Component
public class PlanCapitalOrderRepoImpl implements PlanCapitalOrderRepo {
    
    @Autowired
    private PlanCapitalOrderDAO planCapitalOrderDAO;
    
    @Override
    public int insert(PlanCapitalOrder entity) {
        if (entity == null) {
            return 0;
        }
        return planCapitalOrderDAO.insert(PlanCapitalOrderConvertor.convert2DO(entity));
    }
    
    @Override
    public int insertBatch(List<PlanCapitalOrder> list) {
        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }
        
        List<PlanCapitalOrderDO> doList = Lists.newArrayList();
        for (PlanCapitalOrder planCapitalOrder : list) {
            doList.add(PlanCapitalOrderConvertor.convert2DO(planCapitalOrder));
        }
        return planCapitalOrderDAO.insertBatch(doList);
    }
    
    @Override
    public int insertHis(PlanCapitalOrder entity) {
        if (entity == null) {
            return 0;
        }
        return planCapitalOrderDAO.insertHis(PlanCapitalOrderConvertor.convert2DO(entity));
    }
    
    @Override
    public int updateProcess(PlanCapitalOrder planCapitalOrder) {
        return planCapitalOrderDAO.updateProcess(PlanCapitalOrderConvertor.convert2DO(planCapitalOrder));
    }
    
    @Override
    public int deleteBySeqno(String seqNo, String planNo) {
        return planCapitalOrderDAO.deleteBySeqno(seqNo, planNo);
    }
    
    @Override
    public PlanCapitalOrder lockBySeqNo(String seqNo) {
        return PlanCapitalOrderConvertor.convert2Domain(planCapitalOrderDAO.lockBySeqno(seqNo));
    }
    
    @Override
    public PlanCapitalOrder lockActiveByPlanNo(Date accountDate, String planNo,
                                               PlanCapitalTransTypeEnum transType) {
        return PlanCapitalOrderConvertor.convert2Domain(
                planCapitalOrderDAO.lockActiveByPlanNo(accountDate, planNo, transType.getCode()));
    }
    
    @Override
    public List<PlanCapitalOrder> getInitListByPlanNo(String planNo) {
        List<PlanCapitalOrderDO> ordrDOS = planCapitalOrderDAO.getInitListByPlanNo(planNo);
        return PlanCapitalOrderConvertor.convert2Domains(ordrDOS);
        
    }
    
    /**
     * 根据条件判断当前支付单数量
     * @param accountDate
     * @param planNo
     * @param transType
     * @param statusEnums
     * @return
     */
    @Override
    public Long getActiveCntByCondition(Date accountDate,String planNo,PlanCapitalTransTypeEnum transType, List<TaskStatusEnum> statusEnums) {
        List<String> list = ToolUtil.isEmpty(statusEnums) ? null : statusEnums.stream().map(TaskStatusEnum::getCode).collect(Collectors.toList());
        String type = transType == null ? null : transType.getCode();
        return planCapitalOrderDAO.getActiveCntByCondition(accountDate, planNo, type, list);
    }

    /**
     * desc:根据状态分页查询订单. 默认小于等于当前系统时间的交易单
     * @param planCapitalOrder planCapitalOrder
     * @return PlanCapitalOrderPage
     */
    @Override
    public PlanCapitalOrderPage getCapitalOrderByStatus(PlanCapitalOrderPage planCapitalOrder){
        return planCapitalOrderDAO.getCapitalOrderByStatus(planCapitalOrder);
    }

    @Override
    public void restoreCapitalOrder(Date nextExecTime,String seqNo) {
        planCapitalOrderDAO.restoreCapitalOrder(nextExecTime,seqNo);
    }
    
    /**
     * 查询执行次数较多的交易单
     * @param accountDate
     * @param execTimes
     * @param status    TaskStatusEnum
     * @return
     */
    @Override
    public List<PlanCapitalOrder> queryFailCapitalOrder(Date accountDate, Long execTimes, List<String> status){
        List<PlanCapitalOrderDO> ordrDOS = planCapitalOrderDAO.queryFailCapitalOrder(accountDate, execTimes, status);
        return PlanCapitalOrderConvertor.convert2Domains(ordrDOS);
    }

    @Override
    public Long getFailCount(Long execTimes){
        return planCapitalOrderDAO.getFailCount(execTimes);
    }
    
    /**
     * desc:根据外部交易编码查询交易单用来做mock数据.<br/>
     * @param noList noList
     * @return List<PlanCapitalOrderDO>
     */
    @Override
    public List<PlanCapitalOrder> queryOrderListByOutAcptNo(TaskStatusEnum status, List<String> noList){
        if (ToolUtil.isEmpty(noList)){
            return null;
        }
        List<PlanCapitalOrderDO> ordrDOS = planCapitalOrderDAO.queryOrderListByOutAcptNo(status.getCode(), noList);
        return PlanCapitalOrderConvertor.convert2Domains(ordrDOS);
    }

    @Override
    public CapitalOrderAmt getHisAmtByPlanNo(Date accountDate, String planNo, PlanCapitalTransTypeEnum planCapitalTransTypeEnum){
        return planCapitalOrderDAO.getHisAmtByPlanNoAndActDateAndTransCode(accountDate, planNo, planCapitalTransTypeEnum==null?null:planCapitalTransTypeEnum.getCode());
    }
    
    @Override
    public Long getActiveCntByType(PlanCapitalTransTypeEnum transType) {
        String type = transType == null ? null : transType.getCode();
        return planCapitalOrderDAO.getActiveCntByCondition(null, null,
                type, Lists.newArrayList(TaskStatusEnum.INIT.getCode(), TaskStatusEnum.WAITING.getCode()));
    }
}