/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.report.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanReportDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanReportItemDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanReportItemDefaultDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportItemDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanReportItemDefaultDO;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.model.convertor.PlanReportConvert;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.util.base.Day;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 报表
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.repo.report.impl: PlanReportRepoImpl.java, v 0.1 2019-06-28 20:09 zhenxuan.luo Exp $
 */
@Component
public class PlanReportRepoImpl implements PlanReportRepo {

    @Autowired
    private PlanReportDAO planReportDAO;
    @Autowired
    private PlanReportItemDAO planReportItemDAO;
    @Autowired
    private PlanReportItemDefaultDAO planReportItemDefaultDAO;


    @Override
    public int create(PlanReport planReport) {
        if (null == planReport) {
            return 0;
        }

        PlanReportDO planReportDO = PlanReportConvert.convert2DO(planReport);
        int refCnt = planReportDAO.insert(planReportDO);

        // 如果default 中已存在备份则不生成item
        if (ToolUtil.isNotEmpty(planReportItemDefaultDAO.getDefaultRptItem(
                planReport.getRptDate().getAccountDate(), planReport.getPlanNo()))) {
            return refCnt;
        }
        List<PlanReportItemDO> reportItemDOS = PlanReportConvert.convertItems2Do(planReport);
        if (CollectionUtils.isNotEmpty(reportItemDOS)) {
            planReportItemDAO.insertBatch(reportItemDOS);
        }
        return refCnt;
    }

    /**
     * 获取上日报表作为基础数据
     * 日报 会再根据默认报表数据中获取上一日数据构建初始化数据
     *
     * @param planNo
     * @param rptType
     * @return
     */
    @Override
    public PlanReport getLatestReport(String planNo, ReportTypeEnum rptType) {
        PlanReportDO planReportDO = planReportDAO.getLatest(planNo, rptType.getCode());
        // 判断默认数据日期比实际报表日期大时使用默认数据日期 将对象设置为空
//        if ( planReportDO != null
//             && maxDate !=null
//             && ReportTypeEnum.RPT001 == rptType
//             && DayUtil.compare(new Day(maxDate), new Day(planReportDO.getRptDate())) >= 0){
//            planReportDO = null;
//        }
        if (null == planReportDO && ReportTypeEnum.RPT002 == rptType) {
            return null;
        } else if (null == planReportDO) {
            Date maxDate = planReportItemDefaultDAO.getDefaultRptDate(planNo);
            // 获取初始化日报表基础数据
            List<PlanReportItemDefaultDO> reportItemDOList =
                    planReportItemDefaultDAO.getDefaultRptItem(maxDate, planNo);
            if (ToolUtil.isEmpty(reportItemDOList)) {
                return null;
            }
            PlanReport planReport = PlanReportConvert.defaultRPT001(planNo, rptType, reportItemDOList.get(0));
            planReport.addReportItems(PlanReportConvert.convertDefaultItems2Domain(reportItemDOList));
            return planReport;
        }
        // 构建报表
        PlanReport planReport = PlanReportConvert.convert2Domain(planReportDO);
        // 取统计项
        List<PlanReportItemDO> reportItemDOList =
                planReportItemDAO.getByRpt(planReport.getRptNo(), planReport.getPlanNo());
        // 对应日期item 如被删除则取备份数据
        planReport.addReportItems(convertOrGetDefault(
                planReport.getPlanNo(), planReport.getRptDate().getAccountDate(), rptType, reportItemDOList));
        return planReport;
    }

    @Override
    public PlanReport getReport(String planNo, ReportTypeEnum rptType, String rptDate) {
        PlanReportDO planReportDO = planReportDAO.getReport(planNo, rptDate, rptType.getCode());
        if (null == planReportDO) {
            return null;
        }
        PlanReport planReport = PlanReportConvert.convert2Domain(planReportDO);
        List<PlanReportItemDO> reportItemDOList = planReportItemDAO
                .getByRpt(planReport.getRptNo(), planReport.getPlanNo());
        // 对应日期item 如被删除则取备份数据
        planReport.addReportItems(convertOrGetDefault(planNo,
                planReport.getRptDate().getAccountDate(), rptType, reportItemDOList));
        return planReport;
    }

    /**
     * 获取时段内的报表统计项
     *
     * @param planNo
     * @param startDate
     * @param endDate
     * @return
     */
    @Override
    public List<PlanReportItem> getItemsByDate(String planNo, Date startDate, Date endDate) {
        List<PlanReportItemDO> reportItemDOList = planReportItemDAO.getByDate(endDate, startDate, planNo);
        // 对应日期item 如被删除则取备份数据
        return convertAndMergeDefault(planNo, startDate, endDate, reportItemDOList);
    }

    /**
     * 按日期获取资产报表
     *
     * @param planNo
     * @param startDate
     * @param endDate
     * @param rptType
     * @return
     */
    @Override
    public List<PlanReport> getPlansByDate(String planNo, String startDate, String endDate, ReportTypeEnum rptType) {
        List<PlanReportDO> planReportDOList = planReportDAO.getByDate(planNo, endDate, rptType.getCode(), startDate);
        return getPlanReportList(rptType, planReportDOList);
    }

    /**
     * 获取多日的报表数据
     *
     * @param planNo
     * @param rptType
     * @param dateList
     * @return
     */
    @Override
    public List<PlanReport> getPlansByDateList(String planNo, ReportTypeEnum rptType, List<String> dateList) {
        List<PlanReportDO> planReportDOList = planReportDAO.getByDateList(planNo, rptType.getCode(), dateList);
        return getPlanReportList(rptType, planReportDOList);
    }
    
    private List<PlanReport> getPlanReportList(ReportTypeEnum rptType, List<PlanReportDO> planReportDOList) {
        List<PlanReport> planReportList = Lists.newArrayList();
        for (PlanReportDO planReportDO : planReportDOList) {
            PlanReport planReport = PlanReportConvert.convert2Domain(planReportDO);
            List<PlanReportItemDO> reportItemDOList =
                    planReportItemDAO.getByRpt(planReport.getRptNo(), planReport.getPlanNo());
            // 对应日期item 如被删除则取备份数据
            planReport.addReportItems(convertOrGetDefault(
                    planReport.getPlanNo(), planReport.getRptDate().getAccountDate(), rptType, reportItemDOList));
            planReportList.add(planReport);
        }
        return planReportList;
    }

    /**
     * desc:根据计划和日期获取多个项统计值.<br/>
     *
     * @param accountDate accountDate
     * @param planNo      planNo
     * @param fields      list
     * @return List<PlanReportItemDO>
     */
    @Override
    public Map<String, PlanReportItem> queryRptItemsByPlanNoAndAccDate(Date accountDate, String planNo, List<ReportField> fields) {
        List<String> list = fields.stream().map(ReportField::getCode).collect(Collectors.toList());
        List<PlanReportItemDO> reportItemDOs = planReportItemDAO.queryRptItemsByPlanNoAndAccDate(accountDate, planNo, list);
        // 对应日期item 如被删除则取备份数据
        List<PlanReportItem> reportItems = this.convertOrGetDefault(planNo, accountDate, ReportTypeEnum.RPT001, reportItemDOs);

        if (ToolUtil.isEmpty(reportItems)) {
            return new HashMap<>();
        }
        // 按照获取统计项过滤 并转map
        Map<String, PlanReportItem> result =
                reportItems.stream().filter(one -> list.contains(one.getRptItemCode()))
                        .collect(Collectors.toMap(PlanReportItem::getRptItemCode, a -> a, (k1, k2) -> k1));
        return result;
    }

    /**
     * 根据计划和日期获取区间多个项统计值
     *
     * @param startDate
     * @param endDate
     * @param planNo
     * @param fields
     * @return
     */
    @Override
    public Map<String, PlanReportItem> queryRptItemsByPlanNoSection(Date startDate, Date endDate, String planNo, List<ReportField> fields) {
        List<String> list = fields.stream().map(ReportField::getCode).collect(Collectors.toList());
        List<PlanReportItem> reportItems = PlanReportConvert.convertItems2Domain(
                planReportItemDAO.queryPlanRptItemsByPlanNoSection(endDate, startDate, planNo, list));
        List<PlanReportItemDefaultDO> reportItemDefaultDOs =
                planReportItemDefaultDAO.queryPlanRptItemsByPlanNoSection(endDate, startDate, planNo, list);
        // 累加求和
        PlanReportConvert.summryByItem(reportItemDefaultDOs, reportItems);

        if (ToolUtil.isEmpty(reportItems)) {
            return new HashMap<>();
        }
        // 按照获取统计项过滤 并转map
        Map<String, PlanReportItem> result =
                reportItems.stream().filter(one -> list.contains(one.getRptItemCode()))
                        .collect(Collectors.toMap(PlanReportItem::getRptItemCode, a -> a, (k1, k2) -> k1));
        return result;
    }

    /**
     * desc:根据计划和日期区间获取多个统计项的每日数据.<br/>
     *
     * @param endDate    endDate
     * @param startDate  startDate
     * @param fields     listItem
     * @param listPlanNo listPlanNo
     * @return List<PlanReportItemDO>
     */
    @Override
    public List<PlanReportItem> queryDayRptItemsByPlanNoSection(Date startDate, Date endDate, List<ReportField> fields,
                                                                List<String> listPlanNo) {
        List<String> listItem = fields.stream().map(ReportField::getCode).collect(Collectors.toList());
        List<PlanReportItem> planReportItems = PlanReportConvert.convertItems2Domain(
                planReportItemDAO.queryDayRptItemsByPlanNoSection(endDate, startDate, listItem, listPlanNo));

        List<PlanReportItemDefaultDO> planReportItemDefaultDOS =
                planReportItemDefaultDAO.queryDayRptItemsByPlanNoSection(endDate, startDate, listItem, listPlanNo);

        PlanReportConvert.summryByItem(planReportItemDefaultDOS, planReportItems);
        return planReportItems;
    }


    /**
     * desc:根据计划和日期获取区间最小日期.<br/>
     *
     * @param endDate   endDate
     * @param startDate startDate
     * @param planNo    planNo
     * @param fields    fields
     * @return Date
     */
    @Override
    public Date queryReportMinDateByPlanNoSection(Date startDate, Date endDate, String planNo, List<ReportField> fields) {
        List<String> list = fields.stream().map(ReportField::getCode).collect(Collectors.toList());
        return planReportItemDAO.queryPlanReportMinDateByPlanNoSection(endDate, startDate, planNo, list);
    }

    /**
     * 给定计划是否完成
     *
     * @param planNos
     * @param rptDate
     * @return
     */
    @Override
    public boolean allFinished(List<String> planNos, String rptDate, ReportTypeEnum rptType) {
        if (CollectionUtils.isNotEmpty(planNos)) {
            for (String planNo : planNos) {
                PlanReportDO planReportDO = planReportDAO.getReport(planNo, rptDate, rptType.getCode());
                if (planReportDO == null) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * 初始化报表基础数据
     * 生成 accountDate 上一日的初始数据
     *
     * @param planNo
     * @return
     */
    @Override
    public int defaultRPT001Init(String planNo, Date accountDate) {
        Date yesterday = new Day(accountDate).getYesterday().getStandardDate();
        // 统计项
        List<ReportField> fields = WkAstReportFields.getFields();
        List<PlanReportItemDefaultDO> dos = planReportItemDefaultDAO.getDefaultRptItem(
                yesterday, planNo);
        // 已初始化数据
        if (ToolUtil.isNotEmpty(dos) && dos.size() == fields.size()) {
            return 0;
        }
        List<PlanReportItemDefaultDO> reportItemDOS = PlanReportConvert.initItemDos(planNo, yesterday, fields, dos);
        if (ToolUtil.isNotEmpty(reportItemDOS)) {
            return planReportItemDefaultDAO.insertBatch(reportItemDOS);
        }
        return 0;
    }


    private List<PlanReportItem> convertOrGetDefault(String planNo, Date accountDate, ReportTypeEnum rptType,
                                                     List<PlanReportItemDO> reportItemDOList) {
        if (ToolUtil.isEmpty(reportItemDOList) && rptType == ReportTypeEnum.RPT001) {
            // 对应日期的item 被删除 则取default备份
            List<PlanReportItemDefaultDO> reportItemDefaultList =
                    planReportItemDefaultDAO.getDefaultRptItem(accountDate, planNo);
            return PlanReportConvert.convertDefaultItems2Domain(reportItemDefaultList);
        } else {
            return PlanReportConvert.convertItems2Domain(reportItemDOList);
        }
    }

    private List<PlanReportItem> convertAndMergeDefault(String planNo, Date start, Date end,
                                                        List<PlanReportItemDO> reportItemDOList) {
        List<PlanReportItem> reportItems = PlanReportConvert.convertItems2Domain(reportItemDOList);

        // 合并default备份
        List<PlanReportItemDefaultDO> reportItemDefaultList =
                planReportItemDefaultDAO.getDefaultRptItemByDate(end, start, planNo);
        PlanReportConvert.mergeItem(reportItemDefaultList, reportItems);
        return reportItems;
    }
    
    
}