/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.rule.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.AstScreenRuleDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstScreenRuleDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.ScreenRulePage;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.model.convertor.AstScreenRuleConvert;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @author yaphet
 * @version com.abssqr.plat.common.model.repo.rule: AstScreeningRuleRepoImpl.java, v 0.1 2018-03-06 下午2:35 yaphet Exp $
 */
@Repository
public class AstScreenRuleRepoImpl implements AstScreenRuleRepo {

    @Autowired
    private AstScreenRuleDAO astScreenRuleDAO;

    /**
     * 分页查询获取筛选计划
     *
     * @param param
     * @return
     */
    @Override
    public ScreenRulePage getScreenRuleList(ScreenRulePage param) {
        return astScreenRuleDAO.getScreenRuleList(param);
    }

    @Override
    public AstScreenRule getByNo(String screenRuleNo) {
        AstScreenRuleDO astScreenRuleDO = astScreenRuleDAO.getByScreenRuleNo(screenRuleNo);
        return AstScreenRuleConvert.convert2Domain(astScreenRuleDO);
    }

    /**
     * @param screenRuleNos
     * @return
     * @description 根据筛选规则编号查询筛选规则
     */
    @Override
    public List<AstScreenRule> getByNos(List<String> screenRuleNos) {
        List<AstScreenRuleDO> astScreenRuleDOS = astScreenRuleDAO.getByScreenRuleNosList(screenRuleNos);
        List<AstScreenRule> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(astScreenRuleDOS)) {
            for (AstScreenRuleDO astScreenRuleDO : astScreenRuleDOS) {
                result.add(AstScreenRuleConvert.convert2Domain(astScreenRuleDO));
            }
        }
        return result;
    }
    
    /**
     * 新增筛选规则
     * @param astScreenRule
     * @return
     */
    @Override
    public Integer addScreenRule(AstScreenRule astScreenRule) {
        AstScreenRuleDO astScreenRuleDO=AstScreenRuleConvert.convertRule2Do(astScreenRule);
        return astScreenRuleDAO.insert(astScreenRuleDO);
    }
    
    /**
     * 修改筛选规则
     * @param astScreenRule
     * @return
     */
    @Override
    public Integer update(AstScreenRule astScreenRule) {
        AstScreenRuleDO astScreenRuleDO=AstScreenRuleConvert.convertRule2Do(astScreenRule);
        return astScreenRuleDAO.updateByScreenRuleNo(astScreenRuleDO);
    }
    
    @Override
    public int getSameRuleCnt(String screenRuleNo,String screenRuleName) {
        return astScreenRuleDAO.getSameRuleCnt(screenRuleNo,screenRuleName);
    }
    
}
