/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.task.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.AstSyncTaskCtrDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.AstSyncTaskCtrLogDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstSyncTaskCtrDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstSyncTaskCtrLogDO;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.convertor.AstSyncTaskCtrConvert;
import com.abssqr.plat.common.model.convertor.AstSyncTaskCtrLogConvert;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.alibaba.fastjson.JSON;
import com.general.system.common.util.DateTimeUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;

/**
 * 资产同步
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.task.ast.sync.impl: AstSyncTaskCtrRepoImpl.java, v 0.1 2019-06-14 10:41
 * bangis.wangdf Exp $
 */
@Repository
public class AstSyncTaskCtrRepoImpl implements AstSyncTaskCtrRepo {
    @Autowired
    private AstSyncTaskCtrDAO    astSyncTaskCtrDAO;
    @Autowired
    private AstSyncTaskCtrLogDAO astSyncTaskCtrLogDAO;

    @Override
    public List<AstSyncTaskCtr> getAstSyncTaskCtrs() {
        List<AstSyncTaskCtrDO> dos = astSyncTaskCtrDAO.getByOrgCodes();
        List<AstSyncTaskCtr> result = Lists.newArrayList();
        if(CollectionUtils.isNotEmpty(dos)) {
            dos.forEach(DO->{
                result.add(AstSyncTaskCtrConvert.convert2Domain(DO));
            });

        }
        return result;
    }

    /**
     * 根据机构获取控制
     *
     * @param orgCode
     * @return
     */
    @Override
    public AstSyncTaskCtr getAstSyncTaskCtr(String orgCode) {
        AstSyncTaskCtrDO syncTaskCtrDO = astSyncTaskCtrDAO.getByOrgCode(orgCode);
        return AstSyncTaskCtrConvert.convert2Domain(syncTaskCtrDO);
    }

    @Override
    @PerformTrace
    public AstSyncTaskCtr lockAstSyncTaskCtr(String orgCode) {
        AstSyncTaskCtrDO syncTaskCtrDO = astSyncTaskCtrDAO.lockByOrgCode(orgCode);
        return AstSyncTaskCtrConvert.convert2Domain(syncTaskCtrDO);
    }

    /**
     * 插入同步日志
     * 已经通过Quartz防止并发了，无需加锁
     *
     * @param astSyncTaskCtr
     */
    @Override
    public String insertSyncLog(Date execDate, Day orgDate, AstSyncTaskCtr astSyncTaskCtr, SyncTaskTypeEnum taskType,
                                String hostIp) {
        return insertSyncLog(execDate, orgDate, astSyncTaskCtr, taskType, TaskStatusEnum.EXECUTING, hostIp) ;
    }

    @Override
    public String insertSyncLog(Date execDate, Day orgDate, AstSyncTaskCtr astSyncTaskCtr, SyncTaskTypeEnum taskType,
                                TaskStatusEnum status, String hostIp) {
        AstSyncTaskCtrLogDO astSyncTaskCtrLogDO = new AstSyncTaskCtrLogDO();
        astSyncTaskCtrLogDO.setExecDate(execDate);
        astSyncTaskCtrLogDO.setHostIp(hostIp);
        astSyncTaskCtrLogDO.setStatus(status.getCode());
        astSyncTaskCtrLogDO.setExtData(JSON.toJSONString(astSyncTaskCtr.getExtData()));
        astSyncTaskCtrLogDO.setTaskCode(astSyncTaskCtr.getTaskCode());
        astSyncTaskCtrLogDO.setOrgCode(astSyncTaskCtr.getOrgCode());
        astSyncTaskCtrLogDO.setTaskLogCode(SequenceUtil.genId(IdTypeEnum.SEQ));
        astSyncTaskCtrLogDO.setTaskType(taskType.getCode());
        // 增加机构会计日
        astSyncTaskCtrLogDO.setOrgDate(orgDate.getStandardDate());
        astSyncTaskCtrLogDAO.insert(astSyncTaskCtrLogDO);
        return astSyncTaskCtrLogDO.getTaskLogCode();
    }

    /**
     * 更新日志记录
     *
     * @param hostIp
     * @param taskLogCode
     * @param status
     * @param oldStatus
     */
    @Override
    public void updateSyncLogStatus(String hostIp, String taskLogCode, SyncTaskTypeEnum taskType, TaskStatusEnum status,
                                    TaskStatusEnum oldStatus) {
        astSyncTaskCtrLogDAO.updateStatus(hostIp, status.getCode(), taskType.getCode(), oldStatus.getCode(),
                taskLogCode);
    }
    
    /**
     * 任务是否可以运行
     * @param orgCode 机构编码
     * @param taskCode 任务编码
     * @param taskType 任务类型
     * @param orgDate 机构会计日
     * @return
     */
    @Override
    public boolean checkCanRun(String orgCode, String taskCode, SyncTaskTypeEnum taskType, Date orgDate) {
        return astSyncTaskCtrLogDAO.canRunByOrgDate(orgDate, orgCode, taskCode, taskType.getCode()) == 0;
    }

    @Override
    public AstSyncTaskCtrLog getTaskLogByExecDate(Date execDate, SyncTaskTypeEnum taskType, String orgCode) {
        AstSyncTaskCtrLogDO taskCtrLogDO = astSyncTaskCtrLogDAO.getTaskLogByExecDate(execDate, orgCode, taskType.getCode());
        return AstSyncTaskCtrLogConvert.convert2Model(taskCtrLogDO);
    }

    @Override
    public AstSyncTaskCtrLog getLastTaskLogByOrgDate(Date execDate, SyncTaskTypeEnum taskType, String orgCode) {
        AstSyncTaskCtrLogDO taskCtrLogDO = astSyncTaskCtrLogDAO.getLastTaskLogByOrgDate(execDate, orgCode, taskType.getCode());
        return AstSyncTaskCtrLogConvert.convert2Model(taskCtrLogDO);
    }
    
    @Override
    @PerformTrace
    public AstSyncTaskCtrLog getTaskLogByOrgDate(Date orgDate, SyncTaskTypeEnum taskType, String orgCode) {
        AstSyncTaskCtrLogDO taskCtrLogDO = astSyncTaskCtrLogDAO.getTaskLogByOrgDate(orgDate, orgCode, taskType.getCode());
        return AstSyncTaskCtrLogConvert.convert2Model(taskCtrLogDO);
    }

    @Override
    public List<String> getLastStatusFinishedOrgCodes(Date acctDate, SyncTaskTypeEnum currTaskType) {
        List<AstSyncTaskCtrLogDO> astSyncTaskLogs = astSyncTaskCtrLogDAO.getLastStatusFinishedTasks(acctDate,
                currTaskType.getCode());
        List<String> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(astSyncTaskLogs)) {
            for (AstSyncTaskCtrLogDO astSyncTaskLog : astSyncTaskLogs) {
                if (DateTimeUtil.calcDayDiff(astSyncTaskLog.getExecDate(), astSyncTaskLog.getOrgDate())==1) {
                    result.add(astSyncTaskLog.getOrgCode());
                }
            }
        }
        return result;
    }

    @Override
    public void addSyncSPV(String orgCode, String entityCode) {
        AstSyncTaskCtr astSyncTaskCtr = getAstSyncTaskCtr(orgCode);
        AstSyncTaskCtrDO syncTaskCtrDO;
        if (astSyncTaskCtr == null) {
            astSyncTaskCtr = AstSyncTaskCtrConvert.initSyncTaskCtr(orgCode, entityCode);
            syncTaskCtrDO = AstSyncTaskCtrConvert.convert2Do(astSyncTaskCtr);
            astSyncTaskCtrDAO.insert(syncTaskCtrDO);
        }else{
            astSyncTaskCtr.getExtData().getSpvPlanNos().add(entityCode);
            syncTaskCtrDO = AstSyncTaskCtrConvert.convert2Do(astSyncTaskCtr);
            // 更新扩展字段
            astSyncTaskCtrDAO.updateExtData(syncTaskCtrDO.getOrgCode(), syncTaskCtrDO.getExtData());
        }
    }

//    @Override
//    public Date getTaskLogMaxExecDate(SyncTaskTypeEnum taskType) {
//        return this.getTaskLogMaxExecDate(taskType,null);
//    }

    /**
     * desc:获取指定类型的最大完成日期.<br/>
     *
     * @param taskType taskType
     * @return Date
     */
    @Override
    public Date getTaskLogMaxOrgDate(SyncTaskTypeEnum taskType, String managerOrgCode) {
        return astSyncTaskCtrLogDAO.getTaskLogMaxOrgDate(managerOrgCode, taskType.getCode());
    }

}
