/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.tr.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.TrCtrTaskDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.TrCtrTaskDO;
import com.abssqr.plat.common.facade.enums.AstPackTradeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.convertor.TrCtrTaskConvert;
import com.abssqr.plat.common.model.domain.task.TrCtrTask;
import com.abssqr.plat.common.model.repo.tr.TrCtrTaskRepo;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Optional;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.tr.impl: TrCtrTaskRepoImpl.java, v 0.1 2019-06-17 22:39 bangis.wangdf
 * Exp $
 */
@Repository
public class TrCtrTaskRepoImpl implements TrCtrTaskRepo {
    @Autowired
    private TrCtrTaskDAO trCtrTaskDAO;
    
    /**
     * 批量插入转让任务
     *
     * @param trCtrTasks
     * @return
     */
    @Override
    public int insertBatch(List<TrCtrTask> trCtrTasks) {
        if (CollectionUtils.isNotEmpty(trCtrTasks)) {
            List<TrCtrTaskDO> entitys = TrCtrTaskConvert.convert2DOs(trCtrTasks);
            if (CollectionUtils.isNotEmpty(entitys)) {
                return trCtrTaskDAO.insertBatch(entitys);
            }
        }
        return 0;
    }
    
    @Override
    public TrCtrTask getByStatus(Date accountDate, TaskStatusEnum status, AstPackTradeTypeEnum packType) {
        TrCtrTaskDO trCtrTaskDO = trCtrTaskDAO.getByStatus(accountDate, status.getCode(), packType.getCode());
        return TrCtrTaskConvert.convert2Domain(trCtrTaskDO);
    }
    
    @Override
    public List<TrCtrTask> getByManagerOrgCode(String managerOrgCode, Date accountDate, TaskStatusEnum status, AstPackTradeTypeEnum packType) {
        List<TrCtrTaskDO> trCtrTaskDOs = trCtrTaskDAO.getByManagerOrg(
                accountDate, Optional.ofNullable(status).map(TaskStatusEnum::getCode).orElse(null), packType.getCode(), managerOrgCode);
        List<TrCtrTask> trCtrTasks = Lists.newArrayList() ;
        
        for (TrCtrTaskDO trCtrTaskDO : trCtrTaskDOs) {
            trCtrTasks.add(TrCtrTaskConvert.convert2Domain(trCtrTaskDO));
        }
        return trCtrTasks;
    }
    
    @Override
    public TrCtrTask lockById(Long id) {
        TrCtrTaskDO trCtrTaskDO = trCtrTaskDAO.lockById(id);
        return TrCtrTaskConvert.convert2Domain(trCtrTaskDO);
    }
    
    @Override
    public int updateStatus(Long id, TaskStatusEnum status) {
        return trCtrTaskDAO.updateStatus(id, status.getCode());
    }
    
    @Override
    public int updateExecTms(Long id, Long execTms) {
        return trCtrTaskDAO.updateExecTms(id, execTms);
    }
    
    /**
     * 根据状态日期类型查询控制任务
     * @param tfrDate
     * @param status
     * @param packType
     * @return
     */
    @Override
    public List<TrCtrTask> queryInitCtrTaskList(Date tfrDate, TaskStatusEnum status, AstPackTradeTypeEnum packType){
        List<TrCtrTaskDO> list = trCtrTaskDAO.queryInitCtrTaskList(tfrDate, status.getCode(), packType.getCode());
        return TrCtrTaskConvert.convert2Domains(list);
    }
    
}
