/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.TransInfoTemplate;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.system.TimeStampUtil;
import com.general.system.common.util.LogUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 资产交易操作基类
 * @author zhenxuan.luo
 * @version LoanAstBaseManager.java, v 0.1 2019-06-17 20:21 zhenxuan.luo Exp $
 */
public abstract class LoanAstBaseManager {
    
    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());
    
    @PerformTrace
    public Loan process(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans, Loan loan) {
        
        LogUtil.info(LOGGER, "[AstSync]贷款[{0}]同步操作[{1}] - [{2}]", baseTsfRes.getLoanAstSyncBill(),
                loanAstSyncTrans.getEventCode(), loanAstSyncTrans.getBizNo());
        TransInfoTemplate transInfoTemplate = new TransInfoTemplate();
        transInfoTemplate.setTransTime(loanAstSyncTrans.getTransTime());
        transInfoTemplate.setAccountDate(loanAstSyncTrans.getAccountDate());
        transInfoTemplate.setEventCode(loanAstSyncTrans.getEventCode());
        transInfoTemplate.setOutBizNo(loanAstSyncTrans.getBizNo());
        transInfoTemplate.setBizNo(SequenceUtil.genSeqByLoan(loan, loanAstSyncTrans.getEventCode().getSceneCode()));
        transInfoTemplate.setTimestamp(TimeStampUtil.getDefaultTimeStamp());
        // 构建交易模型
        LoanTransDetail loanTransDetail = LoanTransDetail.createLoanAstTxnDtl(loan, transInfoTemplate);
        
        doProcess(baseTsfRes, loanAstSyncTrans, loanTransDetail);
        
        loanTransDetail.processBeforeUpdate();
        
        return loanTransDetail.getLoan();
        
    }
    
    protected abstract Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                                      LoanTransDetail loanTransDetail);
    
}