/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.CapitalAstMatchEnum;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncBill;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncContract;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlan;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.contract.LoanContract;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.TransInfoTemplate;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.contract.LoanContractRepository;
import com.abssqr.plat.common.model.repo.pool.AstPoolRepository;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.model.repo.prod.LoanProdRiskClRepository;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.model.utils.LoanAstUtil;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.abssqr.plat.core.service.system.TimeStampUtil;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 放款资产同步
 *
 * @author zhenxuan.luo
 * @version LoanAstInitManager.java, v 0.1 2018-01-05 下午23:07
 * zhenxuan.luo Exp $
 */
@Component
public class LoanAstInitManager {

    private final static Logger LOGGER = LoggerFactory.getLogger(LoanAstInitManager.class);

    @Autowired
    private LoanAstAssembler loanAstAssembler;

    @Autowired
    private LoanProdRiskClRepository prodRiskClRepository;

    @Autowired
    private LoanProdRepository loanProdRepository;

    @Autowired
    private LoanAstRiskManager loanAstRiskManager;

    @Autowired
    private LoanContractRepository loanContractRepository;

    @Autowired
    private AstPoolRepository astPoolRepository;

    /**
     * 初始化新的资产
     *
     * @param tsfRes
     * @return
     */
    @PerformTrace
    public Loan initAst(BaseTsfRes tsfRes, LoanAstSyncTrans loanAstSyncTrans) {

        LogUtil.info(LOGGER, "[AstSync-Sync]资产初始化同步[{0}]", tsfRes.getLoanAstSyncBill());

        LoanAstSyncBill loanAstSyncBill = tsfRes.getLoanAstSyncBill();
        LoanAstSyncContract loanAstSyncContract = tsfRes.getLoanContract();
        List<LoanAstSyncRepayPlan> loanAstSyncRepayPlans = tsfRes.getRepayPlans();

        LoanProduct loanProduct = loanProdRepository.getLoanProdByPK(loanAstSyncBill.getProdCode(),
                loanAstSyncBill.getManagerOrgCode());

        if (loanProduct == null) {
            throw new AbssqrBizException("机构产品未配置，prodCode: " + loanAstSyncBill.getProdCode() + ",managerOrg："
                    + loanAstSyncBill.getManagerOrgCode() + ",outAstId: " + loanAstSyncBill,
                    CommonErrorCodeEnum.NO_DATA);
        }

        // 0. 生成资产ID和合同ID
        String astNo = SequenceUtil.genId(IdTypeEnum.AST_ID);
        String contractNo = SequenceUtil.genId(IdTypeEnum.AST_CONTRACT_ID);
        String userId = loanAstSyncContract.getUserId();

        // 1. 组装贷款
        Loan loan = buildLoan(loanProduct, loanAstSyncBill, loanAstSyncRepayPlans, loanAstSyncTrans,
                astNo, contractNo, userId, loanAstSyncContract.getCertNo());

        // 2. 组装合同
        LoanContract loanContract = buildLoanContract(loanAstSyncContract, loan);
        loanContractRepository.create(loanContract);

        loan.setContract(loanContract);
        return loan;
    }

    private Loan buildLoan(LoanProduct loanProduct, LoanAstSyncBill loanAstSyncBill,
                           List<LoanAstSyncRepayPlan> repayPlans, LoanAstSyncTrans loanAstSyncTrans, String astNo,
                           String contractNo, String userId, String certNo) {

        Loan loan = new Loan();
        loan.setAstNo(astNo);
        loan.setUserId(userId);
        loan.setCertNo(certNo);
        loan.setOutAstNo(loanAstSyncBill.getLoanNo());
        loan.setContNo(contractNo);
        loan.setProdNo(loanProduct.getAbssqrProdCode());
        loan.setOutProdNo(loanProduct.getProdCode());
        loan.setOwnPack("");
        loan.setCurTerm(loanAstSyncBill.getCurTerm());
        loan.setTermCnt(loanAstSyncBill.getTermCnt());
        loan.setStatus(loanAstSyncBill.getStatus());
        loan.getAstOrg().setSponsorOrgCode(loanAstSyncBill.getSponserOrgCode());
        loan.getAstOrg().setManagerOrgCode(loanAstSyncBill.getManagerOrgCode());
        // 初始化时 出资机构 与所属机构一致
        loan.getAstOrg().getAstOwnership().setLenderOrgCode(loanAstSyncBill.getLenderOrgCode());
        loan.getAstOrg().getAstOwnership().setOwnOrg(loanAstSyncBill.getLenderOrgCode());
        loan.getAstOrg().getAstOwnership().setOwnStatus(OwnStatusEnum.INIT);
        loan.setLastUpdateDate(loanAstSyncTrans.getAccountDate());
        loan.getAstPeriod().setStartDate(loanAstSyncBill.getStartDate());
        loan.getAstPeriod().setEndDate(loanAstSyncBill.getEndDate());
        loan.getAstPeriod().setInDate(loanAstSyncBill.getStartDate());
        loan.getAstPeriod().setDisbDate(loanAstSyncBill.getDisbDate());
        loan.setCapitalNo(CapitalAstMatchEnum.DEFAULT.getCode());
        List<LoanInstalDomain> loanInstals = Lists.newArrayList();
        for (LoanAstSyncRepayPlan repayPlan : repayPlans) {
            LoanInstalDomain loanInstal = new LoanInstalDomain();
            loanInstal.setRefLoan(loan);
            loanInstal.setRefLoan(loan);
            loanInstal.setAstNo(astNo);
            loanInstal.setUserId(userId);
            loanInstal.setOutAstNo(loan.getOutAstNo());
            loanInstal.setContNo(loan.getContNo());
            loanInstal.setProdNo(loan.getProdNo());
            loanInstal.setOutProdNo(loan.getOutProdNo());
            loanInstal.setOwnPack(loan.getOwnPack());
            loanInstal.setTermNo(repayPlan.getTermNo());
            loanInstal.setDueIntrAmt(repayPlan.getDueIntrAmt().clone());
            loanInstal.setDueFeeAmt(repayPlan.getDueFeeAmt().clone());
            loanInstal.setStatus(repayPlan.getStatus());


            loanInstal.setLastUpdateDate(loan.getLastUpdateDate());
            loanInstal.getAstPeriod().setStartDate(repayPlan.getAstPeriod().getStartDate());
            loanInstal.getAstPeriod().setEndDate(repayPlan.getAstPeriod().getEndDate());
            loanInstal.getAstPeriod().setInDate(repayPlan.getAstPeriod().getInDate());
            loanInstal.getAstPeriod().setGraceEndDate(repayPlan.getAstPeriod().getGraceEndDate());
            loanInstal.getAstPeriod().setClearDate(repayPlan.getAstPeriod().getClearDate());
            loanInstal.getAstPeriod().setDisbDate(loan.getAstPeriod().getDisbDate());

            if (loanInstal.getStatus().isActive()) {
                int ovdDays = (int) DateTimeUtil.calcDayDiff(loanAstSyncTrans.getAccountDate(),
                        loanInstal.getAstPeriod().getEndDate());
                if (ovdDays > 0) {
                    loanInstal.setOvdDays(ovdDays);
                    loanInstal.setStatus(LoanStatusEnum.OVERDUE);
                } else {
                    loanInstal.setStatus(LoanStatusEnum.NORMAL);
                }
            }

            loanInstal.getAstOrg().setSponsorOrgCode(loan.getAstOrg().getSponsorOrgCode());
            loanInstal.getAstOrg().setManagerOrgCode(loan.getAstOrg().getManagerOrgCode());
            loanInstal.getAstOrg().getAstOwnership()
                    .setLenderOrgCode(loan.getAstOrg().getAstOwnership().getLenderOrgCode());
            loanInstal.getAstOrg().getAstOwnership().setLastOwnOrg(loan.getAstOrg().getAstOwnership().getLastOwnOrg());
            loanInstal.getAstOrg().getAstOwnership().setOwnOrg(loan.getAstOrg().getAstOwnership().getOwnOrg());
            loanInstal.getAstOrg().getAstOwnership().setOwnStatus(OwnStatusEnum.INIT);

            // 提前还款无法判定已还信息归属，暂不处理已还信息登记
//            loanInstal.getPaidAmt().addAll(repayPlan.getPaid());
//            loan.getPaid().addAll(repayPlan.getPaid());

            loanInstals.add(loanInstal);
        }
        loan.setLoanInstalList(loanInstals);

        loan.fillInfoByInstalls();
        loan.setLoanRisk(loanAstRiskManager.calcRiskInfo(loan, loanProduct));

        TransInfoTemplate transInfoTemplate = new TransInfoTemplate();
        transInfoTemplate.setTransTime(loanAstSyncTrans.getTransTime());
        transInfoTemplate.setAccountDate(loanAstSyncTrans.getAccountDate());
        transInfoTemplate.setEventCode(loanAstSyncTrans.getEventCode());
        transInfoTemplate.setOutBizNo(loanAstSyncTrans.getBizNo());

        transInfoTemplate.setBizNo(SequenceUtil.genSeqByLoan(loan, transInfoTemplate.getEventCode().getSceneCode()));
        transInfoTemplate.setTimestamp(TimeStampUtil.getDefaultTimeStamp());

        LoanTransDetail loanTransDetail = LoanTransDetail.createLoanAstTxnDtl(loan, transInfoTemplate);

        Map<Integer, LoanAstSyncRepayPlan> repayPlanMap = repayPlans.stream().collect(
                Collectors.toMap(LoanAstSyncRepayPlan::getTermNo, item -> item));

        Map<Integer, LoanAstSyncRepayPlanTrans> repayPlanTransMap = loanAstSyncTrans.getRepayPlanTransList().stream()
                .collect(Collectors
                        .toMap(LoanAstSyncRepayPlanTrans::getTermNo,
                                item -> item));
        for (LoanInstalDomain loanInstal : loan.getLoanInstalList()) {
            LoanAstSyncRepayPlan repayPlan = repayPlanMap.get(loanInstal.getTermNo());
            LoanAstSyncRepayPlanTrans repayPlanTrans = repayPlanTransMap.get(loanInstal.getTermNo());

            LoanInstalTransDetail loanInstalTransDetail = loanTransDetail.addLoanInstalTransDtl(loanInstal,
                    repayPlanTrans.getEventCode());
            // 强制更新记流水
            loanInstalTransDetail.setUpdate();
            loanInstalTransDetail.processTransAmt(repayPlan.getBal());

            LoanAstUtil.initFromInstal(loanInstalTransDetail);
            if (!loanInstal.getStatus().isActive()) {
                loanInstalTransDetail.clearLoanAst(transInfoTemplate.getAccountDate());
            }
        }
        loanTransDetail.processBeforeUpdate();
        return loan;
    }

    private LoanContract buildLoanContract(LoanAstSyncContract loanAstSyncContract, Loan loan) {

        LoanContract loanContract = new LoanContract();

        loanContract.setLenderOrgCode(loan.getAstOrg().getAstOwnership().getLenderOrgCode());
        loanContract.setAstNo(loan.getAstNo());
        loanContract.setUserId(loanAstSyncContract.getUserId());
        loanContract.setOutAstNo(loan.getOutAstNo());
        loanContract.setDisbAmount(loanAstSyncContract.getDisbAmt());
        loanContract.setDisbDate(loanAstSyncContract.getDisbDate());
        loanContract.setContNo(loan.getContNo());
        loanContract.setOutContNo(loanAstSyncContract.getContNo());
        loanContract.setProdNo(loan.getProdNo());
        loanContract.setContractType(loanAstSyncContract.getContType());
        loanContract.getLoanProperty().setRepayMethod(loanAstSyncContract.getRepayMethod());
        loanContract.getLoanProperty().setRepayPeriod(loanAstSyncContract.getRepayPeriod());
        loanContract.getLoanProperty().setRepayPeriodType(loanAstSyncContract.getRepayPeriodType());
        loanContract.getLoanProperty().setRateDays(loanAstSyncContract.getDaysCoef());
        loanContract.getLoanProperty().setRate(loanAstSyncContract.getInterRate());
        loanContract.getLoanProperty().setPnyRate(loanAstSyncContract.getPenalRate());
        loanContract.getLoanProperty().setRateType(loanAstSyncContract.getRateType());
        loanContract.getLoanProperty().setTermCnt(loanAstSyncContract.getTermCnt());
        loanContract.getLoanProperty().setLoanType(loanAstSyncContract.getLoanType());
        loanContract.getLoanProperty().setPurpose(loanAstSyncContract.getPurpose());
        loanContract.getLoanCustInfo().setName(loanAstSyncContract.getUserName());
        loanContract.getLoanCustInfo().setCertType(loanAstSyncContract.getCertType());
        loanContract.getLoanCustInfo().setCertNo(loanAstSyncContract.getCertNo());
        loanContract.getLoanCustInfo().setMobileNo(loanAstSyncContract.getMobileNo());
        loanContract.getLoanCustInfo().setProvince(loanAstSyncContract.getProvince());
        loanContract.getLoanCustInfo().setCity(loanAstSyncContract.getCity());
        loanContract.getLoanCustInfo().setAppLoc(loanAstSyncContract.getApplyLoc());

        loanContract.getContractInst().setSponsorOrgCode(loan.getAstOrg().getSponsorOrgCode());
        loanContract.getContractInst().setManagerOrgCode(loan.getAstOrg().getManagerOrgCode());
        loanContract.getContractInst().setGrantType(loanAstSyncContract.getGrantType());
        loanContract.getContractPeriod().setStartDate(loanAstSyncContract.getStartDate());
        loanContract.getContractPeriod().setEndDate(loanAstSyncContract.getEndDate());
        loanContract.getContractPeriod().setEffectDate(loanAstSyncContract.getStartDate());

        return loanContract;
    }
}