/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlan;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.util.profiler.PerformTraceContext;
import com.abssqr.plat.core.service.profit.ProfitSplitComponent;
import com.abssqr.plat.core.share.context.PlanContextHolder;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.stream.Collectors;

/**
 * 核销处理
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.ast: LoanAstSettleManager.java, v 0.1 2019-06-17 20:07 zhenxuan.luo Exp $
 */
@Component
public class LoanAstWftManager extends LoanAstBaseManager {

    @Override
    protected Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                             LoanTransDetail loanTransDetail) {
        loanTransDetail.writeOff();


        Loan loan = loanTransDetail.getLoan();
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();
        Map<Integer, LoanAstSyncRepayPlan> syncRepayPlanMap = baseTsfRes.getRepayPlans().stream().collect(
                Collectors.toMap(LoanAstSyncRepayPlan::getTermNo, item -> item));

        EventCodeEnum eventCode = EventCodeEnum.WRITE_OFF;
        for (LoanAstSyncRepayPlanTrans repayPlanTrans : loanAstSyncTrans.getRepayPlanTransList()) {

            LoanInstalDomain refInstal = loanInstalMap.get(repayPlanTrans.getTermNo());

            LoanInstalTransDetail loanInstalTransDetail = loanTransDetail
                    .addLoanInstalTransDtl(refInstal, eventCode);
            loanInstalTransDetail.getActiveLog().setTransTime(repayPlanTrans.getTransTime());
            loanInstalTransDetail.getActiveLog().setOutBizNo(repayPlanTrans.getOutBizNo());

            // 1. 处理金额
            loanInstalTransDetail.processTransAmt(refInstal.getBal());


            for (LoanAst loanAst : refInstal.getLoanAsts()) {
                LoanAstTransDetail astTransDetailIn = LoanAstTransDetail.createTransDtl(loanAst, loanInstalTransDetail,
                        EventCodeEnum.WRITE_OFF);
                astTransDetailIn.processTransAmt(loanAst.getPrinBal());
                LoanAstProfit loanAstProfit = loanAst.getLoanAstProfit(loanAst.getTermNo(), ProfitTypeEnum.OWN);
                LoanAstProfitTransDetail transDtl = LoanAstProfitTransDetail
                        .createTransDtl(loanAstProfit,
                                loanInstalTransDetail, EventCodeEnum.WRITE_OFF);
                transDtl.processTransAmt(loanAstProfit.getBal());
            }

        }
        return loanTransDetail.getLoan();
    }
}