/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanRisk;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.common.util.profiler.PerformTrace;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 资产状态与五级分类变更
 * @author zhenxuan.luo
 * @version LoanStatusManager.java, v 0.1 2019-06-18 11:59 zhenxuan.luo Exp $
 */
@Component
public class LoanStatusManager {
    @Autowired
    private LoanAstRiskManager loanAstRiskManager;
    
    @PerformTrace
    public void calcLoanRiskStatus(LoanTransDetail loanTransDetail, boolean checkCurTerm) {
        Loan loan = loanTransDetail.getLoan();
        
        boolean isAllClear = true;
        LoanInstal curTerm = null;
        for (LoanInstal loanInstal : loan.getLoanInstalList()) {
            if (loanInstal.getStatus() != LoanStatusEnum.CLEAR) {
                isAllClear = false;
            }
            
            if (loanInstal.getAstPeriod().isInTerm(loanTransDetail.getActiveLog().getAccountDate())) {
                curTerm = loanInstal;
            }
        }
        
        if (isAllClear) {
            VarChecker.checkArgument(loan.getBal().isZero(), "贷款结清但是余额不为0.[{0}]", loan);
            loanTransDetail.setStatus(LoanStatusEnum.CLEAR);
            loanTransDetail.setOvdDays(0);
            loanTransDetail.setClearDate(loanTransDetail.getActiveLog().getAccountDate());
            return;
        }
        
        // 计算五级分类 内部查询产品进行计算
        LoanRisk loanRisk = loanAstRiskManager.calcRiskInfo(loan, null);
        
        if (loanRisk.getOvdDays() == 0) {
            loanTransDetail.setStatus(LoanStatusEnum.NORMAL);
        } else {
            loanTransDetail.setStatus(LoanStatusEnum.OVERDUE);
        }
        loanTransDetail.setAstCl(loanRisk.getRiskClass());
        loanTransDetail.setOvdDays(loanRisk.getOvdDays());

        
        if (curTerm == null) {
            // 整笔逾期
            curTerm = loan.getLoanInstalList().get(loan.getLoanInstalList().size() - 1);
        }
        if (checkCurTerm) {
            loanTransDetail.setCurTerm(curTerm.getTermNo());
        }
    }
}