/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.notice;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.integration.notice.impl.RobotNoticeIntegration;
import com.abssqr.plat.common.model.domain.AbssqrNotice;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.alibaba.fastjson.JSON;
import com.general.system.common.dal.auto.paging.ReceiverNoticeListPage;
import com.general.system.common.dal.auto.resultMap.ReceiverNoticeListItem;
import com.general.system.common.model.IEnum;
import com.general.system.common.model.domain.User;
import com.general.system.core.common.constant.Const;
import com.general.system.core.common.constant.state.NoticeType;
import com.general.system.modular.result.NoticeResult;
import com.general.system.modular.service.INoticeService;
import com.general.system.modular.service.IUserService;
import com.general.system.modular.transfer.NoticeAddParam;
import com.general.system.tool.util.ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 通知管理
 *
 * @author hanfei
 * @version $Id: AbssqrNoticeManager.java, v 0.1 2019-07-08 5:23 PM hanfei Exp $
 */
@Service
public class AbssqrNoticeManager {

    @Autowired
    private JobControlRepository jobControlRepository;
    @Autowired
    private RobotNoticeIntegration robotNoticeIntegration;
    @Autowired
    private INoticeService noticeService;
    @Autowired
    private IUserService userService;

    /**
     * 系统通知告警
     *
     * @param bizTitle
     * @param bizCode
     * @param bizType
     * @param content
     * @param receivers
     */
    public void sendSystemNotice(String bizTitle,
                                 String bizCode, IEnum bizType, String content, Set<String> receivers) {
        AbssqrNotice abssqrNotice = new AbssqrNotice();
        abssqrNotice.setBizCode(bizCode);
        abssqrNotice.setPlanNo(bizCode);
        abssqrNotice.setPlanName(bizTitle);
        if (bizType != null) {
            abssqrNotice.setBizType(String.valueOf(bizType.getCode()));
            abssqrNotice.setBizTypeName(bizType.getDesc());
        }
        abssqrNotice.setReceivers(receivers);
        abssqrNotice.setContent(content);
        abssqrNotice.setType(NoticeType.SYSTEM);
        this.sendSystemNotice(abssqrNotice);
    }
    

    /**
     * 业务通知告警
     *
     * @param planEntity
     * @param bizCode
     * @param bizType
     * @param content
     * @param receivers
     */
    public void sendBusinessNotice(PlanBase planEntity,
                                   String bizCode, IEnum bizType, String content, Set<String> receivers) {
        AbssqrNotice abssqrNotice = new AbssqrNotice();
        abssqrNotice.setBizCode(bizCode);
        abssqrNotice.setPlanNo(planEntity.getPlanNo());
        abssqrNotice.setPlanName(planEntity.getPlanName());
        if (bizType != null) {
            abssqrNotice.setBizType(String.valueOf(bizType.getCode()));
            abssqrNotice.setBizTypeName(bizType.getDesc());
        }
        abssqrNotice.setContent(content);
        abssqrNotice.setReceivers(receivers);
        abssqrNotice.setType(NoticeType.BUSINESS);
        this.sendSystemNotice(abssqrNotice);

    }

    /**
     * 发送消息
     *
     * @param abssqrNotice
     */
    private void sendSystemNotice(AbssqrNotice abssqrNotice) {
        NoticeAddParam param = new NoticeAddParam();
        param.setBizCode(abssqrNotice.getBizCode());
        param.setBizType(abssqrNotice.getBizType());
        param.setTitle(abssqrNotice.getBizTypeName());
        param.setContent(JSON.toJSONString(abssqrNotice));
        param.setSend(true);
        param.setFixedTime(DateUtil.date());
        if (ToolUtil.isEmpty(abssqrNotice.getReceivers())) {
            param.setWhole(true);
        } else {
            param.setReceivers(abssqrNotice.getReceivers());
        }
        noticeService.createUserNotice(param, abssqrNotice.getType(), AbssqrConstants.DEFALUT_SYS_USER, "");

        // 判断是否发送外部通知
        JobControl dingTalkNotice = jobControlRepository.selectByJobName("DingTalkNotice");
        if (dingTalkNotice.getIsEff() == YNEnum.Y) {
            robotNoticeIntegration.sendMsg(abssqrNotice.getBizTypeName(), abssqrNotice.getContent(), abssqrNotice.getType() == null ? null : abssqrNotice.getType().getCode().toString());
        }
    }

    /**
     * 查询消息通知列表
     *
     * @param page
     * @return
     */
    public ReceiverNoticeListPage queryNoticeByReceiver(ReceiverNoticeListPage page) {
        // 查询收件列表
        noticeService.queryNoticeByReceiver(page);
        page.setDatas(convert(page.getDatas()));
        return page;
    }

    private List convert(List<ReceiverNoticeListItem> list) {
        if (ToolUtil.isEmpty(list)) {
            return null;
        }
        List<AbssqrNotice> newList = new ArrayList<>(list.size());
        list.forEach(one -> {
            AbssqrNotice notice = null;
            if (StrUtil.isEmpty(one.getContent())) {
                notice = new AbssqrNotice();
            } else {
                try {
                    notice = JSON.parseObject(one.getContent(), AbssqrNotice.class);
                } catch (Exception e) {
                }
            }
            if (notice == null) {
                notice = new AbssqrNotice();
                notice.setBizCode(one.getBizCode());
                notice.setContent(one.getContent());
                notice.setBizTypeName(one.getTitle());
            }
            notice.setId(one.getId());
            notice.setNoticeId(one.getNoticeId());
            notice.setGmtCreate(one.getGmtCreate());
            notice.setType(NoticeType.getEnum(one.getType()));
            newList.add(notice);
        });
        return newList;
    }

    public Set<String> getAdminReceivers() {
        // 系统管理员角色用户
        List<User> user = userService.getUserByRoleCode(Const.ADMIN_ROLE_CODE);
        if (ToolUtil.isEmpty(user)) {
            return null;
        } else {
            return user.stream().map(User::getUserCode).collect(Collectors.toSet());
        }
    }

    /**
     * 根据消息id 重试外部消息接口
     * @param noticeId
     * @return
     */
    public boolean retrySend(Long noticeId){
        NoticeResult abssqrNotice = noticeService.getSenderNotice(noticeId);
        if (abssqrNotice == null){
            throw new AbssqrBizException("未找到对应通知消息，错误的id");
        }
        return robotNoticeIntegration.sendMsg(abssqrNotice.getTitle(),
                abssqrNotice.getContent(), abssqrNotice.getType() == null ? null : abssqrNotice.getType().toString());

    }
}
