/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.period;

import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.LoanAstStatusEnum;
import com.abssqr.plat.common.facade.enums.ProfitTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.general.system.common.model.Money;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.MoneyUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * 分期调整
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.period: PeriodAdjComponent.java, v 0.1 2019-11-22 下午14:10 zhenxuan.luo Exp $ 
 */
@Component
public class PeriodAdjComponent {
    
    @Autowired
    private PlanRepo planRepo;
    
    public void process(List<LoanInstalTransDetail> instalTransDetails) {
        
        List<LoanInstalTransDetail> adjInTransList = Lists.newArrayList();
        List<LoanAstEndDate> adjOutLoanAstList = Lists.newArrayList();
        
        // 1. 从分期流水中，抽取转入的分期列表和转出的资产列表
        buildInAndOutList(instalTransDetails, adjInTransList, adjOutLoanAstList);
        
        // 2. 确定转出的机构资产，转入到哪里
        Collections.sort(adjOutLoanAstList, Comparator.comparing(o -> o.ownOrgDate));
        
        Map<String, LoanAstTransDetail> cacheAstTransDetail = Maps.newHashMap();
        
        for (LoanInstalTransDetail adjInTrans : adjInTransList) {
            LoanInstalDomain instalDomain = adjInTrans.getLoanInstal();
            LoanInstalLog instalLog = adjInTrans.getActiveLog();
            
            Money transAmt = adjInTrans.getActiveLog().getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN);
            Money totalAmt = new Money();
            
            Iterator<LoanAstEndDate> it = adjOutLoanAstList.iterator();
            while (it.hasNext()) {
                LoanAstEndDate loanAstEndDate = it.next();
                LoanAst outAst = loanAstEndDate.loanAst;
                
                Money entryAmt = MoneyUtil.min(transAmt, loanAstEndDate.entryAmt);
                if (entryAmt.isGreaterThanZero()) {
                    transAmt.subtractFrom(entryAmt);
                    loanAstEndDate.entryAmt.subtractFrom(entryAmt);
                    totalAmt.addTo(entryAmt);
                    
                    LoanAst loanAst = instalDomain.filterLoanAst(outAst.getOwnOrg(), outAst.getInRule());
                    if (null == loanAst) {
                        loanAst = outAst.cloneOrTermAdj(instalDomain, instalLog.getAccountDate());
                        
                        LoanAstProfit astProfit = loanAst.getLoanAstProfit(loanAst.getTermNo(), ProfitTypeEnum.OWN);
                        LoanAstProfitTransDetail astProfitTransDetail = LoanAstProfitTransDetail
                                                                                .createTransDtl(astProfit, adjInTrans,
                                                                                        instalLog.getEventCode());
                        astProfitTransDetail.setUpdate(true);
                    }
                    LoanAstTransDetail astTransDetail = cacheAstTransDetail.get(loanAst.getLoanAstNo());
                    if (null == astTransDetail) {
                        astTransDetail = LoanAstTransDetail
                                                 .createTransDtl(loanAst, adjInTrans, instalLog.getEventCode());
                        cacheAstTransDetail.put(loanAst.getLoanAstNo(), astTransDetail);
                    }
                    astTransDetail.processTransAmt(entryAmt);
                }
                
                if (loanAstEndDate.entryAmt.isZero()) {
                    it.remove();
                }
                
                if (transAmt.isZero()) {
                    break;
                }
            }
            
            VarChecker.checkZero(transAmt, "分期[{0}-{1}]分期本金转入处理，分期转入金额有[{3}]未处理", transAmt);
            VarChecker.checkEquals(totalAmt, instalLog.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN),
                    "分期[{0}-{1}]分期本金转入处理，机构资产总转入金额[{2}和分期转入金额[{3}]不符",
                    totalAmt, instalLog.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
        }
    }
    
    private void buildInAndOutList(List<LoanInstalTransDetail> instalTransDetails,
                                   List<LoanInstalTransDetail> adjInTransList, List<LoanAstEndDate> adjOutLoanAstList) {
        Map<String, PlanBase> planBaseMap = Maps.newHashMap();
        Collections.sort(instalTransDetails, Comparator.comparingInt(o -> o.getLoanInstal().getTermNo()));
        
        for (LoanInstalTransDetail instalTransDetail : instalTransDetails) {
            LoanInstalLog instalLog = instalTransDetail.getActiveLog();
            LoanInstalDomain instalDomain = instalTransDetail.getLoanInstal();
            
            if (instalLog.getAmt().isZero()) {
                continue;
            }
            
            if (instalLog.getEventCode() == EventCodeEnum.ADJ_TERM_IN) {
                adjInTransList.add(instalTransDetail);
                continue;
            } else if (instalLog.getEventCode() == EventCodeEnum.ADJ_TERM_OUT) {
                List<LoanAstEndDate> loanAstEndDates = Lists.newArrayList();
                
                // 确定每个机构资产对应所属机构的到期日
                for (LoanAst loanAst : instalDomain.getLoanAsts()) {
                    if (loanAst.getAstStatus() != LoanAstStatusEnum.NORMAL) {
                        continue;
                    }
                    
                    PlanBase ownOrg = planBaseMap.get(loanAst.getOwnOrg());
                    if (null == ownOrg) {
                        ownOrg = planRepo.getPlanByNo(loanAst.getOwnOrg());
                        planBaseMap.put(ownOrg.getPlanNo(), ownOrg);
                    }
                    
                    loanAstEndDates.add(new LoanAstEndDate(ownOrg.getEndDate(), loanAst));
                }
                
                if (CollectionUtils.isEmpty(loanAstEndDates)) {
                    continue;
                }
                
                // 优先转出到期日晚的
                Collections.sort(loanAstEndDates, (o1, o2) -> o2.ownOrgDate.compareTo(o2.ownOrgDate));
                // 调出本金金额
                Money remainOutAmt = instalLog.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN);
                Money totalOutAmt = new Money();
                for (LoanAstEndDate loanAstEndDate : loanAstEndDates) {
                    LoanAst loanAst = loanAstEndDate.loanAst;
                    Money entryAmt = MoneyUtil.min(remainOutAmt, loanAst.getPrinBal());
                    remainOutAmt.subtractFrom(entryAmt);
                    
                    if (entryAmt.isGreaterThanZero()) {
                        LoanAstTransDetail astTransDetail = buildOutAstTransDetail(loanAst, instalTransDetail,
                                entryAmt);
                        loanAstEndDate.entryAmt = astTransDetail.getActiveLog().getPrinAmt().clone();
                        adjOutLoanAstList.add(loanAstEndDate);
                        
                        totalOutAmt.addTo(astTransDetail.getActiveLog().getPrinAmt());
                    }
                    if (remainOutAmt.isZero()) {
                        break;
                    }
                }
                VarChecker.checkEquals(totalOutAmt, instalLog.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN),
                        "分期[{0}-{1}]分期本金转出处理，机构资产总转出金额[{2}和分期转出金额[{3}]不符",
                        totalOutAmt, instalLog.getAmt().getByFieldType(DebtMoneyFieldEnum.PRIN));
            } else {
                throw new AbssqrBizException(MessageUtil.formatMsg("不支持的[{0}-{1}]分期调整事件码[{2}]",
                        instalLog.getAstNo(), instalLog.getTermNo(), instalLog.getEventCode()),
                        CommonErrorCodeEnum.INVALID_INTERNAL_MODEL, false);
            }
        }
    }
    
    private LoanAstTransDetail buildOutAstTransDetail(LoanAst loanAst, LoanInstalTransDetail instalTransDetail,
                                                      Money entryAmt) {
        LoanInstalLog instalLog = instalTransDetail.getActiveLog();
        LoanAstTransDetail astTransDetail = LoanAstTransDetail.createTransDtl(loanAst, instalTransDetail,
                instalLog.getEventCode());
        astTransDetail.processTransAmt(entryAmt);
        
        if (loanAst.getPrinBal().isZero()) {
            astTransDetail.clearLoanAst(instalLog.getAccountDate(), instalLog.getEventCode(),
                    LoanAstStatusEnum.OUT);
        }
        return astTransDetail;
    }
    
    private class LoanAstEndDate {
        
        Date    ownOrgDate;
        LoanAst loanAst;
        Money   entryAmt;
        
        public LoanAstEndDate(Date ownOrgDate, LoanAst loanAst) {
            this.ownOrgDate = ownOrgDate;
            this.loanAst = loanAst;
        }
    }
}