/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.plan;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanTransferRule;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 计划同步任务控制条件
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.plan: PlanSyncTaskCtrManager.java, v 0.1 2019-06-28 12:17 zhenxuan.luo Exp $
 */
@Component
public class PlanSyncTaskCtrManager {

    @Autowired
    private AstSyncTaskCtrRepo astSyncTaskCtrRepo;

    @Autowired
    private LoanProdRepository loanProdRepository;

    /**
     * 对应机构下的同步任务类型完成情况 大于等于 机构会计日orgDate
     * @param planBase
     * @param orgDate
     * @param taskType
     * @return
     */
    public boolean checkSyncFinishGtIsTrue(PlanBase planBase, Date orgDate, SyncTaskTypeEnum taskType) {
        Set<String> prodCodes = this.getProdCodes(planBase);
        List<LoanProduct> products = loanProdRepository.getBybssqrProdCodes(Lists.newArrayList(prodCodes));

        Set<String> managerCodes = Sets.newHashSet(products.stream().map(LoanProduct::getManagerOrgCode).collect(
            Collectors.toList()));

        for (String managerCode : managerCodes) {
            AstSyncTaskCtrLog log = astSyncTaskCtrRepo.getLastTaskLogByOrgDate(orgDate, taskType, managerCode);
            if (log == null || log.getStatus() != TaskStatusEnum.SUCCESS) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * 获取计划对应的资产机构编码
     * @param planBase
     * @return
     */
    private Set<String> getProdCodes(PlanBase planBase){
        Set<String> prodCodes = Sets.newHashSet();
        if (planBase.getPlanType() == PlanTypeEnum.SPV && planBase instanceof OrganizationEntity) {
            OrganizationEntity organizationEntity = (OrganizationEntity)planBase;
            prodCodes = Sets.newHashSet(organizationEntity.getProdCodeList());
        } else {
            // 转让规则
            PlanTransferRule planTransferRule = ((Plan)planBase).getPlanTransferRule();
            // 筛选规则
            List<AstScreenRule> astScreenRuleList = planTransferRule.getAstScreenRuleList();
        
            for (AstScreenRule astScreenRule : astScreenRuleList) {
                String prodCode = astScreenRule.getAbssqrProdCode();
                prodCodes.add(prodCode);
            }
        }
        return prodCodes;
    }
    
}