/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.report;

import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;

import java.math.BigDecimal;
import java.util.List;

/**
 * 统计结果线程存储器
 * @author hanfei
 * @version $Id: ReportItemHolder.java, v 0.1 2019-11-15 1:26 PM hanfei Exp $
 */
public class ReportItemHolder {
    
    /** 统计结果暂存项便于跨报表或不同类型统计时取值 */
    private static volatile ThreadLocal<List<PlanReportItem>> planReportItemThreadLocal = ThreadLocal.withInitial(() -> Lists.newArrayList());
    
    private ReportItemHolder() {
    }
    
    /**
     * 获取 当前线程中存储的 统计值
     * @return 如果没有统计值 则返回空对象
     */
    public static BigDecimal getPlanReportItemHolder(String planNo, ReportField field) {
        List<PlanReportItem> items = getHolder();
        BigDecimal findItemAmt = BigDecimal.ZERO;
        if (ToolUtil.isEmpty(items)){
            return findItemAmt;
        }
        for (PlanReportItem item : items){
            if (ToolUtil.equals(item.getPlanNo(), planNo)
                    && ToolUtil.equals(item.getRptItemCode(), field.getCode())){
                findItemAmt = item.getAmt();
                return findItemAmt;
            }
        }
        return findItemAmt;
    }
    
    /**
     * 追加条目
     * @param reportEntities
     */
    public static void add(List<PlanReportItem> reportEntities) {
        List<PlanReportItem> entities = getHolder();
        if (null == entities) {
            entities = Lists.newArrayList();
            planReportItemThreadLocal.set(entities);
        }
        planReportItemThreadLocal.get().addAll(reportEntities);
    }
    
    public static List<PlanReportItem> getHolder() {
        return planReportItemThreadLocal.get();
    }
    
    /**
     * 清除暂存记录
     */
    public static void clearHolder() {
        planReportItemThreadLocal.get().clear();
        planReportItemThreadLocal.set(null);
        planReportItemThreadLocal.remove();
    }
    
    
}
