/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.report.calc.impl;

import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.report.ReportItem;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanLog;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * 报表 逾期核销且核销状态
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.report.calc.impl: OvdWtfItemCalc.java, v 0.1 2019-06-27 00:59 zhenxuan.luo Exp $
 */
@Component
public class OvdWtfItemCalc extends BaseReportItemCalc {
    
    @Override
    protected ReportAstStatTypeEnum getType() {
        return ReportAstStatTypeEnum.OVD_WTF_STATUS;
    }
    
    @Override
    public List<ReportItem> calcByInstal(Loan loan, LoanInstal loanInstal, Date accountDate) {
        return buildCommonItem(loan.getLoanRisk().getOvdDays(), loan.getLoanRisk().getWtfStatus());
    }
    
    @Override
    public List<ReportItem> calcByLog(LoanLog loanLog, LoanInstalLog log) {
        return buildCommonItem(loanLog.getBfOvdDays(), loanLog.getBfWtfStatus());
    }
    
    private List<ReportItem> buildCommonItem(int ovdDays, YNEnum wtfStatus) {
        
        List<ReportItem> reportItems = Lists.newArrayList();
        // 逾期91天以上且未核销
        if ( ovdDays >= AbssqrConstants.WTF_OVD_DAYS &&wtfStatus == YNEnum.N) {
            ReportItem reportItem = new ReportItem();
            reportItem.setAstStatType(getType());
            reportItem.setAstStatVal(CommonItemCalc.COMMON_VAL);
            reportItems.add(reportItem);
        }
        
        return reportItems;
    }
}