package com.abssqr.plat;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherStarter;
import com.abssqr.plat.biz.shared.scheduler.QuartzScheduler;
import com.alibaba.druid.support.http.StatViewServlet;
import com.alibaba.druid.support.http.WebStatFilter;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.general.system.common.model.Money;
import com.general.system.common.serialize.fastjson.MoneyObjectSerializer;
import org.mybatis.spring.annotation.MapperScan;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.ComponentScans;
import org.springframework.context.annotation.FilterType;

import java.util.HashMap;
import java.util.Map;

@RefreshScope
@SpringBootApplication
@ComponentScans(value = {
        @ComponentScan(excludeFilters=@ComponentScan.Filter(
                type = FilterType.ASSIGNABLE_TYPE
                ,classes = {QuartzScheduler.class}
        ))
})
@MapperScan({"com.abssqr.plat.common.dal.mysql.auto.mapper", "com.abssqr.plat.common.dal.sync.auto.mapper"})
public class TestWebApplication{


    private static final Logger log = LoggerFactory.getLogger(TestWebApplication.class);

    public static void main(String[] args) {
        System.out.println("start TestWebApplication");
        SpringApplication.run(TestWebApplication.class, args);
        DispatcherStarter.stopAllDispatcherJob();
    }

    static {
        //Money 输出 元 万元
        SerializeConfig.getGlobalInstance().put(Money.class, new MoneyObjectSerializer());

    }

    @Bean
    public ServletRegistrationBean druidServlet() {
        log.info("init Druid Servlet Configuration ");
        ServletRegistrationBean servletRegistrationBean = new ServletRegistrationBean();
        servletRegistrationBean.setServlet(new StatViewServlet());
        servletRegistrationBean.addUrlMappings("/druid/*");
        Map<String, String> initParameters = new HashMap<String, String>();
        // 用户名
        initParameters.put("loginUsername", "admin");
        // 密码
        initParameters.put("loginPassword", "admin");
        // 禁用HTML页面上的“Reset All”功能
        initParameters.put("resetEnable", "false");
        // IP白名单 (没有配置或者为空，则允许所有访问)
        initParameters.put("allow", "");
        //initParameters.put("deny", "192.168.20.38");// IP黑名单 (存在共同时，deny优先于allow)
        servletRegistrationBean.setInitParameters(initParameters);
        return servletRegistrationBean;
    }

    @Bean
    public FilterRegistrationBean filterRegistrationBean() {
        FilterRegistrationBean filterRegistrationBean = new FilterRegistrationBean();
        filterRegistrationBean.setFilter(new WebStatFilter());
        filterRegistrationBean.addUrlPatterns("/*");
        filterRegistrationBean.addInitParameter("exclusions",
            "*.js,*.gif,*.jpg,*.png,*.css,*.ico,/druid/*,/callback/*");
        return filterRegistrationBean;
    }
    @Bean
    public EnvironmentChangeEventListener environmentChangeEventListener() {
        return new EnvironmentChangeEventListener();
    }

}