/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.test.unittest.acct;

import java.util.Date;

import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.utils.DayUtil;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;

import cn.hutool.core.date.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.Assert;
import org.testng.annotations.Test;

/**
 * 会计日服务单元测试
 *
 * @author hanfei
 * @version $Id: OrgAcctDayTest.java, v 0.1 2020-03-06 10:46 AM hanfei Exp $ 
 */
public class OrgAcctDayTest extends BaseAutoTests {
    
    @Autowired
    OrgAcctDayComponent orgAcctDayComponent;
    @Autowired
    AcctDateDAO         acctDateDAO;
    

    @XTest(relatePath = "unittest/acctday/get")
    @Test(dataProvider = "YamlDataProvider", description = "获取对应机构编码的会计日")
    public void queryDate(ICase iCase, String orgCode, String dayCheck, boolean ex) {
        try {
            Date check = DateUtil.parse(dayCheck);
            OrgCodeEnum orgCodeEnum = OrgCodeEnum.getByCode(orgCode);
            if (ex){
                // 先更新对应的会计日
                super.updateDate(check, orgCode);
            }
            // 查询会计日
            Day acctDate = orgAcctDayComponent.getAstAcctDay(orgCodeEnum);
            // 校验会计日一致
            Assert.assertTrue(DayUtil.compare(acctDate, new Day(check)) == 0);
        }catch (AbssqrBizException e){
            Assert.assertEquals(e.getMessage(), orgCode+"当前机构会计日为空");
        } finally {
            super.clearDB(iCase);
        }
    }
    
    /**
     * 场景：当前日模型同步已完成
     * 模拟dayCheck 对应的五矿标记已完成
     * 校验机构会计日从 nowDay 切换至dayNext
     * @param iCase
     * @param nowDay
     * @param data
     */
    @XTest(relatePath = "unittest/acctday/switch")
    @Test(dataProvider = "YamlDataProvider", description = "对会计日进行切换处理")
    public void acctDaySwitchTest(ICase iCase, String orgCode, String nowDay, boolean data) {
        try {
            // 当前转让日
            Day now = new Day(DateUtil.parse(nowDay));
            // 更新转让会计日
            super.updateDate(now.getStandardDate(), OrgCodeEnum.ABS.getCode());
            
            // 设置五矿dayNext 数据完成标记
            //when(wkDhccBatchRepo.checkDataReady(Mockito.anyString(),  Mockito.any())).thenReturn(data);
            // 执行会计日切换
            orgAcctDayComponent.acctDaySwitch();
            
        } catch (AbssqrBizException e){
            Assert.assertEquals(e.getMessage(), "error-["+orgCode+"]机构会计日"+nowDay+"的模型转换已完成不能进行切换操作，请人工确认");
        } finally {
            // 校验数据
            super.checkDB(iCase);
//            super.clearDB(iCase);
        }
    }
}