package com.abssqr.plat.test.unittest.capitalAstUnMatch;

import com.abssqr.plat.biz.shared.handler.CapitalAstUnMatchHandler;
import com.abssqr.plat.biz.shared.scheduler.job.capitalAssertMatch.CapitalAssertMatchJob;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstScale;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.ast.LoanBakRepo;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import org.apache.shiro.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class CapitalAstUnMatchTest extends BaseAutoTests {
    @Autowired
    CapitalAstUnMatchHandler capitalAstUnMatchHandler;
    @Autowired
    LoanBakRepo loanBakRepo;

    @Autowired
    private CapitalAssertMatchJob capitalAssertMatchJob;
    @XTest(relatePath = "unittest/capitalAstUnMatch")
    @Test(dataProvider = "YamlDataProvider", description = "资金资产解绑")
    public void exec(ICase iCase, String planNo,String allocNo, String acctDay,String layerNo,Integer allOwnCapital) {
        super.validJobCtr(capitalAssertMatchJob.getJobName());
        try {
            Date acctDate = DateTimeUtil.parseFromYMD(acctDay);
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
            capitalAstUnMatchHandler.execute(planNo,allocNo,acctDate);
            List<AstScale> astScaleList = loanBakRepo.getAstScale(acctDate, planNo);
            Map<String, List<AstScale>> capitalAstMap = astScaleList.stream()
                    .collect(Collectors.groupingBy(AstScale::getCapitalNo));
            List<AstScale> astScales = capitalAstMap.get(layerNo);
            Assert.notEmpty(astScales,"找不到投资层级"+layerNo+"对应的资产");
            Money prinBal = astScales.get(0).getPrinBal();
            Assert.isTrue(prinBal.getAmount().compareTo(new BigDecimal(allOwnCapital))<=0,"不符合预期资产规模");

        }catch (Exception e){
            e.printStackTrace();
            throw e;
        }finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
}
