/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.test.unittest.plan;

import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.param.plan.PlanDelOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanQryParam;
import com.abssqr.plat.common.facade.service.plan.PlanService;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.model.exception.BaseCommonException;
import com.general.system.common.util.VarChecker;
import com.general.system.core.common.constant.factory.ConstantFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

/**
 * 计划暂存测试
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.test.unittest.plan: planDraftTets.java, v 0.1 2019-11-14 11:26 AM xiachenxiang Exp $
 */
public class planDraftTest extends BaseAutoTests {

    @Autowired
    private AcctDateDAO acctDateDAO;

    @Autowired
    private PlanService planService;


    @XTest(relatePath = "unittest/plan/planDraftTest")
    @Test(dataProvider = "YamlDataProvider", description = "计划暂存", priority = 1)
    public void create(ICase iCase, String acctDate, PlanOprParam param) {

        ConstantFactory.me().securityLogin("administrator");

        // 更新会计日期
        try {
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());

            planService.edit(param);

        }  finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }

    @XTest(relatePath = "unittest/plan/planDraftDelTest")
    @Test(dataProvider = "YamlDataProvider", description = "计划暂存删除", priority = 2)
    public void draftDel(ICase iCase, String acctDate, PlanDelOprParam param, String msg) {

        ConstantFactory.me().securityLogin("administrator");

        // 更新会计日期
        boolean flag = false;
        try {
    
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());

            planService.deleteDraftPlan(param);

            //校验
            PlanQryParam planQryParam = new PlanQryParam();
            planQryParam.setPlanNo(param.getPlanNo());
            planQryParam.setPlanType(PlanTypeEnum.PLAN.getCode());
            planService.getPlan(planQryParam);
            
        } catch (BaseCommonException e) {
            if (e.getMessage().equals(msg)) {
                flag = true;
            }
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
        VarChecker.checkArgument(flag);

    }
}
