package com.abssqr.plat.test.unittest.workflow;

import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.param.prod.ProdEditParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderPageParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQprParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderBtn;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderVo;
import com.abssqr.plat.common.facade.service.ProdService;
import com.abssqr.plat.common.facade.service.WorkOrderService;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.bpm.domain.result.ProcessSelStartResult;
import com.general.system.common.model.BaseResult;
import com.general.system.core.common.constant.factory.ConstantFactory;
import com.general.system.tool.util.ToolUtil;
import org.junit.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.util.List;

public class ProdWorkOrderTest extends BaseAutoTests {

    @Autowired
    private WorkOrderService workOrderService;
    @Autowired
    private ProdService      prodService;

    @XTest(relatePath = "unittest/workOrder/prod/add")
    @Test(dataProvider = "YamlDataProvider", description = "发起工单审批", priority = 1)
    public void createWorkOrderProd(ICase iCase, ProdEditParam param) {
        try{
            ConstantFactory.me().securityLogin("administrator");
            // 创建产品及工单
            BaseResult<String> result = prodService.addOrUpdate(param);
    
            String checkBizNo = null;
            
            WorkOrderPageParam pageParam = new WorkOrderPageParam();
            // 我的待办
            ConstantFactory.me().securityLogin("042CF5A0-0732-40A6-9499-EA99E6AA5983");
            BasePage<WorkOrderVo> my = workOrderService.myWorkOrder(pageParam);
            for (WorkOrderVo vo : my.getDatas()){
                if (ToolUtil.equals(vo.getBusinessKey(), result.getDataObject())){
                    checkBizNo = vo.getBusinessKey();
                    break;
                }
            }
            Assert.assertNotNull(checkBizNo);
            checkBizNo = null;
            // 我的申请
            ConstantFactory.me().securityLogin("administrator");
            BasePage<WorkOrderVo> apply = workOrderService.applyWorkOrder(pageParam);
            for (WorkOrderVo vo : apply.getDatas()){
                if (ToolUtil.equals(vo.getBusinessKey(), result.getDataObject())){
                    checkBizNo = vo.getBusinessKey();
                    break;
                }
            }
            Assert.assertNotNull(checkBizNo);
            checkBizNo = null;
            // 已办事项
            ConstantFactory.me().securityLogin("administrator");
            BasePage<WorkOrderVo> his = workOrderService.hisWorkOrder(pageParam);
            for (WorkOrderVo vo : his.getDatas()){
                if (ToolUtil.equals(vo.getBusinessKey(), result.getDataObject())){
                    checkBizNo = vo.getBusinessKey();
                    break;
                }
            }
            Assert.assertNotNull(checkBizNo);
    
    
    
            WorkOrderQryParam qryParam = new WorkOrderQryParam();
            qryParam.setBusinessKey(result.getDataObject());
            // 查看已申请信息 包含 实例信息、历史记录及追踪图
            ProcessSelStartResult detail = workOrderService.detail(qryParam);
            Assert.assertNotNull(detail);
            // 查询办理轨迹
            List<WorkOrderVo> tasks = workOrderService.getWorkOrderTasks(qryParam);
            Assert.assertNotNull(tasks);
            // 详情页对应的按钮数据
            List<WorkOrderBtn> btns = workOrderService.getBtns(qryParam);
            Assert.assertNotNull(btns);
            // 获取业务类型枚举
            DataChangeBizTypeEnum typeEnum = workOrderService.getBizType(qryParam);
            Assert.assertTrue(typeEnum == DataChangeBizTypeEnum.PROD);
        }finally {
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
        
    }


    @XTest(relatePath = "unittest/workOrder/prod/qpr")
    @Test(dataProvider = "YamlDataProvider", description = "工单操作入口", priority = 1)
    public void operateProd(ICase iCase, String user, ProdEditParam param, WorkOrderQprParam qprParam) {
        try{
            ConstantFactory.me().securityLogin("administrator");
            // 创建产品及工单
            BaseResult<String> result = prodService.addOrUpdate(param);
            qprParam.setBusinessKey(result.getDataObject());
            
            // 工单操作  通过 撤销 及 拒绝
            ConstantFactory.me().securityLogin(user);
            workOrderService.operate(qprParam);
    
            String checkBizNo = null;
            // 办理人员的已完成事项
            WorkOrderPageParam pageParam = new WorkOrderPageParam();
            BasePage<WorkOrderVo> his = workOrderService.hisWorkOrder(pageParam);
            for (WorkOrderVo vo : his.getDatas()){
                if (ToolUtil.equals(vo.getBusinessKey(), result.getDataObject())){
                    checkBizNo = vo.getBusinessKey();
                }
            }
            Assert.assertNotNull(checkBizNo);
            
//            putDbCkParam("abssqrProdCode", prod.getAbssqrProdCode());
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
    
   
}
