/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.web.controller.report;

import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TimeStampTypeEnum;
import com.abssqr.plat.common.facade.model.report.MonthReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.wk.WkManagerReport;
import com.abssqr.plat.common.facade.param.base.CommonPlanQryParam;
import com.abssqr.plat.common.facade.param.reportForm.ReportFormAstParam;
import com.abssqr.plat.common.facade.param.reportForm.ReportFormMgrParam;
import com.abssqr.plat.common.facade.service.report.ReportService;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.web.fileExport.FileExportTypeEnum;
import com.abssqr.plat.web.fileExport.FileExportUtil;
import com.abssqr.plat.common.facade.utils.ListID;
import com.general.system.common.util.MessageUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *  报表中心
 *
 * @author yangcheng
 * @version ReportFormController.java, v0.12019-06-2819:14 yangchengExp$
 */
@Controller
@RequestMapping(value = "/export")
public class ReportFormController {
    
    @Autowired
    private FileExportUtil fileExportUtil;
    
    @Autowired
    private ReportService reportService;
    
    /**
     * 管理报表
     * @param param
     * @return
     * @description 查询计划报表
     */
    @RequestMapping(path = "/reportForm/plan/months", method = RequestMethod.GET)
    public void getReportFormPlan(ReportFormMgrParam param, HttpServletResponse response) throws IOException {
    
        FileExportTypeEnum fileExportType= param.getPlanType().equals(PlanTypeEnum.SPV.getCode())?
                                                   FileExportTypeEnum.MANAGER_SPV_RPT:FileExportTypeEnum.MANAGER_PLAN_RPT;
    
        WkManagerReport wkManagerReport=reportService.getReportFormPlan(param);
    
        String begin = param.getBeginTime();
        String end = param.getEndTime();
        
        if (wkManagerReport == null) {
            throw new AbssqrBizException(MessageUtil.formatMsg("计划[{0}]对应日期[{1}-{2}]的报表未生成"
                    , param.getPlanNo(),begin,end));
        }
        
        // 报表模型数据
        Map<String, Object> model = new HashMap<>();
        
        String rptDate = param.getYear() + TimeStampTypeEnum.getByCode(param.getTimeStamp()).getDesc();
        model.put("rpt", wkManagerReport);
        model.put("rptDate", rptDate);
        model.put("investId", new ListID());
        model.put("feeId", new ListID());
    
        MonthReportDateField beginDays = new MonthReportDateField();
        beginDays.parse(begin);
        MonthReportDateField endDays = new MonthReportDateField();
        endDays.parse(end);
    
        String fileKey = param.getPlanNo() + "_" + beginDays.desc() + "-" + endDays.desc();
    
        fileExportUtil.exportXls(response, fileExportType, fileKey, model);
    }
    
    /**
     * 查询资产报表
     * @param param
     * @return
     * @description 查询资产报表
     */
    @RequestMapping(path = "/reportForm/ast/days", method = RequestMethod.GET)
    public void getReportFormPlanAst(ReportFormAstParam param, HttpServletResponse response) throws IOException {
    
        FileExportTypeEnum fileExportType= param.getPlanType().equals(PlanTypeEnum.SPV.getCode())?
                                                   FileExportTypeEnum.AST_SPV_RPT:FileExportTypeEnum.AST_PLAN_RPT;
    
        List<PlanReport> reports = reportService.getReportFormAst(param);
        
        if (CollectionUtils.isEmpty(reports)) {
            throw new AbssqrBizException(MessageUtil.formatMsg("计划[{0}]对应日期[{1}-{2}]的报表未生成", param.getPlanNo(),
                    param.getBeginDate(), param.getEndDate()));
        }
        
        PlanReport firstRpt = reports.get(0);
        PlanReport lastRpt = reports.get(reports.size() - 1);
        
        // 报表模型数据
        Map<String, Object> model = new HashMap<>();
        
        model.put("reports", reports);
        
        String fileKey = param.getPlanNo() + "_" + firstRpt.getRptDate().desc() + "-" + lastRpt.getRptDate().desc();
        
        fileExportUtil.exportXls(response, fileExportType, fileKey, model);
    }
    
    
    
    /**
     * 受让池资产月末报表
     * @param param
     * @return
     * @description 查询资产报表
     */
    @RequestMapping(path = "/reportForm/ast/monthEnd", method = RequestMethod.GET)
    public void getReportFormAstMonth(CommonPlanQryParam param, HttpServletResponse response) throws IOException {
        
        List<PlanReport> reports = reportService.getReportFormMonthEnd(param);
        
        if (CollectionUtils.isEmpty(reports)) {
            throw new AbssqrBizException(MessageUtil.formatMsg("计划[{0}]月末报表没有生成", param.getPlanNo()));
        }
        
        PlanReport firstRpt = reports.get(0);
        PlanReport lastRpt = reports.get(reports.size() - 1);
        
        // 报表模型数据
        Map<String, Object> model = new HashMap<>();
        model.put("reports", reports);
        
        String fileKey = param.getPlanNo() + "_" + firstRpt.getRptDate().desc() + "-" + lastRpt.getRptDate().desc();
        
        fileExportUtil.exportXls(response, FileExportTypeEnum.AST_ME_RPT, fileKey, model);
    }
    
}